@@ -13,7 +13,8 @@ import attr
 import requests
 import tenacity
 
-from . import config, line, t
+from . import config, constants, line, t
+from . import messages as m
 
 
 @attr.s(auto_attribs=True)
@@ -23,7 +24,35 @@ class InputContent:
 
     @property
     def lines(self) -> list[line.Line]:
-        return [line.Line(lineNo, text) for lineNo, text in enumerate(self.rawLines, 1)]
+        ret = []
+        offset = 0
+        for i, text in enumerate(self.rawLines, 1):
+            lineNo = i + offset
+            # The early HTML parser can change how nodes print,
+            # so they occupy a different number of lines than they
+            # had in the source. Markdown parser needs to know
+            # the correct source lines, tho, so when this happens,
+            # the nodes will insert special PUA chars to indicate that.
+            # I can remove them here and properly adjust the line number.
+            # Current known causes of this:
+            # * line-ending -- turned into em dashes
+            # * multi-line start tags
+            # * multi-line markdown code spans;
+            #     - the text loses its newlines
+            #     - the original text goes into an attribute on the start
+            #       tag now
+            ilcc = constants.incrementLineCountChar
+            dlcc = constants.decrementLineCountChar
+            if ilcc in text:
+                offset += text.count(ilcc)
+                text = text.replace(ilcc, "")
+            if dlcc in text:
+                offset -= text.count(dlcc)
+                text = text.replace(dlcc, "")
+
+            ret.append(line.Line(lineNo, text))
+
+        return ret
 
     @property
     def content(self) -> str:
@@ -71,7 +100,8 @@ class InputSource:
 
     def directory(self) -> str:
         """Suitable for passing to subprocess(cwd=)."""
-        raise TypeError("{} instances don't have directories.".format(type(self)))
+        msg = f"{type(self)} instances don't have directories."
+        raise TypeError(msg)
 
     def relative(self, _: t.Any) -> InputSource | None:
         """Resolves relativePath relative to this InputSource.
@@ -99,12 +129,12 @@ class InputSource:
         on the subclasses that __new__ dynamically dispatches to.
         See https://stackoverflow.com/a/60731663/455535
         """
-        print(f"No member '{name}' contained in InputSource.")
+        m.warn(f"No member '{name}' contained in InputSource.")
         return ""
 
 
 class StdinInputSource(InputSource):
-    def __init__(self, sourceName: str, **kwargs: t.Any):  # pylint: disable=unused-argument
+    def __init__(self, sourceName: str, **kwargs: t.Any) -> None:  # pylint: disable=unused-argument
         assert sourceName == "-"
         self.type = "stdin"
         self.sourceName = sourceName
@@ -118,7 +148,7 @@ class StdinInputSource(InputSource):
 
 
 class UrlInputSource(InputSource):
-    def __init__(self, sourceName: str, **kwargs: t.Any):  # pylint: disable=unused-argument
+    def __init__(self, sourceName: str, **kwargs: t.Any) -> None:  # pylint: disable=unused-argument
         assert sourceName.startswith("https:")
         self.sourceName = sourceName
         self.type = "url"
@@ -148,14 +178,14 @@ class UrlInputSource(InputSource):
             # Use the response's Date header, although servers don't always set
             # this according to the last change to the file.
             date = email.utils.parsedate_to_datetime(response.headers["Date"]).date()
-        return InputContent(response.text.splitlines(True), date)
+        return InputContent([x + "\n" for x in response.text.splitlines(False)], date)
 
-    def relative(self, relativePath: str) -> UrlInputSource:
-        return UrlInputSource(urllib.parse.urljoin(self.sourceName, relativePath))
+    def relative(self, *relativePathSegs: str) -> UrlInputSource:
+        return UrlInputSource(urllib.parse.urljoin(self.sourceName, "/".join(relativePathSegs)))
 
 
 class FileInputSource(InputSource):
-    def __init__(self, sourceName: str, *, chroot: bool, chrootPath: str | None = None):
+    def __init__(self, sourceName: str, *, chroot: bool, chrootPath: str | None = None) -> None:
         self.sourceName = sourceName
         self.chrootPath = chrootPath
         self.type = "file"
@@ -182,9 +212,9 @@ class FileInputSource(InputSource):
     def directory(self) -> str:
         return os.path.dirname(os.path.abspath(self.sourceName))
 
-    def relative(self, relativePath: str) -> FileInputSource:
+    def relative(self, *relativePathSegs: str) -> FileInputSource:
         return FileInputSource(
-            os.path.join(self.directory(), relativePath),
+            os.path.join(self.directory(), *relativePathSegs),
             chroot=False,
             chrootPath=self.chrootPath,
         )
@@ -201,7 +231,7 @@ class FileInputSource(InputSource):
 
 
 class TarInputSource(InputSource):
-    def __init__(self, sourceName: str, *, tarMemberName: str = "index.bs", **_: t.Any):
+    def __init__(self, sourceName: str, *, tarMemberName: str = "index.bs", **_: t.Any) -> None:
         self.sourceName = sourceName
         self.tarMemberName = tarMemberName
         self.type = "tar"
@@ -221,7 +251,8 @@ class TarInputSource(InputSource):
                 taritem = tarFile.extractfile(self.tarMemberName)
                 if taritem is None:
                     raise FileNotFoundError(
-                        errno.ENOENT, f"{self.tarMemberName} is in the tar file, but isn't a file itself."
+                        errno.ENOENT,
+                        f"{self.tarMemberName} is in the tar file, but isn't a file itself.",
                     )
                 with taritem as f:
                     # Decode the `bytes` to a `str`. (extractfile can't read as text.)
@@ -235,12 +266,13 @@ class TarInputSource(InputSource):
 
     def directory(self) -> str:
         # It would be possible to produce a file listing. But not a meaningful directory path.
-        raise TypeError("{} instances don't have directories.".format(type(self)))
+        msg = f"{type(self)} instances don't have directories."
+        raise TypeError(msg)
 
-    def relative(self, relativePath: str) -> TarInputSource:
+    def relative(self, *relativePathSegs: str) -> TarInputSource:
         """Returns an InputSource relative to this file. Since a TarInputSource is always inside the
         tar file, any relative InputSource is also inside the tar file."""
-        memberPath = os.path.join(os.path.dirname(self.tarMemberName), relativePath)
+        memberPath = os.path.join(os.path.dirname(self.tarMemberName), *relativePathSegs)
         return TarInputSource(self.sourceName, tarMemberName=memberPath)
 
     def cheaplyExists(self, relativePath: str) -> bool | None:
