@@ -4,19 +4,23 @@ from __future__ import annotations
 import glob
 import json
 import os
+import re
 import sys
-from collections import defaultdict, OrderedDict
+import time
+from collections import OrderedDict, defaultdict
 from datetime import datetime
 
 from . import (
+    InputSource,
     biblio,
     boilerplate,
     caniuse,
+    cddl,
     conditional,
-    config,
     constants,
     datablocks,
     dfns,
+    doctypes,
     extensions,
     fingerprinting,
     h,
@@ -25,21 +29,21 @@ from . import (
     idl,
     includes,
     inlineTags,
-    InputSource,
     language,
     lint,
     markdown,
     mdn,
-    messages as m,
     metadata,
     refs,
     retrieve,
     shorthands,
+    stylescript,
     t,
-    testsuite,
-    unsortedJunk as u,
     wpt,
 )
+from . import line as l
+from . import messages as m
+from . import unsortedJunk as u
 
 if t.TYPE_CHECKING:
     import widlparser
@@ -54,15 +58,18 @@ class Spec:
         lineNumbers: bool = False,
         fileRequester: retrieve.DataFileRequester | None = None,
         testing: bool = False,
-    ):
+    ) -> None:
         catchArgparseBug(inputFilename)
         self.valid: bool = False
         self.lineNumbers: bool = lineNumbers
+        if lineNumbers:
+            # line-numbers are too hacky, so force this to be a dry run
+            constants.dryRun = True
         if inputFilename is None:
             inputFilename = findImplicitInputFile()
         if inputFilename is None:  # still
             m.die(
-                "No input file specified, and no *.bs or *.src.html files found in current directory.\nPlease specify an input file, or use - to pipe from STDIN."
+                "No input file specified, and no *.bs or *.src.html files found in current directory.\nPlease specify an input file, or use - to pipe from STDIN.",
             )
             return
         self.inputSource: InputSource.InputSource = InputSource.inputFromName(inputFilename, chroot=constants.chroot)
@@ -76,7 +83,7 @@ class Spec:
         else:
             self.dataFile = fileRequester
 
-        self.nodes: list[h.ParserNode] = []
+        self.lines: list[l.Line] = []
         self.valid = self.initializeState()
 
     def initializeState(self) -> bool:
@@ -103,25 +110,21 @@ class Spec:
 
         self.widl: widlparser.Parser = idl.getParser()
 
-        self.testSuites: dict[str, testsuite.TestSuite] = fetchTestSuites(self.dataFile)
         self.languages: dict[str, language.Language] = fetchLanguages(self.dataFile)
+        self.doctypes: doctypes.DoctypeManager = fetchDoctypes(self.dataFile)
 
-        self.extraStyles: t.DefaultDict[str, str] = defaultdict(str)
-        self.extraStyles["style-colors"] = getModuleFile("Spec-colors.css")
-        self.extraStyles["style-darkmode"] = getModuleFile("Spec-darkmode.css")
-        self.extraStyles["style-md-lists"] = getModuleFile("Spec-mdlists.css")
-        self.extraStyles["style-autolinks"] = getModuleFile("Spec-autolinks.css")
-        self.extraStyles["style-selflinks"] = getModuleFile("Spec-selflinks.css")
-        self.extraStyles["style-counters"] = getModuleFile("Spec-counters.css")
-        self.extraStyles["style-issues"] = getModuleFile("Spec-issues.css")
-        self.extraScripts: t.DefaultDict[str, str] = defaultdict(str)
+        self.extraJC = stylescript.JCManager()
+        self.extraJC.addColors()
+        self.extraJC.addMdLists()
+        self.extraJC.addAutolinks()
+        self.extraJC.addSelflinks()
+        self.extraJC.addCounters()
+        self.extraJC.addIssues()
 
         try:
-            inputContent = self.inputSource.read()
-            text = u.stripBOM(inputContent.content)
-            self.nodes = list(h.initialDocumentParse(text))
-            if inputContent.date is not None:
-                self.mdBaseline.addParsedData("Date", inputContent.date)
+            self.inputContent = self.inputSource.read()
+            if self.inputContent.date is not None:
+                self.mdBaseline.addParsedData("Date", self.inputContent.date)
         except FileNotFoundError:
             m.die(f"Couldn't find the input file at the specified location '{self.inputSource}'.")
             return False
@@ -131,6 +134,70 @@ class Spec:
 
         return True
 
+    def initMetadata(self, inputContent: InputSource.InputContent) -> None:
+        # mdDefault is already set up
+        # and cli() inited the mdCommandLine
+
+        # Get the md from the doc itself
+        # TODO: currently I just leave the node in,
+        #       I should do something about that
+        # TODO: Pure textual hack, no way to, say, put a block
+        #       in a markdown code span or an <xmp> to show off.
+        _, self.mdDocument = metadata.parse(lines=inputContent.lines)
+
+        # Combine the data so far, and compute the doctype
+        # (the other md sources need the doctype in order to be found)
+        self.md = metadata.join(self.mdBaseline, self.mdDocument, self.mdCommandLine)
+        rawDoctype = (self.md.rawOrg, self.md.rawGroup, self.md.rawStatus)
+        self.doctype = self.doctypes.getDoctype(self.md.rawOrg, self.md.rawGroup, self.md.rawStatus)
+
+        self.mdDefaults = metadata.fromJson(
+            data=retrieve.retrieveBoilerplateFile(self, "defaults"),
+            source="defaults",
+        )
+        self.md = metadata.join(self.mdBaseline, self.mdDefaults, self.mdDocument, self.mdCommandLine)
+        if rawDoctype != (self.md.rawOrg, self.md.rawGroup, self.md.rawStatus):
+            # recompute doctype
+            self.doctype = self.doctypes.getDoctype(self.md.rawOrg, self.md.rawGroup, self.md.rawStatus)
+
+        # Using all of that, load up the text macros so I can sub them into the computed-metadata file.
+        self.md.fillTextMacros(self.macros, doc=self)
+        jsonEscapedMacros = {k: json.dumps(v)[1:-1] for k, v in self.macros.items()}
+
+        computedMdText = h.replaceMacrosTextly(
+            retrieve.retrieveBoilerplateFile(self, "computed-metadata"),
+            macros=jsonEscapedMacros,
+            context="? of computed-metadata.include",
+        )
+        self.mdOverridingDefaults = metadata.fromJson(data=computedMdText, source="computed-metadata")
+        # And create the final, complete md combo
+        self.md = metadata.join(
+            self.mdBaseline,
+            self.mdDefaults,
+            self.mdOverridingDefaults,
+            self.mdDocument,
+            self.mdCommandLine,
+        )
+        # Finally, compute the "implicit" things.
+        self.md.computeImplicitMetadata(doc=self)
+        # And compute macros again, in case the preceding steps changed them.
+        self.md.fillTextMacros(self.macros, doc=self)
+
+        self.md.validate(doc=self)
+        m.retroactivelyCheckErrorLevel()
+
+    def earlyParse(self, inputContent: InputSource.InputContent) -> list[l.Line]:
+        text = FIXMEreplaceMarkdownBlockquotes(inputContent.content)
+        nodes = h.initialDocumentParse(text, h.ParseConfig.fromSpec(self))
+        if self.debug:
+            h.debugNodes(nodes)
+        text = h.strFromNodes(nodes, withIlcc=True)
+        inputContent.rawLines = [x + "\n" for x in text.split("\n")]
+        return inputContent.lines
+
+    def checkValidity(self) -> bool:
+        return True
+
     def recordDependencies(self, *inputSources: InputSource.InputSource) -> None:
         self.transitiveDependencies.update(inputSources)
 
@@ -141,58 +208,40 @@ class Spec:
         return self
 
     def assembleDocument(self) -> Spec:
+        self.initMetadata(self.inputContent)
         self.recordDependencies(self.inputSource)
-        # Extract and process metadata
-        self.nodes, self.mdDocument = metadata.parse(self.nodes)
-        # First load the metadata sources from 'local' data
-        self.md = metadata.join(self.mdBaseline, self.mdDocument, self.mdCommandLine)
-        # Using that to determine the Group and Status, load the correct defaults.include boilerplate
-        self.mdDefaults = metadata.fromJson(
-            data=retrieve.retrieveBoilerplateFile(self, "defaults", error=True),
-            source="defaults",
-        )
-        self.md = metadata.join(self.mdBaseline, self.mdDefaults, self.mdDocument, self.mdCommandLine)
-        # Using all of that, load up the text macros so I can sub them into the computed-metadata file.
-        self.md.fillTextMacros(self.macros, doc=self)
-        jsonEscapedMacros = {k: json.dumps(v)[1:-1] for k, v in self.macros.items()}
-        computedMdText = h.replaceMacros(
-            retrieve.retrieveBoilerplateFile(self, "computed-metadata", error=True),
-            macros=jsonEscapedMacros,
-        )
-        self.mdOverridingDefaults = metadata.fromJson(data=computedMdText, source="computed-metadata")
-        self.md = metadata.join(
-            self.mdBaseline,
-            self.mdDefaults,
-            self.mdOverridingDefaults,
-            self.mdDocument,
-            self.mdCommandLine,
-        )
-        # Finally, compute the "implicit" things.
-        self.md.computeImplicitMetadata(doc=self)
-        # And compute macros again, in case the preceding steps changed them.
-        self.md.fillTextMacros(self.macros, doc=self)
-        self.md.validate()
+        self.lines = self.earlyParse(self.inputContent)
+
+        # Remove the metadata
+        # FIXME: This should be done the first time I parse metadata.
+        self.lines, _ = metadata.parse(lines=self.lines)
         extensions.load(self)
 
         # Initialize things
-        self.refs.initializeRefs(doc=self)
-        self.refs.initializeBiblio()
+        self.refs.initializeRefs(doc=self, datablocks=datablocks)
+        self.refs.initializeBiblio(doc=self)
+
+        if "mixed-indents" in self.md.complainAbout:
+            if self.md.indentInfo and self.md.indentInfo.char:
+                checkForMixedIndents(self.lines, self.md.indentInfo)
+
         # Deal with further <pre> blocks, and markdown
-        self.nodes = datablocks.transformDataBlocks(self, self.nodes)
+        self.lines = datablocks.transformDataBlocks(self, self.lines)
+
         markdownFeatures: set[str] = {"headings"}
-        lines = markdown.parse(
-            h.linesFromNodes(self.nodes),
+        self.lines = markdown.parse(
+            self.lines,
             self.md.indent,
             opaqueElements=self.md.opaqueElements,
             blockElements=self.md.blockElements,
             features=markdownFeatures,
         )
-        self.html = "\n".join(lines)
 
-        self.refs.setSpecData(self.md)
+        self.refs.setSpecData(self)
 
+        # Convert to a single string of html now, for convenience.
+        self.html = "".join(x.text for x in self.lines)
         boilerplate.addHeaderFooter(self)
-        self.html = self.fixText(self.html)
 
         # Build the document
         self.document = h.parseDocument(self.html)
@@ -230,13 +279,14 @@ class Spec:
         boilerplate.removeUnwantedBoilerplate(self)
         shorthands.run(self)
         inlineTags.processTags(self)
-        u.canonicalizeShortcuts(self)
+        u.canonicalizeShortcuts(self.body)
         u.addImplicitAlgorithms(self)
         u.fixManualDefTables(self)
         headings.processHeadings(self)
         u.checkVarHygiene(self)
         u.processIssuesAndExamples(self)
         idl.markupIDL(self)
+        cddl.markupCDDL(self)
         u.inlineRemoteIssues(self)
         u.addImageSize(self)
 
@@ -244,6 +294,7 @@ class Spec:
         u.processBiblioLinks(self)
         u.processDfns(self)
         u.processIDL(self)
+        u.processCDDL(self)
         dfns.annotateDfns(self)
         u.formatArgumentdefTables(self)
         u.formatElementdefTables(self)
@@ -256,6 +307,7 @@ class Spec:
         boilerplate.addReferencesSection(self)
         boilerplate.addPropertyIndex(self)
         boilerplate.addIDLSection(self)
+        boilerplate.addCDDLSection(self)
         boilerplate.addIssuesSection(self)
         boilerplate.addCustomBoilerplate(self)
         headings.processHeadings(self, "all")  # again
@@ -265,12 +317,11 @@ class Spec:
         u.processAutolinks(self)
         boilerplate.removeUnwantedBoilerplate(self)
         # Add MDN panels after all IDs/anchors have been added
-        mdnPanels = mdn.addMdnPanels(self)
-        ciuPanels = caniuse.addCanIUsePanels(self)
-        if mdnPanels or ciuPanels:
-            self.extraScripts["position-annos"] = getModuleFile("Spec-position-annos.js")
+        mdn.addMdnPanels(self)
+        caniuse.addCanIUsePanels(self)
         highlight.addSyntaxHighlighting(self)
         boilerplate.addBikeshedBoilerplate(self)
+        boilerplate.addDarkmodeIndicators(self)
         fingerprinting.addTrackingVector(self)
         u.fixIntraDocumentReferences(self)
         u.fixInterDocumentReferences(self)
@@ -332,6 +383,8 @@ class Spec:
         return outputFilename
 
     def finish(self, outputFilename: str | None = None, newline: str | None = None) -> None:
+        # Check the errors one more time.
+        m.retroactivelyCheckErrorLevel(timing="late")
         catchArgparseBug(outputFilename)
         self.printResultMessage()
         outputFilename = self.fixMissingOutputFilename(outputFilename)
@@ -348,9 +401,9 @@ class Spec:
 
     def printResultMessage(self) -> None:
         # If I reach this point, I've succeeded, but maybe with reservations.
-        fatals = m.messageCounts["fatal"]
-        links = m.messageCounts["linkerror"]
-        warnings = m.messageCounts["warning"]
+        fatals = m.state.categoryCounts["fatal"]
+        links = m.state.categoryCounts["link-error"]
+        warnings = m.state.categoryCounts["warning"]
         if self.lineNumbers:
             m.warn("Because --line-numbers was used, no output was saved.")
         if fatals:
@@ -364,8 +417,6 @@ class Spec:
             return
 
     def watch(self, outputFilename: str | None, port: int | None = None, localhost: bool = False) -> None:
-        import time
-
         outputFilename = self.fixMissingOutputFilename(outputFilename)
         if self.inputSource.mtime() is None:
             m.die(f"Watch mode doesn't support {self.inputSource}")
@@ -376,9 +427,9 @@ class Spec:
 
         if port:
             # Serve the folder on an HTTP server
-            import http.server
-            import socketserver
-            import threading
+            import http.server  # noqa: PLC0415
+            import socketserver  # noqa: PLC0415
+            import threading  # noqa: PLC0415
 
             class SilentServer(http.server.SimpleHTTPRequestHandler):
                 def log_message(self, format: t.Any, *args: t.Any) -> None:
@@ -387,7 +438,7 @@ class Spec:
             socketserver.TCPServer.allow_reuse_address = True
             server = socketserver.TCPServer(("localhost" if localhost else "", port), SilentServer)
 
-            print(f"Serving at port {port}")
+            print(f"Serving at http://localhost:{port}/")  # noqa: T201
             thread = threading.Thread(target=server.serve_forever)
             thread.daemon = True
             thread.start()
@@ -407,7 +458,7 @@ class Spec:
                     # stops existing, and it's fine to rebuild if an mtime
                     # somehow gets older.
                     if any(input.mtime() != lastModified for input, lastModified in lastInputModified.items()):
-                        m.resetSeenMessages()
+                        m.state = m.state.replace()
                         m.p("\nSource file modified. Rebuilding...")
                         self.initializeState()
                         self.mdCommandLine = mdCommandLine
@@ -425,36 +476,20 @@ class Spec:
         except Exception as e:
             m.die(f"Something went wrong while watching the file:\n{e}")
 
-    def fixText(self, text: str, moreMacros: dict[str, str] | None = None) -> str:
-        # Do several textual replacements that need to happen *before* the document is parsed as html.
-
-        # If markdown shorthands are on, remove all `foo`s while processing,
-        # so their contents don't accidentally trigger other stuff.
-        # Also handle markdown escapes.
-        if moreMacros is None:
-            moreMacros = {}
-
-        macros = dict(self.macros, **moreMacros)
-        text = h.replaceMacros(text, macros)
-        text = h.fixTypography(text)
-        if "css" in self.md.markupShorthands:
-            text = h.replaceAwkwardCSSShorthands(text)
-        return text
-
     def printTargets(self) -> None:
         m.p("Exported terms:")
         for el in h.findAll("[data-export]", self):
-            for term in config.linkTextsFromElement(el):
+            for term in h.linkTextsFromElement(el):
                 m.p("  " + term)
         m.p("Unexported terms:")
         for el in h.findAll("[data-noexport]", self):
-            for term in config.linkTextsFromElement(el):
+            for term in h.linkTextsFromElement(el):
                 m.p("  " + term)
 
     def isOpaqueElement(self, el: t.ElementT) -> bool:
         if el.tag in self.md.opaqueElements:
             return True
-        if el.get("bs-opaque") is not None:
+        if el.get("data-opaque") is not None or el.get("bs-opaque") is not None:  # noqa: SIM103
             return True
         return False
 
@@ -462,7 +497,7 @@ class Spec:
 def printDone() -> None:
     contents = f"Finished at {datetime.now().strftime('%H:%M:%S %b-%d-%Y')}"
     contentLen = len(contents) + 2
-    if not constants.asciiOnly:
+    if not m.state.asciiOnly:
         m.p(f"╭{'─'*contentLen}╮")
         m.p(f"│ {contents} │")
         m.p(f"╰{'─'*contentLen}╯")
@@ -513,16 +548,12 @@ def catchArgparseBug(string: str | None) -> bool:
 
     if isinstance(string, str) and string.startswith("--") and "=" in string:
         m.die(
-            "You're hitting a bug with Python's argparse library. Please specify both the input and output filenames manually, and move all command-line flags with spaces in their values to after those arguments.\nSee <https://speced.github.io/bikeshed/#md-issues> for details."
+            "You're hitting a bug with Python's argparse library. Please specify both the input and output filenames manually, and move all command-line flags with spaces in their values to after those arguments.\nSee <https://speced.github.io/bikeshed/#md-issues> for details.",
         )
         return False
     return True
 
 
-def fetchTestSuites(dataFile: retrieve.DataFileRequester) -> dict[str, testsuite.TestSuite]:
-    return {k: testsuite.TestSuite(**v) for k, v in json.loads(dataFile.fetch("test-suites.json", str=True)).items()}
-
-
 def fetchLanguages(dataFile: retrieve.DataFileRequester) -> dict[str, language.Language]:
     return {
         k: language.Language(v["name"], v["native-name"])
@@ -530,15 +561,60 @@ def fetchLanguages(dataFile: retrieve.DataFileRequester) -> dict[str, language.L
     }
 
 
+def fetchDoctypes(dataFile: retrieve.DataFileRequester) -> doctypes.DoctypeManager:
+    return doctypes.DoctypeManager.fromKdlStr(dataFile.fetch("boilerplate", "doctypes.kdl", str=True))
+
+
 def addDomintroStyles(doc: Spec) -> None:
     # Adds common WHATWG styles for domintro blocks.
 
     if h.find(".domintro", doc) is None:
         return
 
-    doc.extraStyles["styles-domintro"] = getModuleFile("Spec-domintro.css")
+    doc.extraJC.addDomintro()
 
 
-def getModuleFile(filename: str) -> str:
-    with open(config.scriptPath(filename), "r", encoding="utf-8") as fh:
-        return fh.read()
+def checkForMixedIndents(lines: t.Sequence[l.Line], info: metadata.IndentInfo) -> None:
+    badIndentChar = " " if info.char == "\t" else "\t"
+    for line in lines:
+        if not line.text:
+            continue
+        if line.text.startswith(badIndentChar):
+            if info.char == " ":
+                m.lint("Line starts with tabs, but document is space-indented", lineNum=line.i)
+            else:
+                m.lint("Line starts with spaces, but document is tab-indented", lineNum=line.i)
+        if re.match(r"(\t+ +\t)|( +\t)", line.text):
+            m.lint(f"Line {line.i}'s indent contains tabs after spaces.")
+
+
+def FIXMEreplaceMarkdownBlockquotes(text: str) -> str:
+    # Temporary hack to make the early HTML pass not be broken
+    # by Markdown blockquotes.
+    # * finds sequences of lines that share a ws + angle bracket prefix
+    # * replaces first such > with a blockquote-open PUA
+    # * replaces subsequent > with a space
+    # * adds a blockquote-close PUA to the end of last line
+    # * the HTML parser recognizes those PUAs and emits the correct start/end tags.
+
+    lines = text.split("\n")
+    i = 0
+    while True:
+        if i >= len(lines):
+            break
+        match = re.match(r"\s*>\s?", lines[i])
+        if not match:
+            i += 1
+            continue
+        if i + 1 < len(lines) and re.match(r"\s*>\s?", lines[i + 1]):
+            lines[i] = constants.bqStart + lines[i][len(match[0]) :]
+            i += 1
+            while i < len(lines) and re.match(r"\s*>\s?", lines[i]):
+                match = re.match(r"\s*>\s?", lines[i])
+                assert match is not None
+                lines[i] = lines[i][len(match[0]) :]
+                i += 1
+            lines[i - 1] += constants.bqEnd
+        else:
+            i += 1
+    return "\n".join(lines)
