@@ -2,10 +2,11 @@ from __future__ import annotations
 
 import abc
 import dataclasses
-import re
+import json
 from collections import defaultdict
 
-from . import constants, h, messages as m, t
+from . import constants, h, t
+from . import messages as m
 
 
 @dataclasses.dataclass
@@ -30,13 +31,14 @@ class BiblioEntry(metaclass=abc.ABCMeta):
         elif self.preferredStatus == constants.refStatus.current:
             return t.cast(str, self.currentURL or self.snapshotURL)
         else:
-            raise ValueError(f"Invalid preferredStatus value: {self.preferredStatus}")
+            msg = f"Invalid preferredStatus value: {self.preferredStatus}"
+            raise ValueError(msg)
 
-    def toHTML(self) -> t.NodesT:
-        ...
+    @abc.abstractmethod
+    def toHTML(self) -> t.NodesT: ...
 
-    def valid(self) -> bool:
-        ...
+    @abc.abstractmethod
+    def valid(self) -> bool: ...
 
     def strip(self) -> BiblioEntry:
         self.linkText = self.linkText.strip()
@@ -123,7 +125,7 @@ class SpecBiblioEntry(BiblioEntry):
     for when we don't have "real" bibliography data for a reference.
     """
 
-    def __init__(self, spec: dict[str, str], preferredStatus: str | None = None, order: int = 0):
+    def __init__(self, spec: dict[str, str], preferredStatus: str | None = None, order: int = 0) -> None:
         super().__init__(
             linkText=spec["vshortname"],
             title=spec["description"],
@@ -149,7 +151,7 @@ class StringBiblioEntry(BiblioEntry):
 
     data: str
 
-    def __init__(self, linkText: str, data: str, order: int = 0):
+    def __init__(self, linkText: str, data: str, order: int = 0) -> None:
         doc = h.parseDocument(data)
         titleEl = h.find("cite", doc)
         if titleEl is not None:
@@ -178,7 +180,7 @@ class AliasBiblioEntry(BiblioEntry):
 
     aliasOf: str
 
-    def __init__(self, linkText: str, aliasOf: str, order: int = 0):
+    def __init__(self, linkText: str, aliasOf: str, order: int = 0) -> None:
         super().__init__(linkText=linkText, order=order)
         self.aliasOf = aliasOf.strip()
 
@@ -189,63 +191,6 @@ class AliasBiblioEntry(BiblioEntry):
         return [h.E.small({}, f"(alias of {self.aliasOf})")]
 
 
-def processReferBiblioFile(lines: t.Sequence[str], storage: t.BiblioStorageT, order: int) -> t.BiblioStorageT:
-    singularReferCodes = {
-        "U": "snapshotURL",
-        "T": "title",
-        "D": "date",
-        "S": "status",
-        "L": "linkText",
-        "O": "other",
-    }
-    pluralReferCodes = {
-        "A": "authors",
-        "Q": "authors",
-    }
-    unusedReferCodes = set("BCIJNPRVX")
-
-    for group in groupsFromReferFile(lines):
-        biblio: dict[str, t.Any] = {"order": order}
-        for line in group:
-            match = re.match(r"%(\w)\s+(.*)", line)
-            if match:
-                letter, value = match.groups()
-            else:
-                m.die(f"Biblio line in unexpected format:\n{line}")
-                continue
-
-            if letter in singularReferCodes:
-                biblio[singularReferCodes[letter]] = value
-            elif letter in pluralReferCodes:
-                biblio.setdefault(pluralReferCodes[letter], []).append(value)
-            elif letter in unusedReferCodes:
-                pass
-            else:
-                m.die(f"Unknown line type {letter}:\n{line}")
-                continue
-        storage[biblio["linkText"].lower()].append(NormalBiblioEntry(**biblio))
-    return storage
-
-
-def groupsFromReferFile(lines: t.Sequence[str]) -> t.Generator[list[str], None, None]:
-    group: list[str] = []
-    for line in lines:
-        line = line.strip()
-
-        if line == "":
-            if group:
-                yield group
-                group = []
-        elif line.startswith("#") or line.startswith("%#"):
-            # Comment
-            continue
-        else:
-            group.append(line)
-    # yield the final group, if there wasn't a trailing blank line
-    if group:
-        yield group
-
-
 def processSpecrefBiblioFile(text: str, storage: t.BiblioStorageT, order: int) -> t.BiblioStorageT:
     r"""
     A SpecRef file is a JSON object, where keys are ids
@@ -279,8 +224,6 @@ def processSpecrefBiblioFile(text: str, storage: t.BiblioStorageT, order: int) -
 
     <wg>: {*url:<url>, *shortname:<string>}
     """
-    import json
-
     try:
         datas = json.loads(text)
     except Exception as e:
@@ -419,7 +362,7 @@ def findCloseBiblios(biblioKeys: t.Sequence[str], target: str, n: int = 5) -> li
     names: list[tuple[str, int]] = []
     superStrings: list[str] = []
 
-    for name in biblioKeys:
+    for name in sorted(biblioKeys):
         if target in name:
             superStrings.append(name)
         else:
