@@ -8,16 +8,18 @@ import subprocess
 from collections import OrderedDict, defaultdict
 from datetime import datetime
 
-from . import conditional, config, dfnpanels, h, messages as m, refs as r, retrieve, t
-from .translate import _
+from . import conditional, config, dfnpanels, h, retrieve, t
+from . import messages as m
+from . import refs as r
+from .translate import _t
 
 if t.TYPE_CHECKING:
-    MetadataT: t.TypeAlias = t.Mapping[str, t.Sequence[MetadataValueT]]
     MetadataValueT: t.TypeAlias = str | t.NodesT | None
+    MetadataT: t.TypeAlias = t.Mapping[str, t.Sequence[MetadataValueT]]
 
 
-def boilerplateFromHtml(doc: t.SpecT, htmlString: str) -> t.NodesT:
-    htmlString = doc.fixText(htmlString)
+def boilerplateFromHtml(doc: t.SpecT, htmlString: str, context: str) -> t.NodesT:
+    htmlString = h.parseText(htmlString, h.ParseConfig.fromSpec(doc, context=context))
     bp = h.E.div({}, h.parseHTML(htmlString))
     conditional.processConditionals(doc, bp)
     return h.childNodes(bp, clear=True)
@@ -27,7 +29,7 @@ def loadBoilerplate(doc: t.SpecT, filename: str, bpname: str | None = None) -> N
     if bpname is None:
         bpname = filename
     html = retrieve.retrieveBoilerplateFile(doc, filename)
-    el = boilerplateFromHtml(doc, html)
+    el = boilerplateFromHtml(doc, html, context=f"{filename} boilerplate")
     fillWith(bpname, el, doc=doc)
 
 
@@ -46,7 +48,7 @@ def addBikeshedVersion(doc: t.SpecT) -> None:
         if "bikeshed" not in origin:
             # In a repo, but not bikeshed's;
             # probably pip-installed into an environment's repo or something.
-            raise Exception()
+            raise Exception
         # Otherwise, success, this is a -e install,
         # so we're in Bikeshed's repo.
         bikeshedVersion = (
@@ -112,14 +114,20 @@ def addSpecVersion(doc: t.SpecT) -> None:
         except subprocess.CalledProcessError:
             pass
     if revision:
-        h.appendChild(doc.head, h.E.meta({"name": "document-revision", "content": revision}))
+        h.appendChild(doc.head, h.E.meta({"name": "revision", "content": revision}))
 
 
 def addHeaderFooter(doc: t.SpecT) -> None:
     header = retrieve.retrieveBoilerplateFile(doc, "header") if "header" in doc.md.boilerplate else ""
     footer = retrieve.retrieveBoilerplateFile(doc, "footer") if "footer" in doc.md.boilerplate else ""
 
-    doc.html = "\n".join([header, doc.html, footer])
+    doc.html = "\n".join(
+        [
+            h.parseText(header, h.ParseConfig.fromSpec(doc, context="header.include")),
+            doc.html,
+            h.parseText(footer, h.ParseConfig.fromSpec(doc, context="footer.include")),
+        ],
+    )
 
 
 def fillWith(tag: str, newElements: t.NodesT, doc: t.SpecT) -> None:
@@ -149,7 +157,7 @@ def getFillContainer(tag: str, doc: t.SpecT, default: bool = False) -> t.Element
 
     # Otherwise, append to the end of the document,
     # unless you're in the byos group
-    if doc.md.group == "byos":
+    if doc.doctype.group.name == "BYOS":
         return None
     if default:
         return doc.body
@@ -184,24 +192,11 @@ def addExpiryNotice(doc: t.SpecT) -> None:
         boilerplate = "warning-expired"
     else:
         boilerplate = "warning-expires"
-        doc.extraScripts["script-expires"] = expiryScript
+        doc.extraJC.addExpires()
     loadBoilerplate(doc, boilerplate, "warning")
     h.addClass(doc, doc.body, boilerplate)
 
 
-expiryScript = """
-const warning = document.querySelector('#expiry-notice');
-const expiresOn = warning.dataset.expires;
-const today = new Date().toISOString();
-if(expires < today) {
-    warning.setAttribute("open", "");
-    for(const swap of warning.querySelectorAll("[data-after-expiry]")) {
-        swap.textContent = swap.dataset.afterExpiry;
-    }
-}
-"""
-
-
 def addObsoletionNotice(doc: t.SpecT) -> None:
     if doc.md.warning:
         loadBoilerplate(doc, doc.md.warning[0], "warning")
@@ -212,14 +207,15 @@ def addAtRisk(doc: t.SpecT) -> None:
         return
     html = "<p>The following features are at-risk, and may be dropped during the CR period:\n<ul>"
     for feature in doc.md.atRisk:
-        html += "<li>" + doc.fixText(feature)
+        html += "<li>" + h.parseText(feature, h.ParseConfig.fromSpec(doc, context="At Risk metadata"))
     html += (
         "</ul><p>“At-risk” is a W3C Process term-of-art, and does not necessarily imply that the feature is in danger of being dropped or delayed. "
-        "It means that the WG believes the feature may have difficulty being interoperably implemented in a timely manner, "
-        "and marking it as such allows the WG to drop the feature if necessary when transitioning to the Proposed Rec stage, "
-        "without having to publish a new Candidate Rec without the feature first."
+        + "It means that the WG believes the feature may have difficulty being interoperably implemented in a timely manner, "
+        + "and marking it as such allows the WG to drop the feature if necessary when transitioning to the Proposed Rec stage, "
+        + "without having to publish a new Candidate Rec without the feature first."
     )
-    fillWith("at-risk", h.parseHTML(html), doc=doc)
+    frag = h.parseHTML(html)
+    fillWith("at-risk", frag, doc=doc)
 
 
 def addStyles(doc: t.SpecT) -> None:
@@ -244,40 +240,22 @@ def removeUnwantedBoilerplate(doc: t.SpecT) -> None:
 
 
 def w3cStylesheetInUse(doc: t.SpecT) -> bool:
-    return doc.md.prepTR or doc.md.status in config.snapshotStatuses
-
-
-def keyFromStyles(kv: tuple[str, str]) -> tuple[int, str]:
-    k = kv[0]
-    if k == "style-darkmode":
-        prio = 2
-    else:
-        prio = 1
-
-    return (prio, k)
+    return doc.md.prepTR or doc.doctype.group.name == "W3C"
 
 
 def addBikeshedBoilerplate(doc: t.SpecT) -> None:
-    w3cStylesheet = w3cStylesheetInUse(doc)
-    for k, v in sorted(doc.extraStyles.items(), key=keyFromStyles):
-        if k not in doc.md.boilerplate:
-            continue
-        if w3cStylesheet and k in ["style-colors", "style-darkmode"]:
-            # These are handled by the /TR stylesheet, so don't output them
-            continue
-        container = getFillContainer(k, doc)
+    for style in doc.extraJC.getStyles(doc.md.boilerplate):
+        container = getFillContainer("style-" + style.name, doc)
         if container is None:
             container = getFillContainer("bs-styles", doc, default=True)
         if container is not None:
-            h.appendChild(container, h.E.style(f"/* {k} */\n{v}"))
-    for k, v in sorted(doc.extraScripts.items()):
-        if k not in doc.md.boilerplate:
-            continue
-        container = getFillContainer(k, doc)
+            h.appendChild(container, style.toElement(darkMode=doc.md.darkMode))
+    for script in doc.extraJC.getScripts(doc.md.boilerplate):
+        container = getFillContainer("script-" + script.name, doc)
         if container is None:
             container = getFillContainer("bs-scripts", doc, default=True)
         if container is not None:
-            h.appendChild(container, h.E.script(f"/* {k} */\n{v}"))
+            h.appendChild(container, script.toElement())
 
 
 def addIndexSection(doc: t.SpecT) -> None:
@@ -289,7 +267,7 @@ def addIndexSection(doc: t.SpecT) -> None:
     container = getFillContainer("index", doc=doc, default=True)
     if container is None:
         return
-    h.appendChild(container, h.E.h2({"class": "no-num no-ref", "id": h.safeID(doc, "index")}, _("Index")))
+    h.appendChild(container, h.E.h2({"class": "no-num no-ref", "id": h.safeID(doc, "index")}, _t("Index")))
 
     if hasLocalDfns:
         addIndexOfLocallyDefinedTerms(doc, container)
@@ -303,7 +281,7 @@ def addIndexOfLocallyDefinedTerms(doc: t.SpecT, container: t.ElementT) -> None:
         container,
         h.E.h3(
             {"class": "no-num no-ref", "id": h.safeID(doc, "index-defined-here")},
-            _("Terms defined by this specification"),
+            _t("Terms defined by this specification"),
         ),
     )
 
@@ -313,18 +291,18 @@ def addIndexOfLocallyDefinedTerms(doc: t.SpecT, container: t.ElementT) -> None:
         dfnType = el.get("data-dfn-type")
         if dfnID is None or dfnType is None:
             continue
-        linkTexts = config.linkTextsFromElement(el)
-        headingLevel = h.headingLevelOfElement(el) or _("Unnumbered section")
+        linkTexts = h.linkTextsFromElement(el)
+        headingLevel = h.headingLevelOfElement(el) or _t("Unnumbered section")
         if dfnType == "argument":
             # Don't generate index entries for arguments.
             continue
         if el.get("data-dfn-for") is not None:
-            disamb = _("{type} for {forVals}").format(
+            disamb = _t("{type} for {forVals}").format(
                 type=dfnType,
                 forVals=", ".join(config.splitForValues(el.get("data-dfn-for", ""))),
             )
         elif dfnType == "dfn":
-            disamb = _("definition of")
+            disamb = _t("definition of")
         else:
             disamb = "({})".format(dfnType)
 
@@ -341,14 +319,14 @@ def addIndexOfLocallyDefinedTerms(doc: t.SpecT, container: t.ElementT) -> None:
     h.appendChild(container, indexHTML, allowEmpty=True)
 
 
-def disambiguator(ref: r.RefWrapper, types: set[str] | None, specs: set[str] | None) -> str:
+def disambiguator(ref: r.RefWrapper, types: set[str] | None, specs: list[str] | None) -> str:
     disambInfo = []
     if types is None or len(types) > 1:
         disambInfo.append(ref.type)
     if specs is None or len(specs) > 1:
-        disambInfo.append(_("in {spec}").format(spec=ref.spec))
+        disambInfo.append(_t("in {spec}").format(spec=ref.spec))
     if ref.for_:
-        disambInfo.append(_("for {forVals}").format(forVals=", ".join(x.strip() for x in ref.for_)))
+        disambInfo.append(_t("for {forVals}").format(forVals=", ".join(x.strip() for x in ref.for_)))
     return ", ".join(disambInfo)
 
 
@@ -378,7 +356,7 @@ def addExplicitIndexes(doc: t.SpecT) -> None:
 
         if el.get("data-link-spec"):
             # Yes, this is dumb. Accidental over-firing of a shortcut attribute. >_<
-            specs = {x.strip() for x in el.get("data-link-spec", "").split(",")}
+            specs = sorted({x.strip() for x in el.get("data-link-spec", "").split(",")})
             for s in list(specs):
                 if s not in doc.refs.specs:
                     m.die(f"Unknown spec name '{s}' on {h.outerHTML(el)}", el=el)
@@ -503,7 +481,7 @@ def htmlFromIndexTerms(entries: t.Mapping[str, list[IndexTerm]]) -> t.ElementT:
                 topList,
                 h.E.li(
                     h.E.a({"href": item.url}, text),
-                    h.E.span(_(", in "), item.label) if item.label else "",
+                    h.E.span(_t(", in "), item.label) if item.label else "",
                 ),
             )
         else:
@@ -514,7 +492,7 @@ def htmlFromIndexTerms(entries: t.Mapping[str, list[IndexTerm]]) -> t.ElementT:
                     ul,
                     h.E.li(
                         h.E.a({"href": item.url}, item.disambiguator),
-                        h.E.span(_(", in "), item.label) if item.label else "",
+                        h.E.span(_t(", in "), item.label) if item.label else "",
                     ),
                 )
     return topList
@@ -524,12 +502,11 @@ def addIndexOfExternallyDefinedTerms(doc: t.SpecT, container: t.ElementT) -> Non
     if not doc.externalRefsUsed.hasRefs():
         return
 
-    def makeLink(*contents: t.NodesT) -> t.ElementT:
-        return h.E.span({}, *contents)
+    def makeEntry(ref: t.RefWrapper, contents: t.NodesT) -> t.ElementT:
+        return h.E.span({"id": h.uniqueID("external-term", ref.url, ref.text)}, contents)
 
     ul = h.E.ul({"class": "index"})
 
-    atLeastOnePanel = False
     for specName, specData in doc.externalRefsUsed.sorted():
         # Skip entries that are *solely* a biblio entry.
         if not specData.refs:
@@ -554,28 +531,25 @@ def addIndexOfExternallyDefinedTerms(doc: t.SpecT, container: t.ElementT) -> Non
             h.E.li(h.E.a(attrs, "[", formatBiblioTerm(printableSpec), "]"), " defines the following terms:"),
         )
         termsUl = h.appendChild(specLi, h.E.ul())
-        for refText, refGroup in specData.sorted():
+        for _, refGroup in specData.sorted():
             if len(refGroup) == 1:
                 ref = refGroup.single()
-                link = makeLink(refText)
-                h.appendChild(termsUl, h.E.li(link))
-                dfnpanels.addExternalDfnPanel(link, ref, doc)
+                entry = makeEntry(ref, ref.displayText)
+                h.appendChild(termsUl, h.E.li(entry))
+                dfnpanels.addExternalDfnPanel(entry, ref, doc)
             else:
                 for forVal, ref in refGroup.sorted():
                     if forVal:
-                        link = makeLink(refText, " ", h.E.small({}, f"({_('for')} {forVal})"))
+                        entry = makeEntry(ref, [ref.displayText, " ", h.E.small({}, f"({_t('for')} {forVal})")])
                     else:
-                        link = makeLink(refText)
-                    h.appendChild(termsUl, h.E.li(link))
-                    dfnpanels.addExternalDfnPanel(link, ref, doc)
-            atLeastOnePanel = True
-    if atLeastOnePanel:
-        dfnpanels.addExternalDfnPanelStyles(doc)
+                        entry = makeEntry(ref, ref.displayText)
+                    h.appendChild(termsUl, h.E.li(entry))
+                    dfnpanels.addExternalDfnPanel(entry, ref, doc)
     h.appendChild(
         container,
         h.E.h3(
             {"class": "no-num no-ref", "id": h.safeID(doc, "index-defined-elsewhere")},
-            _(
+            _t(
                 "Terms defined by reference",
             ),
         ),
@@ -596,7 +570,7 @@ def addPropertyIndex(doc: t.SpecT) -> None:
         html,
         h.E.h2(
             {"class": "no-num no-ref", "id": h.safeID(doc, "property-index")},
-            _("Property Index"),
+            _t("Property Index"),
         ),
     )
 
@@ -749,7 +723,7 @@ def addIDLSection(doc: t.SpecT) -> None:
 
     h.appendChild(
         html,
-        h.E.h2({"class": "no-num no-ref", "id": h.safeID(doc, "idl-index")}, _("IDL Index")),
+        h.E.h2({"class": "no-num no-ref", "id": h.safeID(doc, "idl-index")}, _t("IDL Index")),
     )
 
     container = h.appendChild(html, h.E.pre({"class": "idl"}))
@@ -767,6 +741,63 @@ def addIDLSection(doc: t.SpecT) -> None:
     h.addClass(doc, container, "highlight")
 
 
+def addCDDLSection(doc: t.SpecT) -> None:
+    allCddlBlocks = [x for x in h.findAll("pre.cddl, xmp.cddl", doc) if h.isNormative(doc, x)]
+    if len(allCddlBlocks) == 0:
+        return
+    html = getFillContainer("cddl-index", doc=doc, default=True)
+    if html is None:
+        return
+
+    h.appendChild(
+        html,
+        h.E.h2({"class": "no-num no-ref", "id": h.safeID(doc, "cddl-index")}, _t("CDDL Index")),
+    )
+
+    # Specs such as WebDriver BiDi define two sets of CDDL definitions for
+    # the local and remote ends of the protocol. These modules need to be
+    # defined with a dfn of type "cddl-module". CDDL blocks can then reference
+    # one or more modules through a "data-cddl-module" attribute.
+    # When modules are defined, CDDL blocks that do not reference a module
+    # are considered to apply to all modules. In particular, they do not create
+    # a "default" module
+    cddlModules = [
+        (x.get("id", ""), x.get("data-lt", x.text or "").split("|"))
+        for x in h.findAll("dfn[data-dfn-type=cddl-module]", doc)
+    ]
+    if len(cddlModules) == 0:
+        cddlModules = [("", [""])]
+    for module in cddlModules:
+        cddlBlocks = []
+        for block in allCddlBlocks:
+            forModules = [x.strip() for x in block.get("data-cddl-module", "").split(",")]
+            if (len(forModules) == 1 and forModules[0] == "") or any(name in forModules for name in module[1]):
+                cddlBlocks.append(block)
+        if len(cddlBlocks) == 0:
+            continue
+        if module[1][0] != "":
+            h.appendChild(
+                html,
+                h.E.h3(
+                    {"class": "no-num no-ref", "id": h.safeID(doc, "cddl-index-" + module[0])},
+                    _t(module[1][0].capitalize()),
+                ),
+            )
+        container = h.appendChild(html, h.E.pre({"class": "cddl"}))
+        for block in cddlBlocks:
+            if h.hasClass(doc, block, "extract"):
+                continue
+            blockCopy = copy.deepcopy(block)
+            h.appendContents(container, blockCopy)
+            h.appendChild(container, "\n")
+        for el in h.findAll("[id]", container):
+            if el.tag == "dfn":
+                el.tag = "a"
+                el.set("href", "#" + el.get("id", ""))
+            del el.attrib["id"]
+        h.addClass(doc, container, "highlight")
+
+
 def addTOCSection(doc: t.SpecT) -> None:
     toc = getFillContainer("table-of-contents", doc=doc, default=False)
     if toc is None:
@@ -775,7 +806,7 @@ def addTOCSection(doc: t.SpecT) -> None:
         toc,
         h.E.h2(
             {"class": "no-num no-toc no-ref", "id": h.safeID(doc, "contents")},
-            _("Table of Contents"),
+            _t("Table of Contents"),
         ),
     )
 
@@ -838,7 +869,7 @@ def addTOCSection(doc: t.SpecT) -> None:
                         h.E.span({"class": "secno"}, header.get("data-level", "")),
                         " ",
                         copy.deepcopy(h.find(".content", header)),
-                    )
+                    ),
                 ),
             )
             containers[level + 1] = h.appendChild(li, h.E.ol({"class": "toc"}))
@@ -920,7 +951,7 @@ def addSpecMetadataSection(doc: t.SpecT) -> None:
                 missingInfo = True
         if missingInfo:
             m.warn(
-                f"Bikeshed doesn't have all the translation info for '{lang}'. Please add to bikeshed/spec-data/readonly/languages.json and submit a PR!"
+                f"Bikeshed doesn't have all the translation info for '{lang}'. Please add to bikeshed/spec-data/readonly/languages.json and submit a PR!",
             )
         if nativeName:
             return h.E.span(
@@ -948,7 +979,7 @@ def addSpecMetadataSection(doc: t.SpecT) -> None:
         dated = doc.refs.getLatestBiblioRef(key)
         if not dated:
             m.die(
-                f"While trying to generate a Previous Version line, couldn't find a dated biblio reference for {key}."
+                f"While trying to generate a Previous Version line, couldn't find a dated biblio reference for {key}.",
             )
             return None
         return h.E.a({"href": dated.url, "rel": "prev"}, dated.url)
@@ -959,7 +990,7 @@ def addSpecMetadataSection(doc: t.SpecT) -> None:
         md.setdefault("This version", []).append(h.E.a({"href": mac["version"], "class": "u-url"}, mac["version"]))
     if doc.md.TR:
         md.setdefault("Latest published version", []).append(h.E.a({"href": doc.md.TR}, doc.md.TR))
-    if doc.md.ED and doc.md.status in config.snapshotStatuses:
+    if doc.md.ED and "TR" in doc.doctype.status.requires:
         md.setdefault("Editor's Draft", []).append(h.E.a({"href": doc.md.ED}, doc.md.ED))
     if doc.md.previousVersions:
         md["Previous Versions"] = [printPreviousVersion(ver) for ver in doc.md.previousVersions]
@@ -976,7 +1007,7 @@ def addSpecMetadataSection(doc: t.SpecT) -> None:
                     + doc.md.mailingList
                     + "?subject=%5B"
                     + mac["shortname"]
-                    + "%5D%20YOUR%20TOPIC%20HERE"
+                    + "%5D%20YOUR%20TOPIC%20HERE",
                 },
                 doc.md.mailingList,
             ),
@@ -997,13 +1028,10 @@ def addSpecMetadataSection(doc: t.SpecT) -> None:
         md.setdefault("Feedback", []).append(span)
     if doc.md.implementationReport is not None:
         md.setdefault("Implementation Report", []).append(
-            h.E.a({"href": doc.md.implementationReport}, doc.md.implementationReport)
+            h.E.a({"href": doc.md.implementationReport}, doc.md.implementationReport),
         )
     if doc.md.testSuite is not None:
         md.setdefault("Test Suite", []).append(h.E.a({"href": doc.md.testSuite}, doc.md.testSuite))
-    elif (doc.md.vshortname in doc.testSuites) and (doc.testSuites[doc.md.vshortname].url is not None):
-        url = doc.testSuites[doc.md.vshortname].url
-        md.setdefault("Test Suite", []).append(h.E.a({"href": url}, url))
     if doc.md.issues:
         if doc.md.TR:
             md.setdefault("Feedback", []).extend([h.E.a({"href": href}, text) for text, href in doc.md.issues])
@@ -1022,7 +1050,7 @@ def addSpecMetadataSection(doc: t.SpecT) -> None:
             h.E.label(
                 {"for": h.safeID(doc, "hidedel"), "id": h.safeID(doc, "hidedel-label")},
                 "Hide deleted text",
-            )
+            ),
         ]
         h.prependChild(
             doc.body,
@@ -1031,22 +1059,24 @@ def addSpecMetadataSection(doc: t.SpecT) -> None:
                     "type": "checkbox",
                     "id": h.safeID(doc, "hidedel"),
                     "style": "display:none",
-                }
+                },
             ),
         )
-        doc.extraStyles[
-            "style-hidedel"
-        ] = """
-            #hidedel:checked ~ del, #hidedel:checked ~ * del { display:none; }
-            #hidedel ~ #hidedel-label::before, #hidedel ~ * #hidedel-label::before { content: "☐ "; }
-            #hidedel:checked ~ #hidedel-label::before, #hidedel:checked ~ * #hidedel-label::before { content: "☑ "; }
-        """
+        doc.extraJC.addHidedel()
 
     # Merge "custom" metadata into non-custom, when they match up
     # and upgrade html-text values into real elements
     otherMd: OrderedDict[str, list[MetadataValueT]] = OrderedDict()
     for k, vs in doc.md.otherMetadata.items():
-        parsed: list[t.NodesT] = [h.parseHTML(doc.fixText(v)) if isinstance(v, str) else v for v in vs]
+        parsed: list[t.NodesT] = []
+        for v in vs:
+            if isinstance(v, str):
+                if v == "":
+                    continue
+                htmlText = h.parseText(v, h.ParseConfig.fromSpec(doc, context=f"!{k} metadata"))
+                parsed.append(h.parseHTML(htmlText))
+            else:
+                parsed.append(v)
         if k in md:
             md[k].extend(parsed)
         else:
@@ -1080,17 +1110,17 @@ def createMdEntry(key: str, dirtyVals: t.Sequence[MetadataValueT], doc: t.SpecT)
     }
     if len(vals) > 1 and displayKey in pluralization:
         displayKey = pluralization[displayKey]
-    displayKey = _(displayKey)
+    displayKey = _t(displayKey)
     # Handle some custom <dt> structures
     if key in ("Editor", "Former Editor"):
         ret = [h.E.dt({"class": "editor"}, displayKey, ":")]
     elif key == "Translations":
-        ret = [h.E.dt(displayKey, " ", h.E.small(_("(non-normative)")), ":")]
+        ret = [h.E.dt(displayKey, " ", h.E.small(_t("(non-normative)")), ":")]
     else:
         ret = [h.E.dt(displayKey, ":")]
     # Add all the values, wrapping in a <dd> if necessary.
     for val in vals:
-        if h.isElement(val) and val.tag == "dd":
+        if h.isElement(val) and h.tagName(val) == "dd":
             ret.append(val)
         else:
             ret.append(h.E.dd({}, val))
@@ -1140,17 +1170,17 @@ def addReferencesSection(doc: t.SpecT) -> None:
 
     h.appendChild(
         container,
-        h.E.h2({"class": "no-num no-ref", "id": h.safeID(doc, "references")}, _("References")),
+        h.E.h2({"class": "no-num no-ref", "id": h.safeID(doc, "references")}, _t("References")),
     )
 
     normRefs = sorted(doc.normativeRefs.values(), key=lambda r: r.linkText.lower())
-    normRefKeys = set(r.linkText.lower() for r in doc.normativeRefs.values())
+    normRefKeys = {r.linkText.lower() for r in doc.normativeRefs.values()}
     if len(normRefs) > 0:
         dl = h.appendChild(
             container,
             h.E.h3(
                 {"class": "no-num no-ref", "id": h.safeID(doc, "normative")},
-                _("Normative References"),
+                _t("Normative References"),
             ),
             h.E.dl(),
         )
@@ -1175,7 +1205,7 @@ def addReferencesSection(doc: t.SpecT) -> None:
             container,
             h.E.h3(
                 {"class": "no-num no-ref", "id": h.safeID(doc, "informative")},
-                _("Informative References"),
+                _t("Informative References"),
             ),
             h.E.dl(),
         )
@@ -1203,7 +1233,7 @@ def addIssuesSection(doc: t.SpecT) -> None:
         container,
         h.E.h2(
             {"class": "no-num no-ref", "id": h.safeID(doc, "issues-index")},
-            _("Issues Index"),
+            _t("Issues Index"),
         ),
     )
     container = h.appendChild(container, h.E.div({"style": "counter-reset:issue"}))
@@ -1216,7 +1246,7 @@ def addIssuesSection(doc: t.SpecT) -> None:
         h.appendChild(
             el,
             " ",
-            h.E.a({"href": "#" + issue.get("id", ""), "class": "issue-return", "title": _("Jump to section")}, "↵"),
+            h.E.a({"href": "#" + issue.get("id", ""), "class": "issue-return", "title": _t("Jump to section")}, "↵"),
         )
     for idel in h.findAll("[id]", container):
         del idel.attrib["id"]
@@ -1233,3 +1263,38 @@ def formatBiblioTerm(linkText: str) -> str:
     if linkText.islower():
         return linkText.upper()
     return linkText
+
+
+def addDarkmodeIndicators(doc: t.SpecT) -> None:
+    # Unless otherwise indicated, Bikeshed docs are assumed
+    # to be darkmode-aware.
+    if not doc.md.darkMode:
+        return
+
+    # If a boilerplate already contains a color-scheme,
+    # assume they know what they're doing.
+    # Otherwise, add the color-scheme meta to indicate darkmode-ness.
+    existingColorScheme = h.find('meta[name="color-scheme"]', doc)
+    if existingColorScheme is not None:
+        return
+    h.appendChild(
+        doc.head,
+        h.E.meta({"name": "color-scheme", "content": "dark light"}),
+    )
+
+    # Specs using the Bikeshed stylesheet will get darkmode colors
+    # automatically, but W3C specs don't. Instead, auto-add their
+    # darkmode styles.
+    w3cStylesheet = h.find('link[href^="https://www.w3.org/StyleSheets/TR"]', doc)
+    if w3cStylesheet is not None:
+        h.appendChild(
+            doc.head,
+            h.E.link(
+                {
+                    "rel": "stylesheet",
+                    "href": "https://www.w3.org/StyleSheets/TR/2021/dark.css",
+                    "type": "text/css",
+                    "media": "(prefers-color-scheme: dark)",
+                },
+            ),
+        )
