@@ -4,8 +4,9 @@ import json
 from collections import OrderedDict
 from datetime import datetime
 
-from .. import config, h, messages as m, t
-from ..translate import _
+from .. import h, t
+from .. import messages as m
+from ..translate import _t
 
 
 def addCanIUsePanels(doc: t.SpecT) -> list[t.ElementT]:
@@ -54,8 +55,7 @@ def addCanIUsePanels(doc: t.SpecT) -> list[t.ElementT]:
         panels.append(panel)
 
     if panels:
-        doc.extraStyles["style-caniuse-panel"] = getModuleFile("caniuse.css")
-        doc.extraStyles["style-darkmode"] += getModuleFile("caniuse-dark.css")
+        doc.extraJC.addCiu()
 
     return panels
 
@@ -65,7 +65,7 @@ def canIUsePanelFor(id: str, data: t.JSONT, update: str, classFromBrowser: dict[
         {"class": "caniuse-status unpositioned", "data-deco": ""},
         h.E.summary({}, "CanIUse"),
     )
-    mainPara = h.E.p({"class": "support"}, h.E.b({}, _("Support:")))
+    mainPara = h.E.p({"class": "support"}, h.E.b({}, _t("Support:")))
     h.appendChild(panel, mainPara)
     for browser, support in data["support"].items():
         statusCode = support[0]
@@ -80,16 +80,19 @@ def canIUsePanelFor(id: str, data: t.JSONT, update: str, classFromBrowser: dict[
         panel,
         h.E.p(
             {"class": "caniuse"},
-            _("Source: "),
+            _t("Source: "),
             h.E.a({"href": "https://caniuse.com/#feat=" + id}, "caniuse.com"),
-            _(" as of {date}").format(date=update),
+            _t(" as of {date}").format(date=update),
         ),
     )
     return panel
 
 
 def browserCompatSpan(
-    browserCodeName: str, browserFullName: str, statusCode: str, minVersion: str | None = None
+    browserCodeName: str,
+    browserFullName: str,
+    statusCode: str,
+    minVersion: str | None = None,
 ) -> t.ElementT:
     if statusCode == "n" or minVersion is None:
         minVersion = "None"
@@ -102,7 +105,7 @@ def browserCompatSpan(
     statusClass = {"y": "yes", "n": "no", "a": "partial"}[statusCode]
     outer = h.E.span({"class": browserCodeName + " " + statusClass})
     if statusCode == "a":
-        h.appendChild(outer, h.E.span({}, h.E.span({}, browserFullName, _(" (limited)"))))
+        h.appendChild(outer, h.E.span({}, h.E.span({}, browserFullName, _t(" (limited)"))))
     else:
         h.appendChild(outer, h.E.span({}, browserFullName))
     h.appendChild(outer, h.E.span({}, minVersion))
@@ -122,8 +125,8 @@ def validateCanIUseURLs(doc: t.SpecT, canIUseData: CanIUseManager, elements: lis
         if not sawTheURL and url not in doc.md.ignoreCanIUseUrlFailure:
             m.warn(
                 f"The Can I Use URL '{url}' isn't associated with any of the Can I Use features."
-                "Please check Can I Use for the correct spec url, and either correct your spec or correct Can I Use."
-                "If the URL is correct and you'd like to keep it in pre-emptively, add the URL to a 'Ignore Can I Use URL Failure' metadata."
+                + "Please check Can I Use for the correct spec url, and either correct your spec or correct Can I Use."
+                + "If the URL is correct and you'd like to keep it in pre-emptively, add the URL to a 'Ignore Can I Use URL Failure' metadata.",
             )
 
     # Second, ensure that every feature in the data corresponding to one of the listed URLs
@@ -140,12 +143,12 @@ def validateCanIUseURLs(doc: t.SpecT, canIUseData: CanIUseManager, elements: lis
     if unusedFeatures:
         featureList = "\n".join(" * {0} - https://caniuse.com/#feat={0}".format(x) for x in sorted(unusedFeatures))
         m.warn(
-            f"The following Can I Use features are associated with your URLs, but don't show up in your spec:\n{featureList}"
+            f"The following Can I Use features are associated with your URLs, but don't show up in your spec:\n{featureList}",
         )
 
 
 class CanIUseManager:
-    def __init__(self, dataFile: t.DataFileRequester):
+    def __init__(self, dataFile: t.DataFileRequester) -> None:
         self.dataFile = dataFile
         data = json.loads(
             self.dataFile.fetch("caniuse", "data.json", str=True),
@@ -170,8 +173,3 @@ class CanIUseManager:
         )
         self.features[featureName] = data
         return t.cast("t.JSONT", data)
-
-
-def getModuleFile(filename: str) -> str:
-    with open(config.scriptPath("caniuse", filename), "r", encoding="utf-8") as fh:
-        return fh.read()
