@@ -1,10 +1,12 @@
 from __future__ import annotations
+
 import argparse
 import json
 import os
 import sys
 
-from . import config, constants, update, messages as m
+from . import config, constants, printjson, update
+from . import messages as m
 
 
 def main() -> None:
@@ -62,15 +64,28 @@ def main() -> None:
     argparser.add_argument(
         "--print",
         dest="printMode",
-        action="store",
+        choices=m.PRINT_MODES,
         default=None,
-        help="Print mode. Options are 'plain' (just text), 'console' (colored with console color codes), 'markup', and 'json'. Defaults to 'console'.",
+        help="How Bikeshed formats its message output. Options are 'plain' (just text), 'console' (text with console color codes), 'markup' (XML), and 'json' (JSON stream). Defaults to 'console'.",
     )
     argparser.add_argument(
         "--die-on",
         dest="errorLevel",
-        choices=["nothing", "fatal", "link-error", "warning", "everything"],
-        help="Determines what sorts of errors cause Bikeshed to die (quit immediately with an error status code). Default is 'fatal'; the -f flag is a shorthand for 'nothing'",
+        choices=list(m.MESSAGE_LEVELS.keys()),
+        help="Determines what sorts of errors cause Bikeshed to die (refuse to generate an output document). Default is 'fatal'; the -f flag is a shorthand for 'nothing'",
+    )
+    argparser.add_argument(
+        "--die-when",
+        dest="errorTiming",
+        choices=m.DEATH_TIMING,
+        default="late",
+        help="When a disallowed error should force Bikeshed to stop. 'early' causes it to stop immediately so you can deal with the first error; 'late' makes it process the entire document first and only stop at the end so you can see all the errors.",
+    )
+    argparser.add_argument(
+        "--no-update",
+        dest="skipUpdate",
+        action="store_true",
+        help="Skips checking if your data files are up-to-date.",
     )
     argparser.add_argument(
         "--allow-nonlocal-files",
@@ -131,21 +146,30 @@ def main() -> None:
         help="Process a spec source file into a valid output file and publish it according to certain automatic protocols.",
     )
     echidnaParser.add_argument("infile", nargs="?", default=None, help="Path to the source file.")
-    echidnaParser.add_argument(
-        "--gh-token",
-        dest="ghToken",
-        nargs="?",
-        help="GitHub access token. Useful to avoid API rate limits. Generate tokens: https://github.com/settings/tokens.",
-    )
     echidnaParser.add_argument("--u", dest="un", metavar="USERNAME", required=False, help="W3C username.")
     echidnaParser.add_argument("--p", dest="pw", metavar="PASSWORD", required=False, help="W3C password.")
     echidnaParser.add_argument(
         "--decision",
         dest="decision",
-        metavar="DECISION_URL",
+        metavar="URL",
         required=False,
         help="URL recording the decision to publish.",
     )
+    echidnaParser.add_argument(
+        "--gh-token",
+        dest="ghToken",
+        required=False,
+        help="GitHub access token. Useful to avoid API rate limits. Generate tokens: https://github.com/settings/tokens.",
+    )
+    echidnaParser.add_argument(
+        "--timeout",
+        dest="timeout",
+        metavar="SECONDS",
+        required=False,
+        default=3,
+        type=float,
+        help="The timeout for contacting the publication service (default 3s).",
+    )
     echidnaParser.add_argument(
         "--editorial",
         dest="editorial",
@@ -233,20 +257,24 @@ def main() -> None:
         help="Update supporting files (those in /spec-data).",
         epilog="If no options are specified, everything is downloaded.",
     )
-    updateParser.add_argument(
+    updateModeGroup = updateParser.add_mutually_exclusive_group()
+    updateModeGroup.add_argument(
         "--skip-manifest",
-        dest="manifest",
+        dest="updateMode",
         action="store_const",
-        const="skip",
+        const=update.UpdateMode.MANUAL,
         help="Forces Bikeshed to do a full update manually, rather than using the manifest to get the preprocessed update (which can be several minutes old).",
     )
-    updateParser.add_argument(
+    updateModeGroup.add_argument(
         "--force-manifest",
-        dest="manifest",
+        dest="updateMode",
         action="store_const",
-        const="force",
+        const=update.UpdateMode.MANIFEST | update.UpdateMode.FORCE,
         help="Force a manifest-based update even if local manifest data is more recent than the remote manifest.",
     )
+    updateModeGroup.set_defaults(
+        updateMode=update.UpdateMode.BOTH,
+    )
     updateParser.add_argument("--anchors", action="store_true", help="Download crossref anchor data.")
     updateParser.add_argument("--backrefs", action="store_true", help="Download link backref data.")
     updateParser.add_argument("--biblio", action="store_true", help="Download biblio data.")
@@ -348,6 +376,9 @@ def main() -> None:
     sourceParser.add_argument("infile", nargs="?", default=None, help="Path to the source file.")
     sourceParser.add_argument("outfile", nargs="?", default=None, help="Path to the output file.")
 
+    outlineParser = subparsers.add_parser("outline", help="Generates an outline for the spec.")
+    outlineParser.add_argument("infile", nargs="?", default=None, help="Path to the source file.")
+
     testParser = subparsers.add_parser("test", help="Tools for running Bikeshed's testsuite.")
     testParser.add_argument(
         "--rebase",
@@ -356,18 +387,25 @@ def main() -> None:
         help="Rebase the specified files.",
     )
     testParser.add_argument(
-        "--manual-only",
+        "--manual",
         dest="manualOnly",
         default=False,
         action="store_true",
         help="Skip testing the real-world files in the repo, and only run the manually-written ones.",
     )
     testParser.add_argument(
-        "testFiles",
-        default=[],
-        metavar="FILE",
-        nargs="*",
-        help="Run these tests. If called with no args, tests everything.",
+        "--folder",
+        dest="folders",
+        default=None,
+        nargs="+",
+        help="Only run tests whose paths contain any of these folder names.",
+    )
+    testParser.add_argument(
+        "--file",
+        dest="files",
+        default=None,
+        nargs="+",
+        help="Only run tests whose filenames contain any of these strings as substrings.",
     )
 
     profileParser = subparsers.add_parser(
@@ -395,7 +433,13 @@ def main() -> None:
         help="Save the graph to a specified SVG file, rather than outputting with xdot immediately.",
     )
 
-    subparsers.add_parser("template", help="Outputs a skeleton .bs file for you to start with.")
+    templateParser = subparsers.add_parser("template", help="Outputs a skeleton .bs file for you to start with.")
+    templateParser.add_argument(
+        "variant",
+        choices=["base", "minimal", "test"],
+        nargs="?",
+        default="base",
+    )
 
     wptParser = subparsers.add_parser("wpt", help="Tools for writing Web Platform Tests.")
     wptParser.add_argument(
@@ -407,23 +451,29 @@ def main() -> None:
 
     options, extras = argparser.parse_known_args()
 
-    constants.quiet = options.quiet
     if options.silent:
-        constants.quiet = float("infinity")
-    constants.setErrorLevel(options.errorLevel)
-    constants.dryRun = options.dryRun
-    constants.asciiOnly = options.asciiOnly
+        m.state.printOn = "nothing"
+        m.state.silent = True
+    else:
+        m.state.printOn = m.MessagesState.categoryName(options.quiet)
+    if options.errorLevel is not None:
+        m.state.dieOn = options.errorLevel
+    m.state.dieWhen = options.errorTiming
+    m.state.asciiOnly = options.asciiOnly
     if options.printMode is None:
         if "NO_COLOR" in os.environ or os.environ.get("TERM") == "dumb":
-            constants.printMode = "plain"
+            m.state.printMode = "plain"
         else:
-            constants.printMode = "console"
+            m.state.printMode = "console"
     else:
-        constants.printMode = options.printMode
+        m.state.printMode = options.printMode
+    constants.dryRun = options.dryRun
     constants.chroot = not options.allowNonlocalFiles
     constants.executeCode = options.allowExecute
 
-    update.fixupDataFiles()
+    if options.subparserName in ("spec", "echnida", "watch", "serve", "refs"):
+        updateMode = update.UpdateMode.NONE if options.skipUpdate else update.UpdateMode.BOTH
+        update.fixupDataFiles(updateMode=updateMode)
     if options.subparserName == "update":
         handleUpdate(options)
     elif options.subparserName == "spec":
@@ -442,12 +492,14 @@ def main() -> None:
         handleIssuesList(options)
     elif options.subparserName == "source":
         handleSource(options)
+    elif options.subparserName == "outline":
+        handleOutline(options)
     elif options.subparserName == "test":
         handleTest(options, extras)
     elif options.subparserName == "profile":
         handleProfile(options)
     elif options.subparserName == "template":
-        handleTemplate()
+        handleTemplate(options)
     elif options.subparserName == "wpt":
         handleWpt(options)
 
@@ -464,7 +516,7 @@ def handleUpdate(options: argparse.Namespace) -> None:
         languages=options.languages,
         wpt=options.wpt,
         dryRun=constants.dryRun,
-        manifestMode=options.manifest,
+        updateMode=options.updateMode,
     )
 
 
@@ -478,7 +530,14 @@ def handleSpec(options: argparse.Namespace, extras: list[str]) -> None:
         token=options.ghToken,
         lineNumbers=options.lineNumbers,
     )
+    if not doc.valid:
+        m.die("Spec is in an invalid state; exitting.")
+        return
     doc.mdCommandLine = metadata.fromCommandLine(extras)
+    if options.errorLevel:
+        doc.mdCommandLine.addData("Die On", options.errorLevel)
+    if options.errorTiming:
+        doc.mdCommandLine.addData("Die When", options.errorTiming)
     if options.byos:
         doc.mdCommandLine.addData("Group", "byos")
     doc.preprocess()
@@ -490,6 +549,9 @@ def handleEchidna(options: argparse.Namespace, extras: list[str]) -> None:
     from .Spec import Spec
 
     doc = Spec(inputFilename=options.infile, token=options.ghToken)
+    if not doc.valid:
+        m.die("Spec is in an invalid state; exitting.")
+        return
     doc.mdCommandLine = metadata.fromCommandLine(extras)
     doc.mdCommandLine.addData("Prepare For TR", "yes")
     doc.preprocess()
@@ -506,6 +568,7 @@ def handleEchidna(options: argparse.Namespace, extras: list[str]) -> None:
             additionalDirectories=addDirs,
             cc=options.cc,
             editorial=options.editorial,
+            timeout=options.timeout,
         )
 
 
@@ -513,9 +576,10 @@ def handleWatch(options: argparse.Namespace, extras: list[str]) -> None:
     from . import metadata
     from .Spec import Spec
 
-    # Can't have an error killing the watcher
-    constants.setErrorLevel("nothing")
     doc = Spec(inputFilename=options.infile, token=options.ghToken)
+    if not doc.valid:
+        m.die("Spec is in an invalid state; exitting.")
+        return
     doc.mdCommandLine = metadata.fromCommandLine(extras)
     if options.byos:
         doc.mdCommandLine.addData("Group", "byos")
@@ -526,9 +590,13 @@ def handleServe(options: argparse.Namespace, extras: list[str]) -> None:
     from . import metadata
     from .Spec import Spec
 
-    constants.setErrorLevel("nothing")
     doc = Spec(inputFilename=options.infile, token=options.ghToken)
+    if not doc.valid:
+        m.die("Spec is in an invalid state; exitting.")
+        return
     doc.mdCommandLine = metadata.fromCommandLine(extras)
+    # Can't have an error killing the watcher
+    doc.mdCommandLine.addData("Die On", "nothing")
     if options.byos:
         doc.mdCommandLine.addData("Group", "byos")
     doc.watch(outputFilename=options.outfile, port=int(options.port))
@@ -538,8 +606,8 @@ def handleDebug(options: argparse.Namespace, extras: list[str]) -> None:
     from . import metadata
     from .Spec import Spec
 
-    constants.setErrorLevel("nothing")
-    constants.quiet = 2
+    m.state.dieOn = "nothing"
+    m.state.printOn = "fatal"
     if options.printExports:
         doc = Spec(inputFilename=options.infile)
         doc.mdCommandLine = metadata.fromCommandLine(extras)
@@ -556,7 +624,7 @@ def handleDebug(options: argparse.Namespace, extras: list[str]) -> None:
         doc.preprocess()
         exec(f"print({options.code})")
     elif options.refreshData:
-        constants.quiet = 0
+        m.state.printOn = "everything"
         update.updateReadonlyDataFiles()
         m.warn("Don't forget to bump the version number!")
     elif options.printMetadata:
@@ -569,16 +637,16 @@ def handleDebug(options: argparse.Namespace, extras: list[str]) -> None:
             "document": doc.mdDocument.allData if doc.mdDocument else [],
             "command-line": doc.mdCommandLine.allData,
         }
-        print(json.dumps(md, indent=2, default=config.getjson))
+        print(json.dumps(md, indent=2, default=printjson.getjson))  # noqa: T201
 
 
 def handleRefs(options: argparse.Namespace, extras: list[str]) -> None:
-    from . import metadata, datablocks
+    from . import datablocks, metadata
     from .refs import ReferenceManager
     from .Spec import Spec
 
-    constants.setErrorLevel("nothing")
-    constants.quiet = 10
+    m.state.dieOn = "nothing"
+    m.state.printOn = "nothing"
     doc = Spec(inputFilename=options.infile)
     if doc.valid:
         doc.mdCommandLine = metadata.fromCommandLine(extras)
@@ -598,10 +666,10 @@ def handleRefs(options: argparse.Namespace, extras: list[str]) -> None:
         latestOnly=options.latestOnly,
         exact=options.exact,
     )
-    if constants.printMode == "json":
-        m.p(json.dumps(refs, indent=2, default=config.getjson))
+    if m.state.printMode == "json":
+        m.p(json.dumps(refs, indent=2, default=printjson.getjson))
     else:
-        m.p(config.printjson(refs))
+        m.p(printjson.printjson(refs))
 
 
 def handleIssuesList(options: argparse.Namespace) -> None:
@@ -614,25 +682,39 @@ def handleIssuesList(options: argparse.Namespace) -> None:
 
 
 def handleSource(options: argparse.Namespace) -> None:
-    if not options.bigText:  # If no options are given, do all options.
-        options.bigText = True
-    if options.bigText:
-        from . import fonts
+    from . import fonts
 
-        font = fonts.Font()
+    try:
+        fontPath = config.scriptPath("fonts", "smallblocks.bsfont")
+        font = fonts.Font.fromPath(fontPath)
         fonts.replaceComments(font=font, inputFilename=options.infile, outputFilename=options.outfile)
+    except Exception as e:
+        m.die(f"Error trying to embiggen text:\n{e}")
+        return
+
+
+def handleOutline(options: argparse.Namespace) -> None:
+    from . import outline
+    from .Spec import Spec
+
+    doc = Spec(inputFilename=options.infile)
+    if not doc.valid:
+        return
+    with m.messagesSilent() as _:
+        doc.preprocess()
+    m.say(outline.printOutline(doc))
 
 
 def handleTest(options: argparse.Namespace, extras: list[str]) -> None:
     from . import metadata, test
 
     md = metadata.fromCommandLine(extras)
-    constants.setErrorLevel("nothing")
-    constants.quiet = 100
+    m.state.dieOn = "nothing"
+    filters = test.TestFilter.fromOptions(options)
     if options.rebase:
-        test.rebase(options.testFiles, md=md)
+        test.rebase(filters, md=md)
     else:
-        result = test.runAllTests(options.testFiles, manualOnly=options.manualOnly, md=md)
+        result = test.run(filters, md=md)
         sys.exit(0 if result else 1)
 
 
@@ -640,45 +722,73 @@ def handleProfile(options: argparse.Namespace) -> None:
     root = f'--root="{options.root}"' if options.root else ""
     leaf = f'--leaf="{options.leaf}"' if options.leaf else ""
     if options.svgFile:
-        os.system(
-            "time python -m cProfile -o stat.prof -m bikeshed -f spec && gprof2dot -f pstats --skew=.0001 {root} {leaf} stat.prof | dot -Tsvg -o {svg} && rm stat.prof".format(
-                root=root, leaf=leaf, svg=options.svgFile
-            )
+        os.system(  # noqa: S605
+            f"time python -m cProfile -o stat.prof -m bikeshed -f spec && gprof2dot -f pstats --skew=.0001 {root} {leaf} stat.prof | dot -Tsvg -o {options.svgFile} && rm stat.prof",
         )
     else:
-        os.system(
-            "time python -m cProfile -o /tmp/stat.prof -m bikeshed -f spec && gprof2dot -f pstats --skew=.0001 {root} {leaf} /tmp/stat.prof | xdot &".format(
-                root=root, leaf=leaf
-            )
+        os.system(  # noqa: S605
+            f"time python -m cProfile -o /tmp/stat.prof -m bikeshed -f spec && gprof2dot -f pstats --skew=.0001 {root} {leaf} /tmp/stat.prof | xdot &",
         )
 
 
-def handleTemplate() -> None:
-    m.p(
-        """<pre class='metadata'>
+def handleTemplate(options: argparse.Namespace) -> None:
+    if options.variant == "base":
+        m.p(
+            """<pre class='metadata'>
 Title: Your Spec Title
 Shortname: your-spec
 Level: 1
 Status: w3c/UD
 Group: WGNAMEORWHATEVER
+Repository: org/repo-name
 URL: http://example.com/url-this-spec-will-live-at
 Editor: Your Name, Your Company http://example.com/your-company, your-email@example.com, http://example.com/your-personal-website
 Abstract: A short description of your spec, one or two sentences.
+Complain About: accidental-2119 yes, missing-example-ids yes
+Markup Shorthands: markdown yes, css no
 </pre>
 
 Introduction {#intro}
 =====================
 
 Introduction here.
-"""
-    )
+""",
+        )
+    elif options.variant == "minimal":
+        m.p(
+            """<pre class='metadata'>
+Title: Test
+Editor: test
+Abstract: test
+Group: test
+Status: ls
+Shortname: test
+Boilerplate: style-autolinks off, style-colors off, style-counters off, style-selflinks off, style-issues off, style-md-lists off, style-dfn-panel off, script-dfn-panel off, script-dfn-panel-json off, script-dom-helper off, style-ref-hints off, script-ref-hints off, script-link-titles off, table-of-contents off, abstract off, index off, references off
+Markup Shorthands: markdown on
+</pre>
+""",
+        )
+    elif options.variant == "test":
+        m.p(
+            """<pre class=metadata>
+Group: test
+Shortname: foo
+Level: 1
+Status: LS
+ED: http://example.com/foo
+Editor: Example Editor
+Date: 1970-01-01
+Title: Test Title
+Abstract: Test Description
+</pre>
+""",
+        )
 
 
 def handleWpt(options: argparse.Namespace) -> None:
     if options.template:
         m.p(
-            """
-<!DOCTYPE html>
+            """<!DOCTYPE html>
 <meta charset=utf-8>
 <title>window.offscreenBuffering</title>
 <link rel=author title="AUTHOR NAME HERE" href="mailto:AUTHOR EMAIL HERE">
@@ -689,13 +799,13 @@ def handleWpt(options: argparse.Namespace) -> None:
 /* Choose the test type you want: */
 
 
-/* Standard, synchronous test */
+/* 1. Standard, synchronous test */
 test(function() {
     /* test code here */
 }, "TEST NAME HERE / SHORT DESCRIPTION PHRASE");
 
 
-/* Async test */
+/* 2. Async test */
 let t = async_test("TEST NAME HERE / SHORT DESCRIPTION PHRASE");
 somethingWithACallback( function(){ t.step(()=>{ /* test code here */}) );
 something.addEventListener('foo', t.step_func(()=>{ /* test code here */}));
@@ -704,7 +814,7 @@ t.done(); // when all tests are finished running
 something.addEventListener('foo', t.step_func_done(()=>{ /* test code here */}));
 
 
-/* Promise test */
+/* 3. Promise test */
 promise_test(function(){
     return somePromiseFunc().then(()=>{ /* test code here */ });
 }, "TEST NAME HERE / SHORT DESCRIPTION PHRASE");
@@ -721,5 +831,5 @@ assert_true(VALUE HERE, "TEST DESCRIPTION");
 assert_equals(ACTUAL VALUE HERE, EXPECTED VALUE HERE, "TEST DESCRIPTION");
 // lots more at http://web-platform-tests.org/writing-tests/testharness-api.html#list-of-assertions
 </script>
-"""
+""",
         )
