@@ -1,8 +1,10 @@
 from __future__ import annotations
+
 import dataclasses
 import re
 
-from . import config, h, messages as m, t
+from . import h, t
+from . import messages as m
 
 # Any element can have an include-if or exclude-if attribute,
 # containing a comma-separated list of conditions (described below).
@@ -26,7 +28,7 @@ from . import config, h, messages as m, t
 
 def processConditionals(doc: t.SpecT, container: t.ElementT | None = None) -> None:
     for el in h.findAll("[include-if], [exclude-if], if-wrapper", container if container is not None else doc):
-        if el.tag == "if-wrapper" and not h.hasAttr(el, "include-if", "exclude-if"):
+        if el.tag == "if-wrapper" and not h.hasAnyAttr(el, "include-if", "exclude-if"):
             m.die(
                 "<if-wrapper> elements must have an include-if and/or exclude-if attribute.",
                 el=el,
@@ -59,9 +61,9 @@ def processConditionals(doc: t.SpecT, container: t.ElementT | None = None) -> No
 def evalConditions(doc: t.SpecT, el: t.ElementT, conditionString: str) -> t.Generator[bool, None, None]:
     for cond in parseConditions(conditionString, el):
         if cond.type == "status":
-            yield config.looselyMatch(cond.value, doc.md.status)
+            yield doc.doctype.status.looselyMatch(cond.value)
         elif cond.type == "text macro":
-            for k in doc.macros.keys():
+            for k in doc.macros:
                 if k.upper() == cond.value:
                     yield True
                     break
@@ -71,7 +73,7 @@ def evalConditions(doc: t.SpecT, el: t.ElementT, conditionString: str) -> t.Gene
             yield (h.find(f'[boilerplate="{h.escapeCSSIdent(cond.value)}"]', doc) is not None)
         else:
             m.die(
-                f"Program error, some type of include/exclude-if condition wasn't handled: '{repr(cond)}'. Please report!",
+                f"Program error, some type of include/exclude-if condition wasn't handled: '{cond!r}'. Please report!",
                 el,
             )
             yield False
