@@ -47,13 +47,18 @@ dfnClassToType = {
     "facetdef": "facet",
     "http-headerdef": "http-header",
     "permissiondef": "permission",
+    "cddl-module": "cddl-module",
+    "cddl-type": "cddl-type",
+    "cddl-key": "cddl-key",
+    "cddl-parameter": "cddl-parameter",
+    "cddl-value": "cddl-value",
 }
 
-
 dfnTypes = frozenset(list(dfnClassToType.values()) + ["dfn"])
 maybeTypes = frozenset(["value", "type", "at-rule", "function", "selector"])
 cssTypes = frozenset(["property", "value", "at-rule", "descriptor", "type", "function", "selector"])
-markupTypes = frozenset(["element", "element-attr", "element-state", "attr-value"])
+markupTypes = frozenset(["element", "element-attr", "element-state", "attr-value", "element-sub"])
+cddlTypes = frozenset(["cddl-module", "cddl-type", "cddl-key", "cddl-parameter", "cddl-value"])
 idlTypes = frozenset(
     [
         "event",
@@ -78,12 +83,14 @@ idlTypes = frozenset(
         "maplike",
         "setlike",
         "permission",
-    ]
+    ],
 )
 idlNameTypes = frozenset(["interface", "namespace", "dictionary", "enum", "typedef", "callback"])
 functionishTypes = frozenset(["function", "method", "constructor", "stringifier"])
 idlMethodTypes = frozenset(["method", "constructor", "stringifier", "idl", "idl-name"])
-linkTypes = dfnTypes | frozenset(["propdesc", "functionish", "idl", "idl-name", "element-sub", "maybe", "biblio"])
+linkTypes = dfnTypes | frozenset(
+    ["propdesc", "functionish", "idl", "idl-name", "element-sub", "maybe", "biblio", "cddl"],
+)
 typesUsingFor = frozenset(
     [
         "descriptor",
@@ -108,7 +115,7 @@ typesUsingFor = frozenset(
         "mode",
         "context",
         "facet",
-    ]
+    ],
 )
 typesNotUsingFor = frozenset(
     [
@@ -123,7 +130,7 @@ typesNotUsingFor = frozenset(
         "typedef",
         "http-header",
         "permission",
-    ]
+    ],
 )
 assert not (typesUsingFor & typesNotUsingFor)
 lowercaseTypes = (
@@ -133,16 +140,51 @@ lowercaseTypes = (
 )
 
 
+def adjustKey(text: str, type: str) -> tuple[str, str]:
+    # For some types, upper/lowercase is just display, not semantic
+    if type not in lowercaseTypes:
+        return text, text
+    # For those, use lowercased version for link-resolving...
+    key = text.lower()
+    # ...and heuristically decide if the original casing should be preserved or not.
+    # 1. If it's already lowercase, great.
+    # 2. If it's got uppercase letters anywhere but the very first letter, keep the casing.
+    # 3. If it looks non-trivial, keep the casing.
+    # This hopefully removes casing from dfns that just start a sentence,
+    # without removing it from many dfns that need it.
+    # At some point we'll probably add an override for when this is wrong.
+    if re.match(r"[A-Z][^A-Z]+$", text):
+        # only one uppercase letter, right at the beginning.
+        # check if it's "non-trivial"; aka contains something other than
+        # letters, dashes, or whitespace
+        if re.search(r"[^\w\s-]", text):
+            # looks non-trivial, leave it alone
+            displayKey = text
+        else:
+            displayKey = text.lower()
+    else:
+        # Has either 0 or 2+ uppercase letters,
+        # or 1 uppercase in a non-initial position;
+        # leave it alone.
+        displayKey = text
+    return key, displayKey
+
+
 linkTypeToDfnType = {
     "propdesc": frozenset(["property", "descriptor"]),
     "functionish": functionishTypes,
     "idl": idlTypes,
     "idl-name": idlNameTypes,
+    # Generic parameters in CDDL may create duplicates and should not need to
+    # be referenced in practice, prevent generic "cddl" links to them
+    "cddl": frozenset([type for type in cddlTypes if type not in "cddl-parameter"]),
     "element-sub": frozenset(["element-attr", "element-state"]),
     "maybe": maybeTypes,
     "dfn": frozenset(["dfn"]),
     "biblio": frozenset(["biblio"]),
-    "codelike": frozenset(["element", "element-attr", "element-state", "attr-value"]) | idlTypes,
+    "codelike": frozenset(["element", "element-attr", "element-state", "attr-value"])
+    | idlTypes
+    | frozenset([type for type in cddlTypes if type != "cddl-module"]),
     "all": linkTypes,
 }
 for dfnType in dfnClassToType.values():
