@@ -3,16 +3,17 @@ from __future__ import annotations
 import collections.abc
 import os
 import re
+import time
 
 import lxml
 
-from .. import messages
-from .. import t
+from .. import messages, t
 
 
 def englishFromList(items: t.Iterable[str], conjunction: str = "or") -> str:
     # Format a list of strings into an English list.
     items = list(items)
+    assert len(items) > 0
     if len(items) == 1:
         return items[0]
     if len(items) == 2:
@@ -26,7 +27,8 @@ if t.TYPE_CHECKING:
 
 
 def intersperse(
-    iterable: t.Iterable[IntersperseU], delimiter: IntersperseV
+    iterable: t.Iterable[IntersperseU],
+    delimiter: IntersperseV,
 ) -> t.Generator[IntersperseU | IntersperseV, None, None]:
     first = True
     for x in iterable:
@@ -59,19 +61,21 @@ if t.TYPE_CHECKING:
 
 
 @t.overload
-def reSubObject(pattern: re.Pattern, string: str, repl: None) -> list[str | re.Match]:
-    ...
+def reSubObject(pattern: re.Pattern, string: str, repl: None) -> list[str | re.Match]: ...
 
 
 @t.overload
 def reSubObject(
-    pattern: re.Pattern, string: str, repl: t.Callable[[re.Match], ReSubObjectU]
-) -> list[str | ReSubObjectU]:
-    ...
+    pattern: re.Pattern,
+    string: str,
+    repl: t.Callable[[re.Match], ReSubObjectU],
+) -> list[str | ReSubObjectU]: ...
 
 
 def reSubObject(
-    pattern: re.Pattern, string: str, repl: t.Callable[[re.Match], ReSubObjectU] | None = None
+    pattern: re.Pattern,
+    string: str,
+    repl: t.Callable[[re.Match], ReSubObjectU] | None = None,
 ) -> list[str | re.Match] | list[str | ReSubObjectU] | list[str | re.Match | ReSubObjectU]:
     """
     like re.sub, but replacements don't have to be text;
@@ -105,64 +109,12 @@ def simplifyText(text: str) -> str:
     return text
 
 
-def linkTextsFromElement(el: t.ElementT) -> list[str]:
-    from ..h import find, textContent
-
-    if el.get("data-lt") == "":
-        return []
-    elif el.get("data-lt"):
-        rawText = el.get("data-lt", "")
-        if rawText in ["|", "||", "|||"]:
-            texts = [rawText]
-        else:
-            texts = [x.strip() for x in rawText.split("|")]
-    else:
-        if el.tag in ("dfn", "a"):
-            texts = [textContent(el).strip()]
-        elif el.tag in ("h2", "h3", "h4", "h5", "h6"):
-            textEl = find(".content", el)
-            if textEl is None:
-                textEl = el
-            texts = [textContent(textEl).strip()]
-    if el.get("data-local-lt"):
-        localTexts = [x.strip() for x in el.get("data-local-lt", "").split("|")]
-        for text in localTexts:
-            if text in texts:
-                # lt and local-lt both specify the same thing
-                raise DuplicatedLinkText(text, texts + localTexts, el)
-        texts += localTexts
-
-    texts = [re.sub(r"\s+", " ", x) for x in texts if x != ""]
-    return texts
-
-
-class DuplicatedLinkText(Exception):
-    def __init__(self, offendingText: str, allTexts: list[str], el: t.ElementT):
-        super().__init__()
-        self.offendingText = offendingText
-        self.allTexts = allTexts
-        self.el = el
-
-    def __unicode__(self) -> str:
-        return f"<Text '{self.offendingText}' shows up in both lt and local-lt>"
-
-
-def firstLinkTextFromElement(el: t.ElementT) -> str | None:
-    try:
-        texts = linkTextsFromElement(el)
-    except DuplicatedLinkText as e:
-        texts = e.allTexts
-    return texts[0] if len(texts) > 0 else None
+@t.overload
+def splitForValues(forValues: str) -> list[str]: ...
 
 
 @t.overload
-def splitForValues(forValues: str) -> list[str]:
-    ...
-
-
-@t.overload
-def splitForValues(forValues: None) -> None:
-    ...
+def splitForValues(forValues: None) -> None: ...
 
 
 def splitForValues(forValues: str | None) -> list[str] | None:
@@ -212,14 +164,21 @@ def scriptPath(*pathSegs: str) -> str:
     return path
 
 
+def docPath(doc: t.SpecT, *pathSegs: str) -> str | None:
+    ret = doc.inputSource.relative(*pathSegs)
+    if ret:
+        return str(ret)
+    return None
+
+
 def chrootPath(rootPath: str, path: str) -> str:
     rootPath = os.path.abspath(rootPath)
     path = os.path.abspath(path)
     if not path.startswith(rootPath):
         messages.die(
-            f"Attempted to access a file ({path}) outside the source document's directory ({rootPath}). See --allow-nonlocal-files."
+            f"Attempted to access a file ({path}) outside the source document's directory ({rootPath}). See --allow-nonlocal-files.",
         )
-        raise Exception()
+        raise Exception
     else:
         return path
 
@@ -232,8 +191,6 @@ def doEvery(s: float, action: t.Callable, lastTime: float | None = None) -> floa
     # If you want to take action on first call,
     # pass 0 as lastTime;
     # otherwise it won't take action until N seconds.
-    import time
-
     newTime = time.time()
     if lastTime is None:
         lastTime = newTime
@@ -241,3 +198,14 @@ def doEvery(s: float, action: t.Callable, lastTime: float | None = None) -> floa
         action()
         return newTime
     return lastTime
+
+
+if t.TYPE_CHECKING:
+    SafeIndexDefaultT = t.TypeVar("SafeIndexDefaultT")
+
+
+def safeIndex(coll: t.Sequence, needle: t.Any, default: SafeIndexDefaultT = None) -> int | SafeIndexDefaultT:  # type: ignore[assignment]
+    try:
+        return coll.index(needle)
+    except ValueError:
+        return default
