@@ -1,4 +1,6 @@
+from . import parser
 from .dom import (
+    DuplicatedLinkText,
     E,
     addClass,
     addOldIDs,
@@ -11,6 +13,10 @@ from .dom import (
     clearContents,
     closestAncestor,
     closestAttr,
+    collectAutolinks,
+    collectIds,
+    collectLinksWithSectionNames,
+    collectSyntaxHighlightables,
     createElement,
     dedupIDs,
     emptyText,
@@ -21,11 +27,12 @@ from .dom import (
     filterAncestors,
     find,
     findAll,
+    firstLinkTextFromElement,
     fixSurroundingTypography,
-    fixTypography,
     fixupIDs,
     foldWhitespace,
     hasAncestor,
+    hasAnyAttr,
     hasAttr,
     hasAttrs,
     hasChildElements,
@@ -40,6 +47,8 @@ from .dom import (
     isEmpty,
     isNormative,
     isOddNode,
+    isOnlyChild,
+    linkTextsFromElement,
     moveContents,
     nextSiblingElement,
     nextSiblingNode,
@@ -50,69 +59,39 @@ from .dom import (
     parseHTML,
     prependChild,
     previousElements,
+    printNodeTree,
     relevantHeadings,
     removeAttr,
     removeClass,
     removeNode,
-    replaceAwkwardCSSShorthands,
+    renameAttr,
     replaceContents,
-    replaceMacros,
+    replaceMacrosTextly,
     replaceNode,
     replaceWithContents,
     safeID,
     scopingElements,
     sectionName,
     serializeTag,
+    sortElements,
     tagName,
     textContent,
     textContentIgnoringDecorative,
+    transferAttributes,
     treeAttr,
     unescape,
     unfixTypography,
+    uniqueID,
     wrapContents,
 )
-from .serializer import Serializer
-from .parser import test
 from .parser import (
-    Comment,
+    ParseConfig,
     debugNodes,
-    EndTag,
-    Failure,
     initialDocumentParse,
-    isASCII,
-    isASCIIAlpha,
-    isASCIIAlphanum,
-    isASCIILowerAlpha,
-    isASCIIUpperAlpha,
-    isAttrNameChar,
-    isControl,
-    isDigit,
-    isHexDigit,
-    isNoncharacter,
-    isTagnameChar,
-    isWhitespace,
     nodesFromHtml,
-    parseAttribute,
-    parseCharRef,
-    parseComment,
-    parseDoctype,
-    parseEndTag,
-    ParseFailure,
-    parseQuotedAttrValue,
-    ParserNode,
-    parseScriptToEnd,
-    parseStartTag,
-    parseStyleToEnd,
-    parseTagName,
-    parseUnquotedAttrValue,
-    parseWhitespace,
-    parseXmpToEnd,
-    RawElement,
-    Result,
-    StartTag,
-    Stream,
-    Text,
-    WholeElement,
-    linesFromNodes,
+    parseLines,
+    parseText,
+    parseTitle,
     strFromNodes,
 )
+from .serializer import Serializer
