@@ -2,8 +2,8 @@ from __future__ import annotations
 
 import collections.abc
 import hashlib
+import html
 import re
-
 from collections import OrderedDict
 
 import html5lib
@@ -27,8 +27,6 @@ def flatten(arr: t.Iterable) -> t.Iterator:
 
 
 def unescape(string: str) -> str:
-    import html
-
     return html.unescape(string)
 
 
@@ -67,12 +65,7 @@ def escapeCSSIdent(val: str) -> str:
         ):
             ident += rf"\{code:x} "
         elif (
-            code >= 0x80
-            or code == 0x2D
-            or code == 0x5F
-            or 0x30 <= code <= 0x39
-            or 0x41 <= code <= 0x5A
-            or 0x61 <= code <= 0x7A
+            code >= 0x80 or code in (0x2D, 0x5F) or 0x30 <= code <= 0x39 or 0x41 <= code <= 0x5A or 0x61 <= code <= 0x7A
         ):
             ident += chr(code)
         else:
@@ -95,9 +88,8 @@ def validUrlUnit(char: str) -> bool:
     c = ord(char)
     if c < 0xA0:
         # ASCII range
-        if (
-            c == 0x21
-            or c == 0x24
+        return (
+            c in (0x21, 0x24)
             or 0x26 <= c <= 0x29
             or 0x2A <= c <= 0x3B
             or c == 0x3D
@@ -105,13 +97,11 @@ def validUrlUnit(char: str) -> bool:
             or c == 0x5F
             or 0x61 <= c <= 0x7A
             or c == 0x7E
-        ):
-            return True
-        return False
+        )
     else:
         if 0xD800 <= c <= 0xDFFF or 0xFDD0 <= c <= 0xFDEF:
             return False
-        if (c % 0xFFFF) in [0xFFFE, 0xFFFF]:
+        if (c % 0xFFFF) in [0xFFFE, 0xFFFF]:  # noqa: SIM103
             # Last two bytes are FFFE or FFFF
             return False
         return True
@@ -130,12 +120,13 @@ def textContent(el: t.ElementT, exact: bool = False) -> str:
 
 
 def textContentIgnoringDecorative(el: t.ElementT) -> str:
-    str = el.text or ""
-    for child in childElements(el):
-        if child.get("data-deco") is None:
-            str += textContentIgnoringDecorative(child)
-        str += child.tail or ""
-    return str
+    s = ""
+    for child in childNodes(el):
+        if isinstance(child, str):
+            s += child
+        elif child.get("data-deco") is None:
+            s += textContentIgnoringDecorative(child)
+    return s
 
 
 def innerHTML(el: t.ElementT | None) -> str:
@@ -156,12 +147,89 @@ def outerHTML(el: t.NodesT | None, literal: bool = False, with_tail: bool = Fals
     return t.cast(str, tostring(el, with_tail=with_tail, encoding="unicode"))
 
 
+def printNodeTree(node: t.NodeT | str) -> str:
+    # Debugging tool
+    if isinstance(node, str):
+        return "#text: " + repr(node)
+    if isOddNode(node):
+        return outerHTML(node)
+    if isinstance(node, list):
+        s = "[]"
+    else:
+        s = f"{serializeTag(node)}"
+    linesPerChild = [printNodeTree(child).split("\n") for child in childNodes(node, skipOddNodes=False)]
+    if linesPerChild:
+        for childLines in linesPerChild[:-1]:
+            childLines[0] = " ├" + childLines[0]
+            childLines[1:] = [" │" + line for line in childLines[1:]]
+            s += "\n" + "\n".join(childLines)
+        childLines = linesPerChild[-1]
+        childLines[0] = " ╰" + childLines[0]
+        childLines[1:] = ["  " + line for line in childLines[1:]]
+        s += "\n" + "\n".join(childLines)
+    return s
+
+
+def linkTextsFromElement(el: t.ElementT) -> list[str]:
+    texts: list[str] = []
+    if el.get("data-lt") == "":
+        return []
+    elif el.get("data-lt"):
+        rawText = el.get("data-lt", "")
+        if rawText in ["|", "||", "|||"]:
+            texts = [rawText]
+        else:
+            texts = [x.strip() for x in rawText.split("|")]
+    else:
+        if el.tag in ("dfn", "a"):
+            texts = [textContent(el).strip()]
+        elif el.tag in ("h2", "h3", "h4", "h5", "h6"):
+            textEl = find(".content", el)
+            if textEl is None:
+                textEl = el
+            texts = [textContent(textEl).strip()]
+    if el.get("data-local-lt"):
+        localTexts = [x.strip() for x in el.get("data-local-lt", "").split("|")]
+        for text in localTexts:
+            if text in texts:
+                # lt and local-lt both specify the same thing
+                raise DuplicatedLinkText(text, texts + localTexts, el)
+        texts += localTexts
+
+    texts = [re.sub(r"\s+", " ", x) for x in texts if x != ""]
+    return texts
+
+
+class DuplicatedLinkText(Exception):
+    def __init__(self, offendingText: str, allTexts: list[str], el: t.ElementT) -> None:
+        super().__init__()
+        self.offendingText = offendingText
+        self.allTexts = allTexts
+        self.el = el
+
+    def __unicode__(self) -> str:
+        return f"<Text '{self.offendingText}' shows up in both lt and local-lt>"
+
+
+def firstLinkTextFromElement(el: t.ElementT) -> str | None:
+    try:
+        texts = linkTextsFromElement(el)
+    except DuplicatedLinkText as e:
+        texts = e.allTexts
+    return texts[0] if len(texts) > 0 else None
+
+
 def serializeTag(el: t.ElementT) -> str:
     # Serialize *just* the opening tag for the element.
     # Use when you want to output the HTML,
     # but it might be a container with a lot of content.
     tag = "<" + el.tag
-    for n, v in el.attrib.items():
+    for n, v in sorted(el.attrib.items()):
+        # Don't output the bs-* attributes, they're added by BS
+        # and don't show up in the source, so it's confusing to
+        # print them.
+        if t.cast(str, n).startswith("bs-"):
+            continue
         tag += ' {n}="{v}"'.format(n=str(n), v=escapeAttr(str(v)))
     tag += ">"
     return tag
@@ -179,6 +247,10 @@ def foldWhitespace(text: str) -> str:
     return re.sub(r"(\s|\xa0)+", " ", text)
 
 
+def sortElements(el: t.Iterable[t.ElementT]) -> list[t.ElementT]:
+    return sorted(el, key=lambda x: (x.get("bs-line-number", ""), textContent(x)))
+
+
 def parseHTML(text: str) -> list[t.ElementT]:
     doc = html5lib.parse(text, treebuilder="lxml", namespaceHTMLElements=False)
     head = doc.getroot()[0]
@@ -239,27 +311,27 @@ def nextSiblingElement(el: t.ElementT) -> t.ElementT | None:
 
 
 @t.overload
-def appendChild(parent: t.ElementT, *els: t.NodesT, allowEmpty: t.Literal[False] = False) -> t.ElementT:
-    ...
+def appendChild(parent: t.ElementT, *els: t.NodesT, allowEmpty: t.Literal[False] = False) -> t.ElementT: ...
 
 
 @t.overload
-def appendChild(parent: t.ElementT, *els: t.NodesT, allowEmpty: bool) -> t.ElementT | None:
-    ...
+def appendChild(parent: t.ElementT, *els: t.NodesT, allowEmpty: bool) -> t.ElementT | None: ...
 
 
 def appendChild(parent: t.ElementT, *els: t.NodesT, allowEmpty: bool = False) -> t.ElementT | None:
     # Appends either text or an element.
     child: t.NodeT | None = None
+    parentLen = len(parent)
     for child in flatten(els):
         assert child is not None
         if isinstance(child, str):
-            if len(parent) > 0:
+            if parentLen > 0:
                 parent[-1].tail = (parent[-1].tail or "") + child
             else:
                 parent.text = (parent.text or "") + child
         else:
-            if len(parent) == 0 and parent.text is not None:
+            parentLen += 1
+            if parentLen == 0 and parent.text is not None:
                 # LXML "helpfully" assumes you meant to insert it before the text,
                 # and so moves the text into the element's tail when you append.
                 text, parent.text = parent.text, None
@@ -271,7 +343,8 @@ def appendChild(parent: t.ElementT, *els: t.NodesT, allowEmpty: bool = False) ->
                 # doesn't get moved into the appended child or anything.
                 parent.append(child)
     if child is None and not allowEmpty:
-        raise Exception("Empty child list appended without allowEmpty=True")
+        msg = "Empty child list appended without allowEmpty=True"
+        raise Exception(msg)
     if isElement(child):
         return child
     else:
@@ -325,6 +398,10 @@ def insertAfter(target: t.ElementT, *els: t.NodesT) -> t.ElementT:
 
 
 def removeNode(node: t.ElementT) -> t.ElementT:
+    # Kills the node *and* its children.
+    # If you just want to remove the node itself, and lift
+    # the contents up into its place, you wanna call
+    # replaceWithContents()
     parent = node.getparent()
     if parent is None:
         return node
@@ -348,6 +425,12 @@ def replaceNode(node: t.ElementT, *replacements: t.NodesT) -> t.NodesT | None:
     return None
 
 
+def transferAttributes(source: t.ElementT, target: t.ElementT) -> t.ElementT:
+    for k, v in source.attrib.items():
+        target.set(k, v)
+    return target
+
+
 def appendContents(el: t.ElementT, container: t.ElementT | t.Iterable[t.NodesT]) -> t.ElementT:
     # Accepts either an iterable *or* a container element
     if isElement(container):
@@ -411,6 +494,31 @@ def sectionName(doc: t.SpecT, el: t.ElementT) -> str | None:
     return textContent(h)
 
 
+def collectLinksWithSectionNames(
+    doc: t.SpecT,
+    root: t.ElementT | None = None,
+    links: dict[t.ElementT, str | None] | None = None,
+    name: str | None = "Unnamed section",
+) -> dict[t.ElementT, str | None]:
+    # Tree-walk to collect all links, and compute their section names
+    # along the way.
+    if links is None:
+        links = {}
+    if root is None:
+        root = doc.body
+    for child in childElements(root):
+        if tagName(child) == "a":
+            links[child] = name
+            continue
+        if tagName(child) in ("h1", "h2", "h3", "h4", "h5", "h6"):
+            if hasClass(doc, child, "no-ref"):
+                name = None
+            else:
+                name = textContent(child)
+        collectLinksWithSectionNames(doc, child, links, name)
+    return links
+
+
 def scopingElements(startEl: t.ElementT, tags: list[str]) -> t.Generator[t.ElementT, None, None]:
     # Elements that could form a "scope" for the startEl
     # Ancestors, and preceding siblings of ancestors.
@@ -440,8 +548,6 @@ def previousElements(startEl: t.ElementT, tag: str | None = None, *tags: str) ->
 
 
 def childElements(parentEl: t.ElementT, oddNodes: bool = False) -> t.Generator[t.ElementT, None, None]:
-    if len(parentEl) == 0:
-        return
     tag = None if oddNodes else "*"
     yield from parentEl.iterchildren(tag)
 
@@ -456,7 +562,12 @@ def ancestorElements(el: t.ElementT, self: bool = False) -> t.Generator[t.Elemen
     yield from el.iterancestors()
 
 
-def childNodes(parentEl: t.ElementishT, clear: bool = False, skipOddNodes: bool = True) -> list[t.NodeT]:
+def childNodes(
+    parentEl: t.ElementishT,
+    clear: bool = False,
+    skipOddNodes: bool = True,
+    mergeText: bool = False,
+) -> list[t.NodeT]:
     """
     This function returns all the nodes in a parent element in the DOM sense,
     mixing text nodes (strings) and other nodes together
@@ -470,7 +581,7 @@ def childNodes(parentEl: t.ElementishT, clear: bool = False, skipOddNodes: bool
     In other words, the following is a no-op:
 
     ```
-    appendChild(parentEl, *childNodes(parentEl, clear=True), allowEmpty=True)
+    appendChild(parentEl, *childNodes(parentEl, clear=True, skipOddNodes=False), allowEmpty=True)
     ```
 
     Using clear=True is required if you're going to be modifying the element or its children,
@@ -480,20 +591,32 @@ def childNodes(parentEl: t.ElementishT, clear: bool = False, skipOddNodes: bool
 
     skipOddNodes ensures that the return value will only be text and Element nodes;
     if it's false, there might be comments, PIs, etc.
+
+    mergeText merges adjacent text nodes in the output, even if they weren't adjacent
+    in the source document (such as if a skipped odd node separated them).
     """
     ret: list[t.NodeT] = []
 
+    def append(nodes: list[t.NodeT], node: t.NodeT) -> None:
+        if not mergeText:
+            nodes.append(node)
+            return
+        if isinstance(node, str) and len(nodes) > 0 and isinstance(nodes[-1], str):
+            nodes[-1] += node
+        else:
+            nodes.append(node)
+
     if isinstance(parentEl, list):
         for c in parentEl:
             if isinstance(c, str):
-                ret.append(c)
+                append(ret, c)
                 continue
             if skipOddNodes and isOddNode(c):
                 pass
             else:
-                ret.append(c)
+                append(ret, c)
             if not emptyText(c.tail, wsAllowed=False):
-                ret.append(t.cast(str, c.tail))
+                append(ret, t.cast(str, c.tail))
                 if clear:
                     c.tail = None
         if clear:
@@ -501,16 +624,16 @@ def childNodes(parentEl: t.ElementishT, clear: bool = False, skipOddNodes: bool
         return ret
 
     if not emptyText(parentEl.text, wsAllowed=False):
-        ret.append(t.cast(str, parentEl.text))
+        append(ret, t.cast(str, parentEl.text))
         if clear:
             parentEl.text = None
     for c in childElements(parentEl, oddNodes=True):
         if skipOddNodes and isOddNode(c):
             pass
         else:
-            ret.append(c)
+            append(ret, c)
         if not emptyText(c.tail, wsAllowed=False):
-            ret.append(t.cast(str, c.tail))
+            append(ret, t.cast(str, c.tail))
             if clear:
                 c.tail = None
     if clear:
@@ -597,12 +720,23 @@ def removeAttr(el: t.ElementT, *attrNames: str) -> t.ElementT:
     return el
 
 
-def hasAttr(el: t.ElementT, *attrNames: str) -> bool:
+def renameAttr(el: t.ElementT, oldAttr: str, newAttr: str) -> t.ElementT:
+    # If oldAttr exists on the element, moves its value to newAttr
+    # and deletes oldAttr. Silently does nothing if oldAttr doesn't exist.
+    val = el.get(oldAttr)
+    if val is not None:
+        el.set(newAttr, val)
+        removeAttr(el, oldAttr)
+    return el
+
+
+def hasAnyAttr(el: t.ElementT, *attrNames: str) -> bool:
     # Returns True if the element has at least one of the named attributes
-    for attrName in attrNames:
-        if attrName in el.attrib:
-            return True
-    return False
+    return any(attrName in el.attrib for attrName in attrNames)
+
+
+def hasAttr(el: t.ElementT, attrName: str) -> bool:
+    return attrName in el.attrib
 
 
 def hasAttrs(el: t.ElementT) -> bool:
@@ -661,17 +795,14 @@ def isNodes(nodes: t.Any) -> t.TypeGuard[t.NodesT]:
         return True
     if not isinstance(nodes, list):
         return False
-    for child in nodes:
-        if not isNodes(child):
-            return False
-    return True
+    return all(isNodes(child) for child in nodes)
 
 
 def isOddNode(node: t.Any) -> bool:
     # Something other than an element node or string.
     if isinstance(node, str):
         return False
-    if isElement(node):
+    if isElement(node):  # noqa: SIM103
         return False
     return True
 
@@ -739,13 +870,11 @@ def hasOnlyChild(el: t.ElementT, wsAllowed: bool = True) -> t.ElementT | None:
     return single
 
 
-def fixTypography(text: str) -> str:
-    # Replace straight aposes with curly quotes for possessives and contractions.
-    text = re.sub(r"([\w])'([\w])", r"\1’\2", text)
-    text = re.sub(r"(</[\w]+>)'([\w])", r"\1’\2", text)
-    # Fix line-ending em dashes, or --, by moving the previous line up, so no space.
-    text = re.sub(r"([^<][^!])(—|--)\r?\n\s*(\S)", r"\1—<wbr>\3", text)
-    return text
+def isOnlyChild(el: t.ElementT, wsAllowed: bool = True) -> bool:
+    parent = parentElement(el)
+    if parent is None:
+        return True
+    return hasOnlyChild(parent, wsAllowed) is not None
 
 
 def fixSurroundingTypography(el: t.ElementT) -> t.ElementT:
@@ -783,11 +912,10 @@ def hashContents(el: t.ElementT) -> str:
     return hashlib.md5(text).hexdigest()[0:8]
 
 
-def replaceMacros(text: str, macros: t.Mapping[str, str]) -> str:
-    # `macros` is a dict of {lowercaseMacroName => replacementText}
-    # Macro syntax is [FOO], where FOO is /[A-Z0-9-]+/
-    # If written as [FOO?], failure to find a matching macro just replaced it with nothing;
-    # otherwise, it throws a fatal error.
+def replaceMacrosTextly(text: str, macros: t.Mapping[str, str], context: str) -> str:
+    # Same as replaceMacros(), but does the substitution
+    # directly on the text, rather than relying on the
+    # html parser to have preparsed the macro syntax
     def macroReplacer(match: re.Match) -> str:
         fullText = t.cast(str, match.group(0))
         innerText = match.group(2).lower() or ""
@@ -798,36 +926,20 @@ def replaceMacros(text: str, macros: t.Mapping[str, str]) -> str:
         if fullText.startswith("[["):
             # Actually a biblio link
             return fullText
-        if re.match(r"[\d-]+$", innerText):
-            # No refs are all-digits (this is probably JS code, or a regex/grammar).
-            return fullText
         if innerText in macros:
             # For some reason I store all the macros in lowercase,
             # despite requiring them to be spelled with uppercase.
-            return str(macros[innerText])
+            return macros[innerText]
         # Nothing has matched, so start failing the macros.
         if optional:
             return ""
-        die(f"Found unmatched text macro {fullText}. Correct the macro, or escape it with a leading backslash.")
+        die(
+            f"Found unmatched text macro {fullText}. Correct the macro, or escape it with a leading backslash.",
+            lineNum=context,
+        )
         return fullText
 
-    return re.sub(r"(\\|\[)?\[([A-Z0-9-]+)(\??)\]", macroReplacer, text)
-
-
-def replaceAwkwardCSSShorthands(text: str) -> str:
-    # Replace the ''maybe link'' shortcuts.
-    # They'll survive the HTML parser,
-    # but the current shorthand-recognizer code won't find them if they contain an element.
-    # (The other shortcuts are "atomic" and can't contain elements.)
-    def replaceMaybe(match: re.Match) -> str:
-        syntaxAttr = escapeAttr(match.group(0))
-        escape, text = match.groups()
-        if escape:
-            return escapeHTML(match.group(0)[1:])
-        return f"<fake-maybe-placeholder bs-autolink-syntax='{syntaxAttr}'>{text}</fake-maybe-placeholder>"
-
-    text = re.sub(r"(\\)?''([^=\n]+?)''", replaceMaybe, text)
-    return text
+    return re.sub(r"(\\|\[)?\[([A-Z\d-]*[A-Z][A-Z\d-]*)(\??)\]", macroReplacer, text)
 
 
 def fixupIDs(doc: t.SpecT, els: t.Iterable[t.ElementT]) -> None:
@@ -836,12 +948,20 @@ def fixupIDs(doc: t.SpecT, els: t.Iterable[t.ElementT]) -> None:
 
 
 def safeID(doc: t.SpecT, id: str) -> str:
+    # Converts generated IDs into author-specified versions,
+    # for when an ID isn't otherwise settable by the author.
     trans = doc.md.translateIDs
     if id in trans:
         return trans[id]
     return id
 
 
+def uniqueID(*s: str) -> str:
+    # Turns a unique string into a more compact (and ID-safe)
+    # hashed string
+    return hashlib.md5("".join(s).encode("utf-8")).hexdigest()[:8]
+
+
 def addOldIDs(els: t.Iterable[t.ElementT]) -> None:
     for el in els:
         oldIdAttr = el.get("oldids")
@@ -849,13 +969,11 @@ def addOldIDs(els: t.Iterable[t.ElementT]) -> None:
             continue
         oldIDs = [id.strip() for id in oldIdAttr.strip().split(",")]
         for oldID in oldIDs:
-            appendChild(el, E.span({"id": oldID}))
+            prependChild(el, E.span({"id": oldID, "class": "bs-old-id"}))
         removeAttr(el, "oldids")
 
 
 def dedupIDs(doc: t.SpecT) -> None:
-    import itertools as iter
-
     ids: OrderedDict[str, list[t.ElementT]] = OrderedDict()
     for el in findAll("[id]", doc):
         ids.setdefault(t.cast(str, el.get("id")), []).append(el)
@@ -867,41 +985,44 @@ def dedupIDs(doc: t.SpecT) -> None:
         if re.match(r"issue-[0-9a-fA-F]{8}$", dupeId):
             # Don't warn about issues, it's okay if they have the same ID because they're identical text.
             warnAboutDupes = False
-        ints = iter.count(1)
+        if dupeId.startswith("ref-for-"):
+            warnAboutDupes = False
+        complaintEls = []
+        if warnAboutDupes:
+            for el in els:
+                if el.get("data-silently-dedup") is not None:
+                    continue
+                complaintEls.append(el)
+        # Now dedup everything left in the list after the first one.
+        dedupIndex = 1
         for el in els[1:]:
-            # If I registered an alternate ID, try to use that.
             if el.get("data-alternate-id"):
                 altId = el.get("data-alternate-id")
                 assert altId is not None
                 if altId not in ids:
                     el.set("id", safeID(doc, altId))
-                    ids.setdefault(altId, []).append(el)
+                    ids[altId] = [el]
+                    complaintEls.remove(el)
                     continue
-            if el.get("data-silently-dedup") is not None:
-                warnAboutDupes = False
-            if dupeId.startswith("ref-for-"):
-                warnAboutDupes = False
-            # Try to de-dup the id by appending an integer after it.
-            if warnAboutDupes:
-                warn(
-                    f"Multiple elements have the same ID '{dupeId}'.\nDeduping, but this ID may not be stable across revisions.",
-                    el=el,
-                )
-            for x in ints:
-                altId = "{}{}".format(dupeId, circledDigits(x))
-                if altId not in ids:
-                    el.set("id", safeID(doc, altId))
-                    ids.setdefault(altId, []).append(el)
-                    break
+            altId = f"{dupeId}{circledDigits(dedupIndex)}"
+            while altId in ids:
+                dedupIndex += 1
+                altId = f"{dupeId}{circledDigits(dedupIndex)}"
+            el.set("id", safeID(doc, altId))
+            ids[altId] = [el]
+
+        if len(complaintEls) > 1:
+            complaintDetails = [f"<{el.tag}> on line {approximateLineNumber(el)}" for el in complaintEls]
+            warn(
+                f"Multiple elements have the same id '{dupeId}':\n  {', '.join(complaintDetails)}\nDeduping, but this ID may not be stable across revisions.",
+            )
 
 
 def approximateLineNumber(el: t.ElementT, setIntermediate: bool = True) -> str | None:
-    if el.get("line-number"):
-        return el.get("line-number")
+    if el.get("bs-line-number"):
+        return el.get("bs-line-number")
     parent = parentElement(el)
     if not isElement(parent):
-        if el.tag == "html":
-            return None
         return None
     approx = approximateLineNumber(parent, setIntermediate=setIntermediate)
     if approx is None:
@@ -909,7 +1030,7 @@ def approximateLineNumber(el: t.ElementT, setIntermediate: bool = True) -> str |
     if approx[0].isdigit():
         approx = "~" + approx
     if setIntermediate:
-        el.set("line-number", approx)
+        el.set("bs-line-number", approx)
     return approx
 
 
@@ -924,6 +1045,39 @@ def circledDigits(num: int) -> str:
     return result
 
 
+def collectIds(el: t.ElementT, ids: dict[str, t.ElementT] | None = None) -> dict[str, t.ElementT]:
+    if ids is None:
+        ids = {}
+    id = el.get("id", "")
+    if id and id not in ids:
+        ids[id] = el
+    for child in childElements(el):
+        collectIds(child, ids)
+    return ids
+
+
+def collectAutolinks(el: t.ElementT, links: list[t.ElementT] | None = None) -> list[t.ElementT]:
+    if links is None:
+        links = []
+    if tagName(el) == "a" and not hasAttr(el, "href") and el.get("data-link-type", "") != "biblio":
+        links.append(el)
+    elif tagName(el) == "bs-link":
+        links.append(el)
+    for child in childElements(el):
+        collectAutolinks(child, links)
+    return links
+
+
+def collectSyntaxHighlightables(el: t.ElementT, els: list[t.ElementT] | None = None) -> list[t.ElementT]:
+    if els is None:
+        els = []
+    if tagName(el) in ("xmp", "pre", "code"):
+        els.append(el)
+    for child in childElements(el):
+        collectSyntaxHighlightables(child, els)
+    return els
+
+
 def createElement(tag: str, attrs: t.Mapping[str, str | None] | None = None, *children: t.NodesT | None) -> t.ElementT:
     if attrs is None:
         attrs = {}
@@ -937,15 +1091,17 @@ if t.TYPE_CHECKING:
 
     class ElementCreatorFnT(t.Protocol):
         def __call__(
-            self, attrsOrChild: t.Mapping[str, str | None] | t.NodesT | None = None, *children: t.NodesT | None
-        ) -> t.ElementT:
-            ...
+            self,
+            attrsOrChild: t.Mapping[str, str | None] | t.NodesT | None = None,
+            *children: t.NodesT | None,
+        ) -> t.ElementT: ...
 
 
 class ElementCreationHelper:
     def __getattr__(self, name: str) -> ElementCreatorFnT:
         def _creater(
-            attrsOrChild: t.Mapping[str, str | None] | t.NodesT | None = None, *children: t.NodesT | None
+            attrsOrChild: t.Mapping[str, str | None] | t.NodesT | None = None,
+            *children: t.NodesT | None,
         ) -> t.ElementT:
             if isNodes(attrsOrChild):
                 return createElement(name, None, attrsOrChild, *children)
