@@ -3,8 +3,8 @@ from __future__ import annotations
 import io
 import itertools
 
-from . import dom
 from .. import t
+from . import dom
 
 if t.TYPE_CHECKING:
     WriterFn: t.TypeAlias = t.Callable[[str], t.Any]
@@ -49,7 +49,7 @@ class Serializer:
             "progress",
             "math",
             "[]",
-        ]
+        ],
     )
     rawEls = frozenset(["xmp", "script", "style"])
     voidEls = frozenset(
@@ -70,7 +70,7 @@ class Serializer:
             "source",
             "track",
             "wbr",
-        ]
+        ],
     )
     omitEndTagEls = frozenset(
         [
@@ -88,10 +88,10 @@ class Serializer:
             "html",
             "head",
             "body",
-        ]
+        ],
     )
 
-    def __init__(self, opaqueElements: t.Iterable[str], blockElements: t.Iterable[str]):
+    def __init__(self, opaqueElements: t.Iterable[str], blockElements: t.Iterable[str]) -> None:
         self.opaqueEls = frozenset(opaqueElements)
         self.blockEls = frozenset(blockElements)
 
@@ -110,30 +110,19 @@ class Serializer:
             return n.partition("}")[2]
         return n
 
-    def groupIntoBlocks(self, nodes: t.Iterable[t.NodeT]) -> t.Generator[Nodes, None, None]:
-        nonBlockNodes: list[str | t.ElementT] = []
+    def groupIntoBlocks(self, nodes: t.Iterable[t.NodeT]) -> Blocks:
+        # Turns the iterable into a list of block elements and inline "spans"
+        # (a list of text and inline elements).
+        blocks: Blocks = []
         for node in nodes:
             if self.isElement(node) and self.isBlockElement(node.tag):
-                if nonBlockNodes:
-                    yield nonBlockNodes
-                    nonBlockNodes = []
-                yield node
-                continue
+                blocks.append(node)
             else:
-                nonBlockNodes.append(node)
-        if nonBlockNodes:
-            yield nonBlockNodes
-
-    def fixWS(self, text: str) -> str:
-        import string
-
-        t1 = text.lstrip(string.whitespace)
-        if text != t1:
-            t1 = " " + t1
-        t2 = t1.rstrip(string.whitespace)
-        if t1 != t2:
-            t2 = t2 + " "
-        return t2
+                if len(blocks) == 0 or not isinstance(blocks[-1], list):
+                    blocks.append([])
+                lastBlock = t.cast("list[str | t.ElementT]", blocks[-1])
+                lastBlock.append(node)
+        return blocks
 
     def startTag(self, tag: str, el: Nodes, write: WriterFn) -> None:
         if isinstance(el, list):
@@ -145,9 +134,9 @@ class Serializer:
         strs = []
         strs.append("<" + tag)
         for attrName, attrVal in sorted(el.items()):
-            # Omit any attrs with a bs-* prefix, as they're
-            # used for Bikeshed-internal purposes.
             if str(attrName).startswith("bs-"):
+                # Skip bs- prefixed attributes, as they're used
+                # for Bikeshed-internal purposes.
                 continue
             if attrVal == "":
                 strs.append(" " + self.unfuckName(str(attrName)))
@@ -187,7 +176,7 @@ class Serializer:
         if el.tag in ["dt", "dd"]:
             if nextEl is None:
                 return False
-            if self.isElement(nextEl) and nextEl.tag in ["dt", "dd"]:
+            if self.isElement(nextEl) and nextEl.tag in ["dt", "dd"]:  # noqa: SIM103
                 return False
             return True
         return False
@@ -201,11 +190,21 @@ class Serializer:
         write(" " * indent)
         self.startTag(tag, el, write)
 
-    def _writeRawElement(self, tag: str, el: t.ElementT, write: WriterFn) -> None:
+    def _writeRawElement(self, tag: str, el: t.ElementT, write: WriterFn, indent: int) -> None:
+        if tag == "script" and el.get("src") is not None:
+            # A *linking* script, doesn't need to be treated specially.
+            write(" " * indent)
+            self.startTag(tag, el, write)
+            self.endTag(tag, write)
+            return
+
+        # Otherwise, dedent completely, since I don't know if indenting
+        # is significant for the element.
         self.startTag(tag, el, write)
-        for node in dom.childNodes(el):
+        for node in dom.childNodes(el, mergeText=True):
             if self.isElement(node):
-                raise Exception(f"Somehow a CDATA element got an element child:\n{dom.outerHTML(el)}")
+                msg = f"Somehow a CDATA element got an element child:\n{dom.outerHTML(el)}"
+                raise Exception(msg)
             else:
                 assert isinstance(node, str)
                 write(node)
@@ -213,7 +212,7 @@ class Serializer:
 
     def _writeOpaqueElement(self, tag: str, el: t.ElementT, write: WriterFn, indent: int) -> None:
         self.startTag(tag, el, write)
-        for node in dom.childNodes(el):
+        for node in dom.childNodes(el, mergeText=True):
             if self.isElement(node):
                 self._serializeEl(node, write, indent=indent, pre=True)
             else:
@@ -223,12 +222,12 @@ class Serializer:
 
     def _writeInlineElement(self, tag: str, el: Nodes, write: WriterFn, inline: bool) -> None:
         self.startTag(tag, el, write)
-        for node in dom.childNodes(el):
+        for node in dom.childNodes(el, mergeText=True):
             if self.isElement(node):
                 self._serializeEl(node, write, inline=inline)
             else:
                 assert isinstance(node, str)
-                write(dom.escapeHTML(self.fixWS(node)))
+                write(dom.escapeHTML(node))
         self.endTag(tag, write)
 
     def _blocksFromChildren(self, children: t.Iterable[t.NodeT]) -> Nodes:
@@ -239,12 +238,24 @@ class Serializer:
         Figure out what sort of contents the block has,
         so we know what serialization strategy to use.
         """
+
+        # Early exit for a totally empty element (including just ws)
+        # This will fail if there are odd nodes, even tho they're not serialized.
         if self.isElement(el) and len(el) == 0 and dom.emptyText(el.text):
             return "empty", None
-        children = dom.childNodes(el, clear=True)
+
+        # See if there are any block children
+        children = dom.childNodes(el, clear=True, mergeText=True)
         for child in children:
             if self.isElement(child) and self.isBlockElement(child.tag):
                 return "blocks", self._blocksFromChildren(children)
+
+        # See if it's an empty element *when ignoring odd nodes*,
+        # which childNodes() does by default
+        if all(isinstance(x, str) and dom.emptyText(x) for x in children):
+            return "empty", None
+
+        # Otherwise, it's all inline content.
         return "inlines", children
 
     def _writeBlockElement(self, tag: str, el: t.ElementT, write: WriterFn, indent: int, nextEl: Nodes | None) -> None:
@@ -308,7 +319,7 @@ class Serializer:
             self._writeVoidElement(tag, el, write, indent)
         elif self.isRawElement(tag):
             assert self.isElement(el)
-            self._writeRawElement(tag, el, write)
+            self._writeRawElement(tag, el, write, indent)
         elif pre or self.isOpaqueElement(tag):
             assert self.isElement(el)
             self._writeOpaqueElement(tag, el, write, indent)
