@@ -1,6 +1,7 @@
 from __future__ import annotations
 
-from . import config, h, messages as m, t
+from . import config, h, t
+from . import messages as m
 
 
 def processHeadings(doc: t.SpecT, scope: str = "doc") -> None:
@@ -21,8 +22,8 @@ def processHeadings(doc: t.SpecT, scope: str = "doc") -> None:
     addHeadingBonuses(headings)
     for el in headings:
         h.addClass(doc, el, "settled")
-    if scope == "all" and doc.md.group in config.megaGroups["priv-sec"]:
-        checkPrivacySecurityHeadings(h.findAll(".heading", doc))
+    if scope == "all" and doc.doctype.group.privSec:
+        checkPrivacySecurityHeadings(doc, h.findAll(".heading", doc))
 
 
 def resetHeadings(headings: list[t.ElementT]) -> None:
@@ -46,16 +47,16 @@ def addHeadingIds(doc: t.SpecT, headings: list[t.ElementT]) -> None:
     if len(neededIds) == 0:
         pass
     elif 1 <= len(neededIds) <= 5:
-        for el in neededIds:
+        for el in h.sortElements(neededIds):
             m.warn(f"The heading '{h.textContent(el)}' needs a manually-specified ID.", el=el)
     else:
         m.warn(
             "You should manually provide IDs for your headings:\n"
-            + "\n".join("  " + h.textContent(el) for el in neededIds)
+            + "\n".join("  " + h.textContent(el) for el in h.sortElements(neededIds)),
         )
 
 
-def checkPrivacySecurityHeadings(headings: list[t.ElementT]) -> None:
+def checkPrivacySecurityHeadings(doc: t.SpecT, headings: list[t.ElementT]) -> None:
     security = False
     privacy = False
     for header in headings:
@@ -65,7 +66,7 @@ def checkPrivacySecurityHeadings(headings: list[t.ElementT]) -> None:
             security = True
         if "privacy" in text and "considerations" in text:
             privacy = True
-        if "security" in text and "privacy" in text and "considerations" in text:
+        if "security" in text and "privacy" in text and "considerations" in text and doc.doctype.org.name == "W3C":
             m.warn(
                 "W3C policy requires Privacy Considerations and Security Considerations to be separate sections, but you appear to have them combined into one.",
                 el=header,
@@ -74,15 +75,15 @@ def checkPrivacySecurityHeadings(headings: list[t.ElementT]) -> None:
             return
     if not security and not privacy:
         m.warn(
-            "This specification has neither a 'Security Considerations' nor a 'Privacy Considerations' section. Please consider adding both, see https://w3ctag.github.io/security-questionnaire/."
+            "This specification has neither a 'Security Considerations' nor a 'Privacy Considerations' section. Please consider adding both, see https://w3ctag.github.io/security-questionnaire/.",
         )
     elif not security:
         m.warn(
-            "This specification does not have a 'Security Considerations' section. Please consider adding one, see https://w3ctag.github.io/security-questionnaire/."
+            "This specification does not have a 'Security Considerations' section. Please consider adding one, see https://w3ctag.github.io/security-questionnaire/.",
         )
     elif not privacy:
         m.warn(
-            "This specification does not have a 'Privacy Considerations' section. Please consider adding one, see https://w3ctag.github.io/security-questionnaire/."
+            "This specification does not have a 'Privacy Considerations' section. Please consider adding one, see https://w3ctag.github.io/security-questionnaire/.",
         )
 
 
