@@ -1,18 +1,54 @@
 from __future__ import annotations
 
+import dataclasses
 import re
 
 import widlparser
 
-from . import config, h, messages as m, t
+from . import config, h, t
+from . import messages as m
 
 
+@dataclasses.dataclass
 class IDLUI:
+    lineStart: int | None
+    parseContext: str | None = None
+
+    @staticmethod
+    def fromEl(el: t.ElementT) -> IDLUI:
+        lineNum = el.get("bs-line-number", None)
+        try:
+            if lineNum is None:
+                num = None
+            elif ":" in lineNum:
+                lineNum, _, _ = lineNum.partition(":")
+                num = int(lineNum)
+            else:
+                num = int(lineNum)
+        except ValueError:
+            lineNum = None
+        context = el.get("bs-parse-context", None)
+        return IDLUI(lineStart=num, parseContext=context)
+
+    def formatLineNum(self, localNum: int) -> str:
+        ret = str((self.lineStart or 0) + localNum)
+        if self.parseContext:
+            ret += f" of {self.parseContext}"
+        return ret
+
+    def formatMessage(self, msg: str) -> m.MessageOptions:
+        match = re.match(r".*?LINE: (\d+) - (.*)", msg.strip())
+        if not match:
+            return m.MessageOptions(msg.rstrip())
+        localLineNum = int(match[1])
+        detail = match[2]
+        return m.MessageOptions(detail, lineNum=self.formatLineNum(localLineNum))
+
     def warn(self, msg: str) -> None:
-        m.die(msg.rstrip())
+        self.formatMessage(msg).die()
 
     def note(self, msg: str) -> None:
-        m.warn(msg.rstrip())
+        self.formatMessage(msg).warn()
 
 
 class IDLSilent:
@@ -31,7 +67,9 @@ class DebugMarker(widlparser.protocols.Marker):
     # Debugging tool for IDL markup
 
     def markup_construct(
-        self, text: str, construct: widlparser.Construct
+        self,
+        text: str,
+        construct: widlparser.Construct,
     ) -> MarkupReturnT:  # pylint: disable=unused-argument
         return (
             "<construct-" + construct.idl_type + ">",
@@ -69,36 +107,51 @@ class DebugMarker(widlparser.protocols.Marker):
         )
 
     def markup_type_name(
-        self, text: str, construct: widlparser.Construct
+        self,
+        text: str,
+        construct: widlparser.Construct,
     ) -> MarkupReturnT:  # pylint: disable=unused-argument
         return ('<TYPE-NAME idlType="' + construct.idl_type + '">', "</TYPE-NAME>")
 
     def markup_name(
-        self, text: str, construct: widlparser.Construct
+        self,
+        text: str,
+        construct: widlparser.Construct,
     ) -> MarkupReturnT:  # pylint: disable=unused-argument
         return ('<NAME idlType="' + construct.idl_type + '">', "</NAME>")
 
     def markup_keyword(
-        self, text: str, construct: widlparser.Construct
+        self,
+        text: str,
+        construct: widlparser.Construct,
     ) -> MarkupReturnT:  # pylint: disable=unused-argument
         return ('<KEYWORD idlType="' + construct.idl_type + '">', "</KEYWORD>")
 
     def markup_enum_value(
-        self, text: str, construct: widlparser.Construct
+        self,
+        text: str,
+        construct: widlparser.Construct,
     ) -> MarkupReturnT:  # pylint: disable=unused-argument
         return ('<ENUM-VALUE for="' + t.cast(str, construct.name) + '">', "</ENUM-VALUE>")
 
+    def encode(self, text: str) -> str:
+        return h.escapeHTML(text)
+
 
 class IDLMarker(widlparser.protocols.Marker):
     def markup_construct(
-        self, text: str, construct: widlparser.Construct
+        self,
+        text: str,
+        construct: widlparser.Construct,
     ) -> MarkupReturnT:  # pylint: disable=unused-argument
         # Fires for every 'construct' in the WebIDL.
         # Some things are "productions", not "constructs".
         return (None, None)
 
     def markup_type(
-        self, text: str, construct: widlparser.Construct
+        self,
+        text: str,
+        construct: widlparser.Construct,
     ) -> MarkupReturnT:  # pylint: disable=unused-argument
         # Fires for entire type definitions.
         # It'll contain keywords or names, or sometimes more types.
@@ -107,22 +160,30 @@ class IDLMarker(widlparser.protocols.Marker):
         return (None, None)
 
     def markup_primitive_type(
-        self, text: str, construct: widlparser.Construct
+        self,
+        text: str,
+        construct: widlparser.Construct,
     ) -> MarkupReturnT:  # pylint: disable=unused-argument
         return ("<a data-link-type=interface>", "</a>")
 
     def markup_string_type(
-        self, text: str, construct: widlparser.Construct
+        self,
+        text: str,
+        construct: widlparser.Construct,
     ) -> MarkupReturnT:  # pylint: disable=unused-argument
         return ("<a data-link-type=interface>", "</a>")
 
     def markup_buffer_type(
-        self, text: str, construct: widlparser.Construct
+        self,
+        text: str,
+        construct: widlparser.Construct,
     ) -> MarkupReturnT:  # pylint: disable=unused-argument
         return ("<a data-link-type=interface>", "</a>")
 
     def markup_object_type(
-        self, text: str, construct: widlparser.Construct
+        self,
+        text: str,
+        construct: widlparser.Construct,
     ) -> MarkupReturnT:  # pylint: disable=unused-argument
         return ("<a data-link-type=interface>", "</a>")
 
@@ -140,10 +201,12 @@ class IDLMarker(widlparser.protocols.Marker):
             # the "value" is a DOMString attr
             member = construct.parent.member  # type: ignore
             if hasattr(member, "rest"):
-                type = member.rest.type
+                memberType = member.rest.type
             elif hasattr(member, "attribute"):
-                type = member.attribute.type
-            typeName = str(type).strip()
+                memberType = member.attribute.type
+            else:
+                assert False, "unreachable"
+            typeName = str(memberType).strip()
             if typeName.endswith("?"):
                 typeName = typeName[:-1]
             return (
@@ -160,10 +223,12 @@ class IDLMarker(widlparser.protocols.Marker):
             )
 
         if construct.idl_type == "constructor":
-            # This shows up for the method name in a [NamedConstructor] extended attribute.
-            # The "NamedConstructor" Name already got markup up, so ignore this one.
-            return (None, None)
-
+            interfaceName = construct.parent.name
+            methodName = construct.name
+            return (
+                f"<idl data-idl-type=constructor data-idl-for='{interfaceName}' data-lt='{methodName}()'>",
+                "</idl>",
+            )
         return ('<a data-link-type="idl-name">', "</a>")
 
     def markup_keyword(self, text: str, construct: widlparser.Construct) -> MarkupReturnT:
@@ -181,7 +246,7 @@ class IDLMarker(widlparser.protocols.Marker):
             # Otherwise, you *can* point to/dfn stringification behavior if you want.
             return (
                 "<idl data-export data-idl-type=dfn data-idl-for='{0}' data-lt='stringification behavior' id='{0}-stringification-behavior'>".format(
-                    construct.parent.full_name
+                    construct.parent.full_name,
                 ),
                 "</idl>",
             )
@@ -202,7 +267,9 @@ class IDLMarker(widlparser.protocols.Marker):
         return (None, None)
 
     def markup_name(
-        self, text: str, construct: widlparser.Construct
+        self,
+        text: str,
+        construct: widlparser.Construct,
     ) -> MarkupReturnT:  # pylint: disable=unused-argument
         # Fires for defining names: method names, arg names, interface names, etc.
         idlType = construct.idl_type
@@ -210,11 +277,14 @@ class IDLMarker(widlparser.protocols.Marker):
             return (None, None)
 
         if idlType == "constructor":
-            # the [Constructor] extended attr, now deprecated
-            m.die(
-                f"The [Constructor] extended attribute (on {construct.parent.name}) is deprecated, please switch to a constructor() method."
-            )
-            return (None, None)
+            # one of the constructor extended attr, now deprecated
+            if text == "Constructor":
+                m.die(
+                    f"The [Constructor] extended attribute (on {construct.parent.name}) is deprecated, please switch to a constructor() method.",
+                )
+                return (None, None)
+            # Otherwise it's [LegacyNamedConstructor], which is allowed
+            return ("<a data-link-type=extended-attribute data-link-for>", "</a>")
 
         if idlType == "argument" and construct.parent.idl_type == "constructor":
             # Don't mark up the arguments to [Constructor] either
@@ -243,10 +313,10 @@ class IDLMarker(widlparser.protocols.Marker):
                 readonly = "data-readonly"
             else:
                 readonly = ""
-            extraParameters = '{} data-type="{}"'.format(readonly, str(rest.type).strip())
+            extraParameters = '{} data-type="{}"'.format(readonly, str(rest.type.type) + str(rest.type.suffix or ""))
         elif idlType == "dict-member":
             assert isinstance(construct, widlparser.DictionaryMember)
-            extraParameters = f'data-type="{construct.type}"'
+            extraParameters = 'data-type="{}"'.format(str(construct.type.type) + str(construct.type.suffix or ""))
             if construct.default is not None:
                 value = str(construct.default).split("=", 1)[1].strip()
                 if value.startswith("["):
@@ -287,7 +357,8 @@ class IDLMarker(widlparser.protocols.Marker):
     def markup_enum_value(self, text: str, construct: widlparser.Construct) -> MarkupReturnT:
         return (
             "<idl data-idl-type=enum-value data-idl-for='{}' data-lt='{}'>".format(
-                h.escapeAttr(t.cast(str, construct.name)), h.escapeAttr(text)
+                h.escapeAttr(t.cast(str, construct.name)),
+                h.escapeAttr(text),
             ),
             "</idl>",
         )
@@ -330,7 +401,6 @@ class IDLMarker(widlparser.protocols.Marker):
 
 
 def markupIDL(doc: t.SpecT) -> None:
-    highlightingOccurred = False
     idlEls = h.findAll("pre.idl:not([data-no-idl]), xmp.idl:not([data-no-idl])", doc)
     for el in h.findAll("script[type=idl]", doc):
         # To help with syntax-highlighting, <script type=idl> is also allowed here.
@@ -349,14 +419,14 @@ def markupIDL(doc: t.SpecT) -> None:
         if h.isNormative(doc, el):
             text = h.textContent(el)
             # Parse once with a fresh parser, so I can spit out just this <pre>'s markup.
-            widl = widlparser.parser.Parser(text, ui=IDLUI(), symbol_table=symbolTable)
+            widl = widlparser.parser.Parser(text, ui=IDLUI.fromEl(el), symbol_table=symbolTable)
             marker = DebugMarker() if doc.debug else IDLMarker()
 
             h.replaceContents(el, h.parseHTML(str(widl.markup(marker))))
             # Parse a second time with the global one, which collects all data in the doc.
             doc.widl.parse(text)
         h.addClass(doc, el, "highlight")
-        highlightingOccurred = True
+        doc.extraJC.addIDLHighlighting()
     if doc.md.slimBuildArtifact:
         # Remove the highlight-only spans
         for el in idlEls:
@@ -364,14 +434,6 @@ def markupIDL(doc: t.SpecT) -> None:
                 contents = h.childNodes(span, clear=True)
                 h.replaceNode(span, *contents)
         return
-    if highlightingOccurred:
-        doc.extraStyles[
-            "style-syntax-highlighting"
-        ] += """
-            pre.idl.highlight {
-                background: var(--borderedblock-bg, var(--def-bg));
-            }
-            """
 
 
 def markupIDLBlock(pre: t.ElementT, doc: t.SpecT) -> set[t.ElementT]:
@@ -385,10 +447,9 @@ def markupIDLBlock(pre: t.ElementT, doc: t.SpecT) -> set[t.ElementT]:
     for el in h.findAll("idl", pre):
         idlType = el.get("data-idl-type")
         assert isinstance(idlType, str)
-        url = None
         forceDfn = False
         ref = None
-        idlText = None
+        idlText: str
         for idlText in (el.get("data-lt") or "").split("|"):
             if idlType == "interface" and idlText in forcedInterfaces:
                 forceDfn = True
@@ -403,11 +464,10 @@ def markupIDLBlock(pre: t.ElementT, doc: t.SpecT) -> set[t.ElementT]:
                     error=False,
                 )
                 if ref:
-                    url = ref.url
                     break
             if ref:
                 break
-        if url is None or forceDfn:
+        if ref is None or isinstance(ref, str) or forceDfn:
             el.tag = "dfn"
             el.set("data-dfn-type", idlType)
             del el.attrib["data-idl-type"]
@@ -416,7 +476,7 @@ def markupIDLBlock(pre: t.ElementT, doc: t.SpecT) -> set[t.ElementT]:
                 del el.attrib["data-idl-for"]
         else:
             # Copy over the auto-generated linking text to the manual dfn.
-            dfn = h.find(url, doc)
+            dfn = ref.el
             # How in the hell does this work, the url is not a selector???
             assert dfn is not None
             lts = combineIdlLinkingTexts(el.get("data-lt"), dfn.get("data-lt"))
