@@ -4,7 +4,8 @@ import hashlib
 import itertools
 import re
 
-from . import datablocks, markdown, h, messages as m, t
+from . import datablocks, h, markdown, t
+from . import messages as m
 
 
 def processInclusions(doc: t.SpecT) -> None:
@@ -45,6 +46,10 @@ def handleBikeshedInclude(el: t.ElementT, doc: t.SpecT) -> None:
         doc.recordDependencies(includedInputSource)
         try:
             lines = includedInputSource.read().rawLines
+        except FileNotFoundError:
+            m.die(f"Couldn't find include file '{path}'.", el=el)
+            h.removeNode(el)
+            return
         except Exception as err:
             m.die(f"Couldn't find include file '{path}'. Error was:\n{err}", el=el)
             h.removeNode(el)
@@ -70,10 +75,12 @@ def handleBikeshedInclude(el: t.ElementT, doc: t.SpecT) -> None:
             m.die("Nesting depth > 100, literally wtf are you doing.", el=el)
             h.removeNode(el)
             return
+        parseConfig = h.ParseConfig.fromSpec(doc)
+        parseConfig.macros = {**parseConfig.macros, **macros}
+        lines = h.parseLines(lines, parseConfig)
         lines = datablocks.transformDataBlocks(doc, lines)
         lines = markdown.parse(lines, doc.md.indent, opaqueElements=doc.md.opaqueElements)
         text = "".join(lines)
-        text = doc.fixText(text, moreMacros=macros)
         subtree = h.parseHTML(text)
         for childInclude in h.findAll("pre.include", h.E.div({}, *subtree)):
             childInclude.set("hash", hash)
@@ -106,6 +113,10 @@ def handleCodeInclude(el: t.ElementT, doc: t.SpecT) -> None:
     doc.recordDependencies(includedInputSource)
     try:
         lines = includedInputSource.read().rawLines
+    except FileNotFoundError:
+        m.die(f"Couldn't find include-code file '{path}'.", el=el)
+        h.removeNode(el)
+        return
     except Exception as err:
         m.die(f"Couldn't find include-code file '{path}'. Error was:\n{err}", el=el)
         h.removeNode(el)
@@ -149,6 +160,10 @@ def handleRawInclude(el: t.ElementT, doc: t.SpecT) -> None:
     doc.recordDependencies(includedInputSource)
     try:
         content = includedInputSource.read().content
+    except FileNotFoundError:
+        m.die(f"Couldn't find include-raw file '{path}'.", el=el)
+        h.removeNode(el)
+        return
     except Exception as err:
         m.die(f"Couldn't find include-raw file '{path}'. Error was:\n{err}", el=el)
         h.removeNode(el)
