@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import dataclasses
 
 from . import t
@@ -8,11 +10,11 @@ class Line:
     i: int
     text: str
 
-    def __unicode__(self):
+    def __unicode__(self) -> str:
         return self.text
 
 
-def rectify(lines: t.Union[t.List[str], t.List[Line]]) -> t.List[Line]:
+def rectify(lines: list[str] | list[Line]) -> list[Line]:
     if any(isinstance(x, str) for x in lines):
         return [Line(-1, t.cast(str, x)) for x in lines]
-    return t.cast("t.Union[t.List[Line]]", lines)
+    return t.cast("list[Line]", lines)
