@@ -2,7 +2,8 @@ from __future__ import annotations
 
 import widlparser  # pylint: disable=unused-import
 
-from .. import messages as m, t
+from .. import messages as m
+from .. import t
 
 
 def missingExposed(doc: t.SpecT) -> None:
@@ -32,7 +33,7 @@ def missingExposed(doc: t.SpecT) -> None:
                     break
             if not good:
                 m.lint(
-                    f"The '{construct.name}' namespace is missing an [Exposed] extended attribute. Does it need [Exposed=Window], or something more?"
+                    f"The '{construct.name}' namespace is missing an [Exposed] extended attribute. Does it need [Exposed=Window], or something more?",
                 )
         elif construct.idl_type == "interface":
             good = False
@@ -45,7 +46,7 @@ def missingExposed(doc: t.SpecT) -> None:
                     break
             if not good:
                 m.lint(
-                    f"The '{construct.name}' interface is missing an [Exposed] extended attribute. Does it need [Exposed=Window], or something more?"
+                    f"The '{construct.name}' interface is missing an [Exposed] extended attribute. Does it need [Exposed=Window], or something more?",
                 )
         elif construct.idl_type == "callback":
             if not hasattr(construct, "interface"):
