@@ -1,7 +1,8 @@
 from __future__ import annotations
 
+from .. import h, t
+from .. import messages as m
 from ..config import dfnElementsSelector
-from .. import h, messages as m, t
 
 
 def unusedInternalDfns(doc: t.SpecT) -> None:
@@ -25,5 +26,6 @@ def unusedInternalDfns(doc: t.SpecT) -> None:
     for el in noexportDfns:
         if el.get("id") not in localHrefs:
             m.lint(
-                f"Unexported dfn that's not referenced locally - did you mean to export it?\n{h.outerHTML(el)}", el=el
+                "Unexported dfn that's not referenced locally - did you mean to export it?",
+                el=el,
             )
