@@ -3,17 +3,16 @@ from __future__ import annotations
 import functools
 import re
 
-from .. import h, messages as m, t
-
-DEBUG = False
+from .. import constants, h, t
+from .. import line as l
+from .. import messages as m
 
 if t.TYPE_CHECKING:
 
     class TokenT(t.TypedDict, total=False):
         type: t.Required[str]
         prefixlen: int | float
-        line: str
-        lineNum: int
+        line: l.Line
         text: str
 
     class HeadingTokenT(TokenT, total=False):
@@ -31,13 +30,38 @@ if t.TYPE_CHECKING:
         start: int
 
 
+if t.TYPE_CHECKING:
+
+    @t.overload
+    def parse(
+        lines: list[str],
+        numSpacesForIndentation: int,
+        features: set[str] | None = None,
+        opaqueElements: list[str] | None = None,
+        blockElements: list[str] | None = None,
+    ) -> list[str]: ...
+
+    @t.overload
+    def parse(
+        lines: list[l.Line],
+        numSpacesForIndentation: int,
+        features: set[str] | None = None,
+        opaqueElements: list[str] | None = None,
+        blockElements: list[str] | None = None,
+    ) -> list[l.Line]: ...
+
+
 def parse(
-    lines: list[str],
+    lines: list[str] | list[l.Line],
     numSpacesForIndentation: int,
     features: set[str] | None = None,
     opaqueElements: list[str] | None = None,
     blockElements: list[str] | None = None,
-) -> list[str]:
+) -> list[str] | list[l.Line]:
+    fromStrings = False
+    if any(isinstance(x, str) for x in lines):
+        fromStrings = True
+    lines = l.rectify(lines)
     tokens = tokenizeLines(
         lines,
         numSpacesForIndentation,
@@ -45,12 +69,15 @@ def parse(
         opaqueElements=opaqueElements,
         blockElements=blockElements,
     )
-    htmlLines = parseTokens(tokens, numSpacesForIndentation)
-    return htmlLines
+    html = parseTokens(tokens, numSpacesForIndentation)
+    if fromStrings:
+        return [x.text for x in html]
+    else:
+        return html
 
 
 def tokenizeLines(
-    lines: list[str],
+    lines: list[l.Line],
     numSpacesForIndentation: int,
     features: set[str] | None = None,
     opaqueElements: list[str] | None = None,
@@ -119,25 +146,17 @@ def tokenizeLines(
         if tagname in inlineElements:
             return True
         assert blockElements is not None
-        if "-" in tagname and tagname not in blockElements:
-            # Assume custom elements are inline by default
-            return True
-        return False
+
+        # Assume custom elements are inline by default
+        return "-" in tagname and tagname not in blockElements
 
     tokens: list[TokenT] = []
     rawStack: list[RawTokenT] = []
     rawElements = "|".join(re.escape(x) for x in opaqueElements)
 
-    i = 0
-    for line in lines:
-        i += 1
-        # Some earlier adjustments can replace elements, giving
-        # a different number of lines.
-        match = re.match(r"<!--LINE NUMBER RESET (\d+)-->$", line)
-        if match:
-            i = int(match.group(1))
-            if DEBUG:
-                print(f"Line Reset: {i}")
+    for i, line in enumerate(lines):
+        # Skip lines that are entirely a censored comment.
+        if line.text.strip() == constants.bsComment:
             continue
 
         # Three kinds of "raw" elements, which prevent markdown processing inside of them.
@@ -157,18 +176,18 @@ def tokenizeLines(
                 rawStack.pop()
                 if endTag["type"] == "fenced":
                     stripCommonWsPrefix(tokens[endTag["start"] + 1 :])
-                    line = "</xmp>"
-                tokens.append({"type": "raw", "prefixlen": float("inf"), "line": line, "lineNum": i})
+                    line.text = "</xmp>"
+                tokens.append({"type": "raw", "prefixlen": float("inf"), "line": line})
                 continue
             elif not endTag["nest"]:
                 # Just an internal line, but for the no-nesting elements,
                 # so guaranteed no more work needs to be done.
-                tokens.append({"type": "raw", "prefixlen": float("inf"), "line": line, "lineNum": i})
+                tokens.append({"type": "raw", "prefixlen": float("inf"), "line": line})
                 continue
 
         # We're either in a nesting raw element or not in a raw element at all,
         # so check if the line starts a new element.
-        match = re.match(r"(\s*)(`{3,}|~{3,})([^`]*)$", line)
+        match = re.match(r"(\s*)(`{3,}|~{3,})([^`]*)$", line.text)
         if match:
             ws, tag, infoString = match.groups()
             rawStack.append({"type": "fenced", "tag": tag, "nest": False, "start": i})
@@ -179,27 +198,25 @@ def tokenizeLines(
                 classAttr = f" class='language-{h.escapeAttr(lang)}'"
             else:
                 classAttr = ""
-            line = f"{ws}<xmp{classAttr}>"
+            line.text = f"{ws}<xmp{classAttr}>"
             tokens.append(
                 {
                     "type": "raw",
                     "prefixlen": prefixCount(ws, numSpacesForIndentation),
                     "line": line,
-                    "lineNum": i,
-                }
+                },
             )
             continue
-        match = re.match(rf"\s*<({rawElements})[ >]", line)
+        match = re.match(rf"\s*<({rawElements})[ >]", line.text)
         if match:
             tokens.append(
                 {
                     "type": "raw",
-                    "prefixlen": prefixCount(line, numSpacesForIndentation),
+                    "prefixlen": prefixCount(line.text, numSpacesForIndentation),
                     "line": line,
-                    "lineNum": i,
-                }
+                },
             )
-            if re.search(r"</({})>".format(match.group(1)), line):
+            if re.search(r"</({})>".format(match.group(1)), line.text):
                 # Element started and ended on same line, cool, don't need to do anything.
                 pass
             else:
@@ -209,21 +226,14 @@ def tokenizeLines(
                         "type": "element",
                         "tag": "</{}>".format(match.group(1)),
                         "nest": nest,
-                    }
+                    },
                 )
             continue
         if rawStack:
-            tokens.append(
-                {
-                    "type": "raw",
-                    "prefixlen": float("inf"),
-                    "line": line,
-                    "lineNum": i,
-                }
-            )
+            tokens.append({"type": "raw", "prefixlen": float("inf"), "line": line})
             continue
 
-        lineText = line.strip()
+        lineText = line.text.strip()
 
         token: TokenT
 
@@ -295,44 +305,19 @@ def tokenizeLines(
         if token["type"] == "blank":
             token["prefixlen"] = float("inf")
         else:
-            token["prefixlen"] = prefixCount(line, numSpacesForIndentation)
+            token["prefixlen"] = prefixCount(line.text, numSpacesForIndentation)
         token["line"] = line
-        token["lineNum"] = i
         tokens.append(token)
 
-    if DEBUG:  # pylint: disable=using-constant-test
-        for token in tokens:
-            lineNum = " " * (2 - len(str(token["lineNum"]))) + str(token["lineNum"])
-            tokenType = " " * (11 - len(token["type"])) + token["type"]
-            print(f"{lineNum} {tokenType}: {repr(token['line'])}")
+    if False:  # pylint: disable=using-constant-test
+        for i, token in enumerate(tokens):
+            print(  # noqa: T201
+                f"{' '*(2-len(str(i)))}{i} {' ' * (11 - len(token['type']))}{token['type']}: {token['line'].text.rstrip()}",
+            )
 
     return tokens
 
 
-def stripCommentsFromLine(line: str, inComment: bool = False) -> tuple[str, bool]:
-    # Removes HTML comments from the line.
-    # Returns true if the comment wasn't closed by the end of the line
-    if inComment:
-        # The line starts out in a comment.
-        pre, sep, post = line.partition("-->")
-        if sep == "":
-            # The entire line is a comment
-            return "", True
-        else:
-            # Drop the comment part, see if there are any more
-            return stripCommentsFromLine(post)
-    else:
-        # The line starts out as non-comment content.
-        pre, sep, post = line.partition("<!--")
-        if sep == "":
-            # No comments in the line
-            return pre, False
-        else:
-            # Keep the non-comment part, see if there's any more to do
-            res, inComment = stripCommentsFromLine(post, inComment=True)
-            return pre + res, inComment
-
-
 def prefixCount(text: str, numSpacesForIndentation: int) -> int:
     i = 0
     prefixLen = 0
@@ -351,7 +336,7 @@ def prefixCount(text: str, numSpacesForIndentation: int) -> int:
 def stripPrefix(token: TokenT, numSpacesForIndentation: int, len: int) -> str:
     """Removes len number of prefix groups"""
 
-    text = token["line"]
+    text = token["line"].text
 
     # Don't mess with "infinite" prefix lines
     if token["prefixlen"] == float("inf"):
@@ -369,16 +354,17 @@ def stripPrefix(token: TokenT, numSpacesForIndentation: int, len: int) -> str:
             offset += numSpacesForIndentation
         else:
             m.die(
-                f'Line {token["lineNum"]} isn\'t indented enough (needs {len} indent{"" if len == 1 else "s"}) to be valid Markdown:\n"{text[:-1]}"'
+                f'Line {token["line"].i} isn\'t indented enough (needs {len} indent{"" if len == 1 else "s"}) to be valid Markdown:\n"{text[:-1]}"',
             )
             return text
     return text[offset:]
 
 
-def lineEndsRawBlock(line: str, rawToken: RawTokenT) -> bool:
-    elementEnds = bool(rawToken["type"] == "element" and re.search(rawToken["tag"], line))
+def lineEndsRawBlock(line: l.Line, rawToken: RawTokenT) -> bool:
+    elementEnds = bool(rawToken["type"] == "element" and re.search(rawToken["tag"], line.text))
     fencedEnds = bool(
-        rawToken["type"] == "fenced" and re.match(r"\s*{}{}*\s*$".format(rawToken["tag"], rawToken["tag"][0]), line)
+        rawToken["type"] == "fenced"
+        and re.match(r"\s*{}{}*\s*$".format(rawToken["tag"], rawToken["tag"][0]), line.text),
     )
     return elementEnds or fencedEnds
 
@@ -387,16 +373,16 @@ def stripCommonWsPrefix(tokens: list[TokenT]) -> list[TokenT]:
     # Remove the longest common whitespace prefix from the lines.
     if not tokens:
         return tokens
-    ws = [getWsPrefix(t["line"]) for t in tokens]
+    ws = [getWsPrefix(t["line"].text) for t in tokens]
     prefix = functools.reduce(commonPrefix, ws)
     prefixLen = len(prefix) if prefix else 0
     for token in tokens:
-        stripped = token["line"]
-        if len(token["line"]) > prefixLen:
+        stripped = token["line"].text
+        if len(token["line"].text) > prefixLen:
             # Don't remove from blank lines,
             # or else you'll eat the newline!
-            stripped = token["line"][prefixLen:]
-        token["line"] = stripped
+            stripped = token["line"].text[prefixLen:]
+        token["line"].text = stripped
     return tokens
 
 
@@ -423,7 +409,7 @@ def getWsPrefix(line: str) -> str | None:
     return t.cast(str, match.group(1))
 
 
-def parseTokens(tokens: list[TokenT], numSpacesForIndentation: int) -> list[str]:
+def parseTokens(tokens: list[TokenT], numSpacesForIndentation: int) -> list[l.Line]:
     """
     Token types:
     eof
@@ -442,7 +428,7 @@ def parseTokens(tokens: list[TokenT], numSpacesForIndentation: int) -> list[str]
     raw
     """
     stream = TokenStream(tokens, numSpacesForIndentation)
-    lines: list[str] = []
+    lines: list[l.Line] = []
 
     while True:
         if stream.ended():
@@ -470,33 +456,48 @@ def parseTokens(tokens: list[TokenT], numSpacesForIndentation: int) -> list[str]
         elif stream.currtype() == "blockquote":
             lines += parseBlockquote(stream)
         else:
-            lines.append(stream.currline())
+            line = stream.currline()
+            lines.append(line)
             stream.advance()
 
+    # for line in lines:
+    #    print f"«{line.text.rstrip()}»"
+
     return lines
 
 
+def lineFromStream(stream: TokenStream, text: str) -> l.Line:
+    # Shortcut for when you're producing a new line from the currline in the stream,
+    # with some modified text.
+    return l.Line(stream.currline().i, text)
+
+
 # Each parser gets passed the stream
 # and must return the lines it returns.
 # The stream should be advanced to the *next* line,
 # after the lines you've used up dealing with the construct.
 
 
-def parseSingleLineHeading(stream: TokenStream) -> list[str]:
+def parseSingleLineHeading(stream: TokenStream) -> list[l.Line]:
     if "id" in stream.curr():
         idattr = f" id='{stream.currid()}'"
     else:
         idattr = ""
     lines = [
-        "<h{level}{idattr} bs-line-number={i}>{text}</h{level}>".format(
-            idattr=idattr, i=stream.currlineNum(), **stream.curr()
-        )
+        lineFromStream(
+            stream,
+            "<h{level}{idattr} bs-line-number={i}>{text}</h{level}>\n".format(
+                idattr=idattr,
+                i=stream.currline().i,
+                **stream.curr(),
+            ),
+        ),
     ]
     stream.advance()
     return lines
 
 
-def parseMultiLineHeading(stream: TokenStream) -> list[str]:
+def parseMultiLineHeading(stream: TokenStream) -> list[l.Line]:
     if stream.nexttype() == "equals-line":
         level = 2
     elif stream.nexttype() == "dash-line":
@@ -506,8 +507,9 @@ def parseMultiLineHeading(stream: TokenStream) -> list[str]:
             "Markdown parser error: tried to parse a multiline heading from:\n"
             + stream.prevraw()
             + stream.currraw()
-            + stream.nextraw()
+            + stream.nextraw(),
         )
+        level = 2
     match = re.search(r"(.*?)\s*\{\s*#([^ }]+)\s*\}\s*$", stream.currtext())
     if match:
         text = match.group(1)
@@ -516,31 +518,34 @@ def parseMultiLineHeading(stream: TokenStream) -> list[str]:
         text = stream.currtext()
         idattr = ""
     lines = [
-        "<h{level} {idattr} bs-line-number={i}>{htext}</h{level}>".format(
-            idattr=idattr,
-            level=level,
-            htext=text,
-            i=stream.currlineNum(),
-            **stream.curr(),
-        )
+        lineFromStream(
+            stream,
+            "<h{level} {idattr} bs-line-number={i}>{htext}</h{level}>\n".format(
+                idattr=idattr,
+                level=level,
+                htext=text,
+                i=stream.currline().i,
+                **stream.curr(),
+            ),
+        ),
     ]
     stream.advance(2)
     return lines
 
 
-def parseHorizontalRule(stream: TokenStream) -> list[str]:
-    lines = ["<hr>"]
+def parseHorizontalRule(stream: TokenStream) -> list[l.Line]:
+    lines = [lineFromStream(stream, "<hr>\n")]
     stream.advance()
     return lines
 
 
-def parseParagraph(stream: TokenStream) -> list[str]:
+def parseParagraph(stream: TokenStream) -> list[l.Line]:
     line = stream.currtext()
-    i = stream.currlineNum()
+    i = stream.currline().i
     initialPrefixLen = stream.currprefixlen()
     endTag = "</p>"
     if re.match(r"note[:,]\s*", line, re.I):
-        p = "<p class='replace-with-note-class'>"
+        p = f"<p bs-line-number={i} class='replace-with-note-class'>"
         matchNote = re.match(r"(note[:,])(\s*)(.*)", line, re.I)
         if matchNote:
             line = matchNote.group(3)
@@ -558,31 +563,29 @@ def parseParagraph(stream: TokenStream) -> list[str]:
         match = re.match(r"issue\(([^)]+)\):(.*)", line, re.I)
         if match:
             line = match.group(2)
-            p = "<p bs-line-number={} data-remote-issue-id='{}' class='replace-with-issue-class'>".format(
-                i, match.group(1)
-            )
+            p = f"<p bs-line-number={i} data-remote-issue-id='{match.group(1)}' class='replace-with-issue-class'>"
         else:
             p = f"<p bs-line-number={i}>"
-    lines = [f"{p}{line}"]
+    lines = [lineFromStream(stream, f"{p}{line}\n")]
     while True:
         stream.advance()
         if stream.currtype() not in ["text"] or stream.currprefixlen() < initialPrefixLen:
-            lines[-1] = lines[-1].rstrip() + endTag
+            lines[-1].text = lines[-1].text.rstrip() + endTag + "\n"
             return lines
         lines.append(stream.currline())
 
 
-def parseBulleted(stream: TokenStream) -> list[str]:
+def parseBulleted(stream: TokenStream) -> list[l.Line]:
     prefixLen = stream.currprefixlen()
     numSpacesForIndentation = stream.numSpacesForIndentation
-    ul_i = stream.currlineNum()
+    ul_i = stream.currline().i
 
-    def parseItem(stream: TokenStream) -> tuple[list[str], int]:
+    def parseItem(stream: TokenStream) -> tuple[list[l.Line], int]:
         # Assumes it's being called with curr being a bulleted line.
         # Remove the bulleted part from the line
-        firstLine = stream.currtext()
-        i = stream.currlineNum()
-        lines = [firstLine]
+        firstLine = stream.currtext() + "\n"
+        i = stream.currline().i
+        lines = [lineFromStream(stream, firstLine)]
         while True:
             stream.advance()
             # All the conditions that indicate we're *past* the end of the item.
@@ -595,9 +598,14 @@ def parseBulleted(stream: TokenStream) -> list[str]:
             if stream.currtype() == "eof":
                 return lines, i
             # Remove the prefix from each line before adding it.
-            lines.append(stripPrefix(stream.curr(), numSpacesForIndentation, prefixLen + 1))
+            lines.append(
+                lineFromStream(
+                    stream,
+                    stripPrefix(stream.curr(), numSpacesForIndentation, prefixLen + 1),
+                ),
+            )
 
-    def getItems(stream: TokenStream) -> t.Generator[tuple[list[str], int], None, None]:
+    def getItems(stream: TokenStream) -> t.Generator[tuple[list[l.Line], int], None, None]:
         while True:
             # The conditions that indicate we're past the end of the list itself
             if stream.currtype() == "eof":
@@ -610,26 +618,26 @@ def parseBulleted(stream: TokenStream) -> list[str]:
                 stream.advance()
             yield parseItem(stream)
 
-    lines = [f"<ul data-md bs-line-number={ul_i}>"]
+    lines = [l.Line(-1, f"<ul data-md bs-line-number={ul_i}>")]
     for li_lines, i in getItems(stream):
-        lines.append(f"<li data-md bs-line-number={i}>")
+        lines.append(l.Line(-1, f"<li data-md bs-line-number={i}>"))
         lines.extend(parse(li_lines, numSpacesForIndentation))
-        lines.append("</li>")
-    lines.append("</ul>")
+        lines.append(l.Line(-1, "</li>"))
+    lines.append(l.Line(-1, "</ul>"))
     return lines
 
 
-def parseNumbered(stream: TokenStream, start: int = 1) -> list[str]:
+def parseNumbered(stream: TokenStream, start: int = 1) -> list[l.Line]:
     prefixLen = stream.currprefixlen()
-    ol_i = stream.currlineNum()
+    ol_i = stream.currline().i
     numSpacesForIndentation = stream.numSpacesForIndentation
 
-    def parseItem(stream: TokenStream) -> tuple[list[str], int]:
+    def parseItem(stream: TokenStream) -> tuple[list[l.Line], int]:
         # Assumes it's being called with curr being a numbered line.
         # Remove the numbered part from the line
-        firstLine = stream.currtext()
-        i = stream.currlineNum()
-        lines = [firstLine]
+        firstLine = stream.currtext() + "\n"
+        i = stream.currline().i
+        lines = [lineFromStream(stream, firstLine)]
         while True:
             stream.advance()
             # All the conditions that indicate we're *past* the end of the item.
@@ -642,9 +650,14 @@ def parseNumbered(stream: TokenStream, start: int = 1) -> list[str]:
             if stream.currtype() == "eof":
                 return lines, i
             # Remove the prefix from each line before adding it.
-            lines.append(stripPrefix(stream.curr(), numSpacesForIndentation, prefixLen + 1))
+            lines.append(
+                lineFromStream(
+                    stream,
+                    stripPrefix(stream.curr(), numSpacesForIndentation, prefixLen + 1),
+                ),
+            )
 
-    def getItems(stream: TokenStream) -> t.Generator[tuple[list[str], int], None, None]:
+    def getItems(stream: TokenStream) -> t.Generator[tuple[list[l.Line], int], None, None]:
         while True:
             # The conditions that indicate we're past the end of the list itself
             if stream.currtype() == "eof":
@@ -658,28 +671,28 @@ def parseNumbered(stream: TokenStream, start: int = 1) -> list[str]:
             yield parseItem(stream)
 
     if start == 1:
-        lines = [f"<ol data-md bs-line-number={ol_i}>"]
+        lines = [l.Line(-1, f"<ol data-md bs-line-number={ol_i}>")]
     else:
-        lines = [f"<ol data-md start='{start}' bs-line-number={ol_i}>"]
+        lines = [l.Line(-1, f"<ol data-md start='{start}' bs-line-number={ol_i}>")]
     for li_lines, i in getItems(stream):
-        lines.append(f"<li data-md bs-line-number={i}>")
+        lines.append(l.Line(-1, f"<li data-md bs-line-number={i}>"))
         lines.extend(parse(li_lines, numSpacesForIndentation))
-        lines.append("</li>")
-    lines.append("</ol>")
+        lines.append(l.Line(-1, "</li>"))
+    lines.append(l.Line(-1, "</ol>"))
     return lines
 
 
-def parseDl(stream: TokenStream) -> list[str]:
+def parseDl(stream: TokenStream) -> list[l.Line]:
     prefixLen = stream.currprefixlen()
-    dl_i = stream.currlineNum()
+    dl_i = stream.currline().i
     numSpacesForIndentation = stream.numSpacesForIndentation
 
-    def parseItem(stream: TokenStream) -> tuple[str, list[str], int]:
+    def parseItem(stream: TokenStream) -> tuple[str, list[l.Line], int]:
         # Assumes it's being called with curr being a :/:: prefixed line.
-        firstLine = stream.currtext()
-        i = stream.currlineNum()
+        firstLine = stream.currtext() + "\n"
+        i = stream.currline().i
         type = stream.currtype()
-        lines = [firstLine]
+        lines = [lineFromStream(stream, firstLine)]
         while True:
             stream.advance()
             # All the conditions that indicate we're *past* the end of the item.
@@ -696,9 +709,14 @@ def parseDl(stream: TokenStream) -> list[str]:
             if stream.currtype() == "eof":
                 return type, lines, i
             # Remove the prefix from each line before adding it.
-            lines.append(stripPrefix(stream.curr(), numSpacesForIndentation, prefixLen + 1))
+            lines.append(
+                lineFromStream(
+                    stream,
+                    stripPrefix(stream.curr(), numSpacesForIndentation, prefixLen + 1),
+                ),
+            )
 
-    def getItems(stream: TokenStream) -> t.Generator[tuple[str, list[str], int], None, None]:
+    def getItems(stream: TokenStream) -> t.Generator[tuple[str, list[l.Line], int], None, None]:
         while True:
             # The conditions that indicate we're past the end of the list itself
             if stream.currtype() == "eof":
@@ -715,28 +733,32 @@ def parseDl(stream: TokenStream) -> list[str]:
                 stream.advance()
             yield parseItem(stream)
 
-    lines = [f"<dl data-md bs-line-number={dl_i}>"]
+    lines = [l.Line(-1, f"<dl data-md bs-line-number={dl_i}>")]
     for type, di_lines, i in getItems(stream):
-        lines.append(f"<{type} data-md bs-line-number={i}>")
+        lines.append(l.Line(-1, f"<{type} data-md bs-line-number={i}>"))
         lines.extend(parse(di_lines, numSpacesForIndentation))
-        lines.append(f"</{type}>")
-    lines.append("</dl>")
+        lines.append(l.Line(-1, f"</{type}>"))
+    lines.append(l.Line(-1, "</dl>"))
     return lines
 
 
-def parseBlockquote(stream: TokenStream) -> list[str]:
+def parseBlockquote(stream: TokenStream) -> list[l.Line]:
     prefixLen = stream.currprefixlen()
-    i = stream.currlineNum()
-    lines = [stream.currtext()]
+    i = stream.currline().i
+    lines = [lineFromStream(stream, stream.currtext() + "\n")]
     while True:
         stream.advance()
         if stream.currprefixlen() < prefixLen:
             break
         if stream.currtype() in ["blockquote", "text"]:
-            lines.append(stream.currtext())
+            lines.append(lineFromStream(stream, stream.currtext() + "\n"))
         else:
             break
-    return [f"<blockquote bs-line-number={i}>"] + parse(lines, stream.numSpacesForIndentation) + ["</blockquote>"]
+    return (
+        [l.Line(-1, f"<blockquote bs-line-number={i}>\n")]
+        + parse(lines, stream.numSpacesForIndentation)
+        + [l.Line(-1, "</blockquote>\n")]
+    )
 
 
 class TokenStream:
@@ -746,7 +768,7 @@ class TokenStream:
         numSpacesForIndentation: int,
         before: TokenT | None = None,
         after: TokenT | None = None,
-    ):
+    ) -> None:
         self.tokens = tokens
         self.i = 0
         self.numSpacesForIndentation = numSpacesForIndentation
