@@ -1,8 +1,11 @@
 from __future__ import annotations
 
 import dataclasses
+import re
 
-from . import railroaddiagrams as rr, messages as m, t
+from . import messages as m
+from . import railroaddiagrams as rr
+from . import t
 
 
 def parse(string: str) -> rr.Diagram | None:
@@ -20,7 +23,6 @@ def parse(string: str) -> rr.Diagram | None:
             Terminal: foo
             Terminal raw: bar
     """
-    import re
 
     lines = string.splitlines()
 
