@@ -6,8 +6,10 @@ import random
 import re
 from collections import defaultdict
 
-from .. import biblio, config, constants, datablocks, h, messages as m, retrieve, t
-from . import utils, source, headingdata, wrapper
+from .. import biblio, config, constants, h, retrieve, t
+from .. import messages as m
+from . import headingdata, source, utils, wrapper
+from .utils import LinkFailure
 
 if t.TYPE_CHECKING:
     from .wrapper import RefWrapper
@@ -17,24 +19,24 @@ if t.TYPE_CHECKING:
 
 class ReferenceManager:
     __slots__ = [
-        "dataFile",
-        "specs",
-        "defaultSpecs",
-        "ignoredSpecs",
-        "replacedSpecs",
-        "biblios",
-        "loadedBiblioGroups",
+        "anchorBlockRefs",
         "biblioKeys",
         "biblioNumericSuffixes",
-        "preferredBiblioNames",
-        "headings",
+        "biblios",
+        "dataFile",
+        "defaultSpecs",
         "defaultStatus",
-        "localRefs",
-        "anchorBlockRefs",
         "foreignRefs",
+        "headings",
+        "ignoredSpecs",
+        "loadedBiblioGroups",
+        "localRefs",
+        "preferredBiblioNames",
+        "replacedSpecs",
         "shortname",
-        "specLevel",
         "spec",
+        "specLevel",
+        "specs",
         "testing",
     ]
 
@@ -43,7 +45,7 @@ class ReferenceManager:
         defaultStatus: str | None = None,
         fileRequester: t.DataFileRequester | None = None,
         testing: bool = False,
-    ):
+    ) -> None:
         self.dataFile: t.DataFileRequester
         if fileRequester is None:
             self.dataFile = retrieve.defaultRequester
@@ -53,7 +55,7 @@ class ReferenceManager:
         self.testing: bool = testing
 
         # Dict of {spec vshortname => spec data}
-        self.specs: dict[str, SpecDataT] = dict()
+        self.specs: dict[str, SpecDataT] = {}
 
         # Dict of {linking text => link-defaults data}
         self.defaultSpecs: t.LinkDefaultsT = defaultdict(list)
@@ -75,13 +77,13 @@ class ReferenceManager:
 
         # Dict of {suffixless key => [keys with numeric suffixes]}
         # (So you can tell when it's appropriate to default a numeric-suffix ref to a suffixless one.)
-        self.biblioNumericSuffixes: dict[str, list[str]] = dict()
+        self.biblioNumericSuffixes: dict[str, list[str]] = {}
 
         # Dict of {base key name => preferred display name}
-        self.preferredBiblioNames: dict[str, str] = dict()
+        self.preferredBiblioNames: dict[str, str] = {}
 
         # Dict of {spec vshortname => headings}
-        self.headings: dict[str, headingdata.SpecHeadings] = dict()
+        self.headings: dict[str, headingdata.SpecHeadings] = {}
 
         self.defaultStatus: str
         if defaultStatus is None:
@@ -102,7 +104,7 @@ class ReferenceManager:
         self.specLevel: str | None = None
         self.spec: str | None = None
 
-    def initializeRefs(self, doc: t.SpecT | None = None) -> None:
+    def initializeRefs(self, datablocks: t.ModuleType, doc: t.SpecT | None = None) -> None:
         """
         Load up the xref data
         This is oddly split up into sub-functions to make it easier to track performance.
@@ -119,7 +121,10 @@ class ReferenceManager:
                 self.foreignRefs.methods[arglessSig] = variants
                 for argfullSig, data in argfulls.items():
                     variants.variants[argfullSig] = source.MethodVariant(
-                        argfullSig, data["args"], data["for"], data["shortname"]
+                        argfullSig,
+                        data["args"],
+                        data["for"],
+                        data["shortname"],
                     )
 
         initMethods()
@@ -128,10 +133,9 @@ class ReferenceManager:
             self.foreignRefs.fors.update(json.loads(self.dataFile.fetch("fors.json", str=True)))
 
         initFors()
-        if doc and doc.inputSource and doc.inputSource.hasDirectory:
+        if doc and doc.inputSource and doc.inputSource.hasDirectory():
             ldLines = self.dataFile.fetch("link-defaults.infotree").read().split("\n")
-            fakeTag = h.StartTag(tag="pre", line=-1, endLine=-1)
-            datablocks.transformInfo(lines=ldLines, doc=doc, indent="", startTag=fakeTag)
+            datablocks.transformInfo(lines=ldLines, doc=doc, firstLine=ldLines[0], tagName="pre", lineNum=None)
 
             # Get local anchor data
             shouldGetLocalAnchorData = doc.md.externalInfotrees["anchors.bsdata"]
@@ -139,7 +143,7 @@ class ReferenceManager:
                 m.warn(
                     "Found anchors.bsdata next to the specification without a matching\n"
                     + "External Infotrees: anchors.bsdata yes\n"
-                    + "in the metadata. This data won't be found when building via a URL."
+                    + "in the metadata. This data won't be found when building via a URL.",
                 )
                 # We should remove this after giving specs time to react to the warning:
                 shouldGetLocalAnchorData = True
@@ -147,9 +151,15 @@ class ReferenceManager:
                 try:
                     anchorFile = doc.inputSource.relative("anchors.bsdata")
                     if not anchorFile:
-                        raise OSError()
+                        raise OSError
                     anchorLines = anchorFile.read().rawLines
-                    datablocks.transformAnchors(lines=anchorLines, doc=doc, indent="", startTag=fakeTag)
+                    datablocks.transformAnchors(
+                        lines=anchorLines,
+                        doc=doc,
+                        firstLine=anchorLines[0],
+                        tagName="pre",
+                        lineNum=None,
+                    )
                 except OSError:
                     m.warn("anchors.bsdata not found despite being listed in the External Infotrees metadata.")
 
@@ -159,7 +169,7 @@ class ReferenceManager:
                 m.warn(
                     "Found link-defaults.infotree next to the specification without a matching\n"
                     + "External Infotrees: link-defaults.infotree yes\n"
-                    + "in the metadata. This data won't be found when building via a URL."
+                    + "in the metadata. This data won't be found when building via a URL.",
                 )
                 # We should remove this after giving specs time to react to the warning:
                 shouldGetLocalLinkDefaults = True
@@ -167,9 +177,15 @@ class ReferenceManager:
                 try:
                     ldFile = doc.inputSource.relative("link-defaults.infotree")
                     if not ldFile:
-                        raise OSError()
+                        raise OSError
                     ldLines = ldFile.read().rawLines
-                    datablocks.transformInfo(lines=ldLines, doc=doc, indent="", startTag=fakeTag)
+                    datablocks.transformInfo(
+                        lines=ldLines,
+                        doc=doc,
+                        firstLine=ldLines[0],
+                        tagName="pre",
+                        lineNum=None,
+                    )
                 except OSError:
                     m.warn("link-defaults.infotree not found despite being listed in the External Infotrees metadata.")
 
@@ -187,22 +203,28 @@ class ReferenceManager:
             return specHeadings
 
     def fetchHeading(
-        self, spec: str, id: str, status: str | None = None, el: t.ElementT | None = None
+        self,
+        spec: str,
+        id: str,
+        status: str | None = None,
+        el: t.ElementT | None = None,
     ) -> dict[str, str] | None:
         specHeadings = self.fetchHeadings(spec)
         if status is None:
             status = self.defaultStatus
         return specHeadings.get(id, status, el)
 
-    def initializeBiblio(self) -> None:
+    def initializeBiblio(self, doc: t.SpecT) -> None:
         self.biblioKeys.update(json.loads(self.dataFile.fetch("biblio-keys.json", str=True)))
         self.biblioNumericSuffixes.update(json.loads(self.dataFile.fetch("biblio-numeric-suffixes.json", str=True)))
 
         # Get local bibliography data
         try:
             storage: t.BiblioStorageT = defaultdict(list)
-            with open("biblio.json", encoding="utf-8") as fh:
-                biblio.processSpecrefBiblioFile(fh.read(), storage, order=2)
+            localBiblioPath = config.docPath(doc, "biblio.json")
+            if localBiblioPath:
+                with open(localBiblioPath, encoding="utf-8") as fh:
+                    biblio.processSpecrefBiblioFile(fh.read(), storage, order=2)
         except OSError:
             # Missing file is fine
             pass
@@ -222,21 +244,22 @@ class ReferenceManager:
                 snapshotURL="https://datatracker.ietf.org/doc/html/rfc2119",
                 order=3,
                 authors=["S. Bradner"],
-            )
+            ),
         )
 
-    def setSpecData(self, md: t.MetadataManager) -> None:
-        if md.defaultRefStatus:
-            self.defaultStatus = md.defaultRefStatus
-        elif md.status in config.snapshotStatuses:
-            self.defaultStatus = constants.refStatus.snapshot
-        elif md.status in config.shortToLongStatus:
-            self.defaultStatus = constants.refStatus.current
-        self.shortname = md.shortname
-        self.specLevel = md.level
-        self.spec = md.vshortname
+    def setSpecData(self, doc: t.SpecT) -> None:
+        if doc.md.defaultRefStatus:
+            self.defaultStatus = doc.md.defaultRefStatus
+        elif doc.doctype.status:
+            if "TR" in doc.doctype.status.requires:
+                self.defaultStatus = constants.refStatus.snapshot
+            else:
+                self.defaultStatus = constants.refStatus.current
+        self.shortname = doc.md.shortname
+        self.specLevel = doc.md.level
+        self.spec = doc.md.vshortname
 
-        for term, defaults in md.linkDefaults.items():
+        for term, defaults in doc.md.linkDefaults.items():
             for default in defaults:
                 self.defaultSpecs[term].append(default)
 
@@ -260,8 +283,8 @@ class ReferenceManager:
             elId = el.get("id")
             assert elId is not None
             try:
-                linkTexts = config.linkTextsFromElement(el)
-            except config.DuplicatedLinkText as e:
+                linkTexts = h.linkTextsFromElement(el)
+            except h.DuplicatedLinkText as e:
                 m.die(
                     f"The term '{e.offendingText}' is in both lt and local-lt of the element {h.outerHTML(e.el)}.",
                     el=e.el,
@@ -269,7 +292,6 @@ class ReferenceManager:
                 linkTexts = e.allTexts
             for linkText in linkTexts:
                 linkText = h.unfixTypography(linkText)
-                linkText = re.sub(r"\s+", " ", linkText)
                 linkType = h.treeAttr(el, "data-dfn-type")
                 if linkType not in config.dfnTypes:
                     m.die(f"Unknown local dfn type '{linkType}':\n  {h.outerHTML(el)}", el=el)
@@ -279,9 +301,12 @@ class ReferenceManager:
                 dfnForAttr = h.treeAttr(el, "data-dfn-for")
                 if dfnForAttr is None:
                     dfnFor: set[str] = set()
-                    existingRefs = self.localRefs.queryRefs(linkType=linkType, text=linkText, linkFor="/", exact=True)[
-                        0
-                    ]
+                    existingRefs, _, _ = self.localRefs.queryRefs(
+                        linkType=linkType,
+                        text=linkText,
+                        linkFor="/",
+                        exact=True,
+                    )
                     if existingRefs and existingRefs[0].el is not el:
                         m.die(f"Multiple local '{linkType}' <dfn>s have the same linking text '{linkText}'.", el=el)
                         continue
@@ -289,12 +314,12 @@ class ReferenceManager:
                     dfnFor = set(config.splitForValues(dfnForAttr))
                     encounteredError = False
                     for singleFor in dfnFor:
-                        existingRefs = self.localRefs.queryRefs(
+                        existingRefs, _, _ = self.localRefs.queryRefs(
                             linkType=linkType,
                             text=linkText,
                             linkFor=singleFor,
                             exact=True,
-                        )[0]
+                        )
                         if existingRefs and existingRefs[0].el is not el:
                             encounteredError = True
                             m.die(
@@ -311,9 +336,11 @@ class ReferenceManager:
                     if match:
                         dfnFor.add(match.group(1).strip())
                 # convert back into a list now, for easier JSONing
-                dfnForList = list(sorted(dfnFor))
+                dfnForList = sorted(dfnFor)
+                refKey, displayKey = config.adjustKey(linkText, linkType)
                 ref = wrapper.RefWrapper(
-                    linkText,
+                    refKey,
+                    displayKey,
                     {
                         "type": linkType,
                         "status": "local",
@@ -344,9 +371,9 @@ class ReferenceManager:
         )
 
     def queryAllRefs(self, **kwargs: t.Any) -> list[RefWrapper]:
-        r1, _ = self.localRefs.queryRefs(**kwargs)
-        r2, _ = self.anchorBlockRefs.queryRefs(**kwargs)
-        r3, _ = self.foreignRefs.queryRefs(**kwargs)
+        r1, _, _ = self.localRefs.queryRefs(**kwargs)
+        r2, _, _ = self.anchorBlockRefs.queryRefs(**kwargs)
+        r3, _, _ = self.foreignRefs.queryRefs(**kwargs)
         refs = r1 + r2 + r3
         if kwargs.get("ignoreObsoletes") is True:
             refs = self.filterObsoletes(refs)
@@ -364,9 +391,10 @@ class ReferenceManager:
         linkForHint: str | None = None,
         error: bool = True,
         el: t.ElementT | None = None,
-    ) -> RefWrapper | None:
+    ) -> RefWrapper | str | None:
         # If error is False, this function just shuts up and returns a reference or None
-        # Otherwise, it pops out debug messages for the user.
+        # Otherwise, it pops out debug messages for the user,
+        # and might return an error message as a str.
 
         # 'maybe' links might not link up, so it's fine for them to have no references.
         # The relevent errors are gated by this variable.
@@ -389,7 +417,7 @@ class ReferenceManager:
 
         # Local refs always get precedence, unless you manually specified a spec.
         if spec is None:
-            localRefs, _ = self.localRefs.queryRefs(
+            localRefs, _, _ = self.localRefs.queryRefs(
                 linkType=linkType,
                 text=text,
                 linkFor=linkFor,
@@ -401,18 +429,25 @@ class ReferenceManager:
             # but there was a for-less version in a foreign spec,
             # emit a warning (unless it was supressed).
             if localRefs and linkFor is None and any(x.for_ for x in localRefs):
-                forlessRefs, _ = self.anchorBlockRefs.queryRefs(
-                    linkType=linkType, text=text, linkFor="/", export=True, el=el
+                forlessRefs, _, _ = self.anchorBlockRefs.queryRefs(
+                    linkType=linkType,
+                    text=text,
+                    linkFor="/",
+                    export=True,
+                    el=el,
                 )
                 forlessRefs = self.filterObsoletes(forlessRefs)
                 if not forlessRefs:
-                    forlessRefs, _ = self.foreignRefs.queryRefs(
-                        linkType=linkType, text=text, linkFor="/", export=True, el=el
+                    forlessRefs, _, _ = self.foreignRefs.queryRefs(
+                        linkType=linkType,
+                        text=text,
+                        linkFor="/",
+                        export=True,
+                        el=el,
                     )
                 forlessRefs = self.filterObsoletes(forlessRefs)
                 if forlessRefs:
-                    reportAmbiguousForlessLink(el, text, forlessRefs, localRefs)
-                    return None
+                    return reportAmbiguousForlessLink(el, text, forlessRefs, localRefs)
             if len(localRefs) == 1:
                 return localRefs[0]
             elif len(localRefs) > 1:
@@ -421,7 +456,7 @@ class ReferenceManager:
                     chosenRef = sorted(localRefs, key=lambda x: x.url)[0]
                 else:
                     # CHAOS MODE (so you're less likely to rely on it)
-                    chosenRef = random.choice(localRefs)
+                    chosenRef = random.choice(localRefs)  # noqa: S311
                 if error:
                     m.linkerror(
                         f"Multiple possible '{linkType}' local refs for '{text}'.\nRandomly chose one of them; other instances might get a different random choice.",
@@ -451,15 +486,8 @@ class ReferenceManager:
                     linkType = dfnType
                     break
 
-        if spec == "css2":
-            # TEMP - moving data sources changed css2 to css22,
-            # so manually fix it up for a little while
-            spec = "css22"
-            if error:
-                m.linkerror("The spec 'css2' was renamed to 'css22'; please update your links.")
-
         # Then anchor-block refs get preference
-        blockRefs, _ = self.anchorBlockRefs.queryRefs(
+        blockRefs, _, _ = self.anchorBlockRefs.queryRefs(
             linkType=linkType,
             text=text,
             spec=spec,
@@ -469,11 +497,16 @@ class ReferenceManager:
             el=el,
         )
         if blockRefs and linkFor is None and any(x.for_ for x in blockRefs):
-            forlessRefs, _ = self.foreignRefs.queryRefs(linkType=linkType, text=text, linkFor="/", export=True, el=el)
+            forlessRefs, _, _ = self.foreignRefs.queryRefs(
+                linkType=linkType,
+                text=text,
+                linkFor="/",
+                export=True,
+                el=el,
+            )
             forlessRefs = self.filterObsoletes(forlessRefs)
             if forlessRefs:
-                reportAmbiguousForlessLink(el, text, forlessRefs, blockRefs)
-                return None
+                return reportAmbiguousForlessLink(el, text, forlessRefs, blockRefs)
         if len(blockRefs) == 1:
             return blockRefs[0]
         elif len(blockRefs) > 1:
@@ -492,7 +525,7 @@ class ReferenceManager:
             export = True
         else:
             export = None
-        refs, failure = self.foreignRefs.queryRefs(
+        refs, failure, oldRefs = self.foreignRefs.queryRefs(
             text=text,
             linkType=linkType,
             spec=spec,
@@ -554,7 +587,7 @@ class ReferenceManager:
                 for p in linkForPatterns:
                     for s in statuses:
                         for method in possibleMethods[0]:
-                            refs, failure = self.foreignRefs.queryRefs(
+                            refs, _, _ = self.foreignRefs.queryRefs(
                                 text=text,
                                 linkType=linkType,
                                 spec=spec,
@@ -573,11 +606,11 @@ class ReferenceManager:
 
             # Allow foo(bar) to be for'd to with just foo() if it's completely unambiguous.
             methodPrefix = methodName[:-1]
-            candidates, _ = self.localRefs.queryRefs(linkType="functionish", linkFor=interfaceName)
+            candidates, _, _ = self.localRefs.queryRefs(linkType="functionish", linkFor=interfaceName)
             methodRefs = list({c.url: c for c in candidates if c.text.startswith(methodPrefix)}.values())
             if not methodRefs:
                 # Look for non-locals, then
-                c1, _ = self.anchorBlockRefs.queryRefs(
+                c1, _, _ = self.anchorBlockRefs.queryRefs(
                     linkType="functionish",
                     spec=spec,
                     status=status,
@@ -586,7 +619,7 @@ class ReferenceManager:
                     export=export,
                     ignoreObsoletes=True,
                 )
-                c2, _ = self.foreignRefs.queryRefs(
+                c2, _, _ = self.foreignRefs.queryRefs(
                     linkType="functionish",
                     spec=spec,
                     status=status,
@@ -599,54 +632,23 @@ class ReferenceManager:
                 methodRefs = list({c.url: c for c in candidates if c.text.startswith(methodPrefix)}.values())
             if zeroRefsError and len(methodRefs) > 1:
                 # More than one possible foo() overload, can't tell which to link to
-                m.linkerror(
-                    f"Too many possible method targets to disambiguate '{linkFor}/{text}'. Please specify the names of the required args, like 'foo(bar, baz)', in the 'for' attribute.",
-                    el=el,
-                )
-                return None
+                msg = f"Too many possible method targets to disambiguate '{linkFor}/{text}'. Please specify the names of the required args, like 'foo(bar, baz)', in the 'for' attribute."
+                m.linkerror(msg, el=el)
+                return msg
             # Otherwise
 
-        if failure in ("text", "type"):
-            if linkType in ("property", "propdesc", "descriptor") and text.startswith("--"):
-                # Custom properties/descriptors aren't ever defined anywhere
-                return None
-            if zeroRefsError:
-                m.linkerror(f"No '{linkType}' refs found for '{text}'.", el=el)
-            return None
-        elif failure == "export":
-            if zeroRefsError:
-                m.linkerror(f"No '{linkType}' refs found for '{text}' that are marked for export.", el=el)
-            return None
-        elif failure == "spec":
-            if zeroRefsError:
-                m.linkerror(f"No '{linkType}' refs found for '{text}' with spec '{spec}'.", el=el)
-            return None
-        elif failure == "for":
-            if zeroRefsError:
-                if spec is None:
-                    m.linkerror(f"No '{linkType}' refs found for '{text}' with for='{linkFor}'.", el=el)
-                else:
-                    m.linkerror(
-                        f"No '{linkType}' refs found for '{text}' with for='{linkFor}' in spec '{spec}'.", el=el
-                    )
-            return None
-        elif failure == "status":
-            if zeroRefsError:
-                if spec is None:
-                    m.linkerror(f"No '{linkType}' refs found for '{text}' compatible with status '{status}'.", el=el)
-                else:
-                    m.linkerror(
-                        f"No '{linkType}' refs found for '{text}' compatible with status '{status}' in spec '{spec}'.",
-                        el=el,
-                    )
-            return None
-        elif failure == "ignored-specs":
-            if zeroRefsError:
-                m.linkerror(f"The only '{linkType}' refs for '{text}' were in ignored specs:\n{h.outerHTML(el)}", el=el)
-            return None
-        elif failure:
-            m.die(f"Programming error - I'm not catching '{failure}'-type link failures. Please report!", el=el)
-            return None
+        if failure:
+            return errorFromFailureCategory(
+                failure,
+                oldRefs,
+                zeroRefsError,
+                linkType=linkType,
+                text=text,
+                linkFor=linkFor,
+                spec=spec,
+                status=status,
+                el=el,
+            )
 
         if len(refs) == 1:
             # Success!
@@ -720,7 +722,7 @@ class ReferenceManager:
             if failFromWrongSuffix and not quiet:
                 numericSuffixes = self.biblioNumericSuffixes[unversionedKey]
                 m.die(
-                    f"A biblio link references {text}, but only {config.englishFromList(numericSuffixes)} exists in SpecRef."
+                    f"A biblio link references {text}, but only {config.englishFromList(numericSuffixes)} exists in SpecRef.",
                 )
             return None
 
@@ -752,12 +754,12 @@ class ReferenceManager:
                 if newBib is None:
                     if not quiet:
                         m.die(
-                            f"[{bib.linkText}] claims to be obsoleted by [{bib.obsoletedBy}], which doesn't exist. Either change the reference, of use [{bib.linkText} obsolete] to ignore the obsoletion chain."
+                            f"[{bib.linkText}] claims to be obsoleted by [{bib.obsoletedBy}], which doesn't exist. Either change the reference, of use [{bib.linkText} obsolete] to ignore the obsoletion chain.",
                         )
                     return None
                 if not quiet:
                     m.linkerror(
-                        f"Obsolete biblio ref: [{bib.linkText}] is replaced by [{newBib.linkText}]. Either update the reference, or use [{bib.linkText} obsolete] if this is an intentionally-obsolete reference."
+                        f"Obsolete biblio ref: [{bib.linkText}] is replaced by [{newBib.linkText}]. Either update the reference, or use [{bib.linkText} obsolete] if this is an intentionally-obsolete reference.",
                     )
                 bib = newBib
 
@@ -851,7 +853,7 @@ def simplifyPossibleRefs(refs: list[RefWrapper], alwaysShowFor: bool = False) ->
                     spec=spec,
                     for_=None,
                     url=fors[0][1],
-                )
+                ),
             )
     return retRefs
 
@@ -879,7 +881,7 @@ def reportMultiplePossibleRefs(
     linkFor: str | list[str] | None,
     defaultRef: RefWrapper,
     el: t.ElementT | None,
-) -> None:
+) -> str:
     # Sometimes a badly-written spec has indistinguishable dfns.
     # Detect this by seeing if more than one stringify to the same thing.
     allRefs = defaultdict(list)
@@ -908,14 +910,78 @@ def reportMultiplePossibleRefs(
             for ref in refs:
                 error += "\n  " + ref.url
     m.linkerror(error, el=el)
+    return error
 
 
 def reportAmbiguousForlessLink(
-    el: t.ElementT | None, text: str, forlessRefs: list[RefWrapper], localRefs: list[RefWrapper]
-) -> None:
+    el: t.ElementT | None,
+    text: str,
+    forlessRefs: list[RefWrapper],
+    localRefs: list[RefWrapper],
+) -> str:
     localRefText = "\n".join([refToText(ref) for ref in simplifyPossibleRefs(localRefs, alwaysShowFor=True)])
     forlessRefText = "\n".join([refToText(ref) for ref in simplifyPossibleRefs(forlessRefs, alwaysShowFor=True)])
+    msg = f"Ambiguous for-less link for '{text}', please see <https://speced.github.io/bikeshed/#ambi-for> for instructions:\nLocal references:\n{localRefText}\nfor-less references:\n{forlessRefText}"
     m.linkerror(
-        f"Ambiguous for-less link for '{text}', please see <https://speced.github.io/bikeshed/#ambi-for> for instructions:\nLocal references:\n{localRefText}\nfor-less references:\n{forlessRefText}",
+        msg,
         el=el,
     )
+    return msg
+
+
+def errorFromFailureCategory(
+    failure: LinkFailure,
+    oldRefs: list[t.RefWrapper],
+    zeroRefsError: bool,
+    linkType: str,
+    text: str,
+    linkFor: str | list[str] | None,
+    spec: str | None,
+    status: str | None,
+    el: t.ElementT | None,
+) -> str | None:
+    if not zeroRefsError:
+        return None
+    if failure == LinkFailure.Text or failure == LinkFailure.Type:  # noqa: PLR1714
+        if linkType in ("property", "propdesc", "descriptor") and text.startswith("--"):
+            # Custom properties/descriptors aren't ever defined anywhere
+            return None
+        msg = f"No '{linkType}' refs found for '{text}'."
+        m.linkerror(msg, el=el)
+        return msg
+    elif failure == LinkFailure.Export:
+        msg = f"No '{linkType}' refs found for '{text}' that are marked for export."
+        if oldRefs:
+            msg += "\n  (Possible specs this could be from: "
+            msg += config.englishFromList(sorted({x.spec for x in oldRefs if x.spec is not None}), "or")
+            msg += ")"
+        m.linkerror(msg, el=el)
+        return msg
+    elif failure == LinkFailure.Spec:
+        msg = f"No '{linkType}' refs found for '{text}' with spec '{spec}'."
+        m.linkerror(msg, el=el)
+        return msg
+    elif failure == LinkFailure.For:
+        if spec is None:
+            msg = f"No '{linkType}' refs found for '{text}' with for='{linkFor}'."
+            m.linkerror(msg, el=el)
+            return msg
+        else:
+            msg = f"No '{linkType}' refs found for '{text}' with for='{linkFor}' in spec '{spec}'."
+            m.linkerror(msg, el=el)
+            return msg
+    elif failure == LinkFailure.Status:
+        if spec is None:
+            msg = f"No '{linkType}' refs found for '{text}' compatible with status '{status}'."
+            m.linkerror(msg, el=el)
+            return msg
+        else:
+            msg = f"No '{linkType}' refs found for '{text}' compatible with status '{status}' in spec '{spec}'."
+            m.linkerror(msg, el=el)
+            return msg
+    elif failure == LinkFailure.IgnoredSpecs:
+        msg = f"The only '{linkType}' refs for '{text}' were in ignored specs:\n{h.outerHTML(el)}"
+        m.linkerror(msg, el=el)
+        return msg
+    else:
+        t.assert_never(failure)
