@@ -5,25 +5,28 @@ import dataclasses
 import re
 from collections import defaultdict
 
-from .. import config, constants, messages as m, retrieve, t
+from .. import config, constants, retrieve, t
+from .. import messages as m
 from . import utils, wrapper
+from .utils import LinkFailure
+
+LAZY_LOADED_SOURCES: list[str] = ["foreign"]
 
 
 class RefSource:
     __slots__ = [
-        "dataFile",
-        "source",
-        "refs",
-        "methods",
-        "fors",
-        "specs",
-        "ignoredSpecs",
-        "replacedSpecs",
         "_loadedAnchorGroups",
+        "dataFile",
+        "fors",
+        "ignoredSpecs",
+        "methods",
+        "refs",
+        "replacedSpecs",
+        "source",
+        "specs",
     ]
 
     # Which sources use lazy-loading; other sources always have all their refs loaded immediately.
-    lazyLoadedSources: list[str] = ["foreign"]
 
     def __init__(
         self,
@@ -32,7 +35,7 @@ class RefSource:
         ignored: set[str] | None = None,
         replaced: set[tuple[str, str]] | None = None,
         fileRequester: t.DataFileRequester | None = None,
-    ):
+    ) -> None:
         if fileRequester is None:
             self.dataFile = retrieve.defaultRequester
         else:
@@ -61,7 +64,7 @@ class RefSource:
         if key in self.refs:
             return self.refs[key]
 
-        if self.source not in self.lazyLoadedSources:
+        if self.source not in LAZY_LOADED_SOURCES:
             return []
 
         group = config.groupFromKey(key)
@@ -77,7 +80,7 @@ class RefSource:
     def fetchAllRefs(self) -> list[tuple[str, list[t.RefWrapper]]]:
         """Nuts to lazy-loading, just load everything at once."""
 
-        if self.source not in self.lazyLoadedSources:
+        if self.source not in LAZY_LOADED_SOURCES:
             return list(self.refs.items())
 
         for file in self.dataFile.walkFiles("anchors"):
@@ -107,7 +110,7 @@ class RefSource:
         exact: bool | None = None,
         error: bool = False,
         el: t.ElementT | None = None,
-    ) -> tuple[list[t.RefWrapper], str | None]:
+    ) -> tuple[list[t.RefWrapper], LinkFailure | None, list[t.RefWrapper]]:
         if exact is not None:
             return self._queryRefs(
                 text=text,
@@ -128,7 +131,7 @@ class RefSource:
             )
         else:
             # First search for the exact term, and only if it fails fall back to conjugating.
-            results, errorCode = self._queryRefs(
+            results, linkFailure, _ = self._queryRefs(
                 text=text,
                 spec=spec,
                 linkType=linkType,
@@ -145,7 +148,7 @@ class RefSource:
                 el=el,
                 exact=True,
             )
-            if errorCode:
+            if linkFailure:
                 return self._queryRefs(
                     text=text,
                     spec=spec,
@@ -164,7 +167,25 @@ class RefSource:
                     exact=False,
                 )
             else:
-                return results, None
+                return results, None, []
+
+    def allRefsIterator(self) -> t.Generator[t.RefWrapper, None, None]:
+        # Turns a dict of arrays of refs into an iterator of refs
+        for _, group in self.fetchAllRefs():
+            yield from group
+
+    def textRefsIterator(self, texts: list[str]) -> t.Generator[t.RefWrapper, None, None]:
+        # Same as above, but only grabs those keyed to a given text
+        for x in texts:
+            yield from self.fetchRefs(x)
+
+    def forRefsIterator(self, targetFors: str | list[str]) -> t.Generator[t.RefWrapper, None, None]:
+        # Same as above, but only grabs those for certain values
+        if isinstance(targetFors, str):
+            targetFors = [targetFors]
+        for for_ in targetFors:
+            for x in self.fors[for_]:
+                yield from self.fetchRefs(x)
 
     def _queryRefs(
         self,
@@ -183,48 +204,34 @@ class RefSource:
         exact: bool = False,
         error: bool = False,
         el: t.ElementT | None = None,
-    ) -> tuple[list[t.RefWrapper], str | None]:
+    ) -> tuple[list[t.RefWrapper], LinkFailure | None, list[t.RefWrapper]]:
         # Query the ref database.
         # If it fails to find a ref, also returns the stage at which it finally ran out of possibilities.
-        def allRefsIterator() -> t.Generator[t.RefWrapper, None, None]:
-            # Turns a dict of arrays of refs into an iterator of refs
-            for _, group in self.fetchAllRefs():
-                for ref in group:
-                    yield ref
-
-        def textRefsIterator(texts: list[str]) -> t.Generator[t.RefWrapper, None, None]:
-            # Same as above, but only grabs those keyed to a given text
-            for text in texts:
-                for ref in self.fetchRefs(text):
-                    yield ref
-
-        def forRefsIterator(targetFors: str | list[str]) -> t.Generator[t.RefWrapper, None, None]:
-            # Same as above, but only grabs those for certain values
-            if isinstance(targetFors, str):
-                targetFors = [targetFors]
-            for for_ in targetFors:
-                for text in self.fors[for_]:
-                    for ref in self.fetchRefs(text):
-                        yield ref
 
         # Set up the initial list of refs to query
         if text:
             if exact:
-                refs = list(textRefsIterator([text]))
+                refs = list(self.textRefsIterator([text]))
             else:
                 textsToSearch = list(utils.linkTextVariations(text, linkType))
-                if text.endswith("()") and text in self.methods:
-                    textsToSearch += list(self.methods[text].variants.keys())
+
+                if any(st.endswith("()") for st in textsToSearch):
+                    # Let argless methods (either with () at the end, or no parens at all)
+                    # pull in their argful variants for searching.
+                    for st in textsToSearch[:]:
+                        textsToSearch += getArgfulMethodVariants(st, self)
+
                 if (linkType is None or linkType in config.lowercaseTypes) and text.lower() != text:
                     textsToSearch += [t.lower() for t in textsToSearch]
-                refs = list(textRefsIterator(textsToSearch))
+                refs = list(self.textRefsIterator(textsToSearch))
         elif linkFor:
-            refs = list(forRefsIterator(linkFor))
+            refs = list(self.forRefsIterator(linkFor))
         else:
-            refs = list(allRefsIterator())
+            refs = list(self.allRefsIterator())
         if not refs:
-            return refs, "text"
+            return [], LinkFailure.Text, []
 
+        oldRefs = refs
         if linkType:
             if linkType in config.dfnTypes:
                 linkTypes = [linkType]
@@ -233,20 +240,22 @@ class RefSource:
             else:
                 if error:
                     m.linkerror(f"Unknown link type '{linkType}'.", el=el)
-                return [], "type"
+                return [], LinkFailure.Type, []
             refs = [x for x in refs if x.type in linkTypes]
         if not refs:
-            return refs, "type"
+            return [], LinkFailure.Type, oldRefs
 
+        oldRefs = refs
         if export is not None:
             refs = [x for x in refs if x.export == export]
         if not refs:
-            return refs, "export"
+            return [], LinkFailure.Export, oldRefs
 
+        oldRefs = refs
         if spec:
             refs = [x for x in refs if spec in (x.spec, x.shortname)]
         if not refs:
-            return refs, "spec"
+            return [], LinkFailure.Spec, oldRefs
 
         """
         for=A | forHint=B | explicitFor
@@ -260,18 +269,7 @@ class RefSource:
         ✔ | ✔ | ✔ = A/
         """
 
-        def filterByFor(refs: t.Sequence[t.RefWrapper], linkFor: str | list[str]) -> list[t.RefWrapper]:
-            return [x for x in refs if matchFor(x.for_, linkFor)]
-
-        def matchFor(forVals: list[str], forTest: str | list[str]) -> bool:
-            # forTest can be a string, either / for no for or the for value to match,
-            # or an array of strings, of which any can match
-            if forTest == "/":
-                return not bool(forVals)
-            if isinstance(forTest, str):
-                return forTest in forVals
-            return any(matchFor(forVals, ft) for ft in forTest)
-
+        oldRefs = refs
         if linkFor:
             refs = filterByFor(refs, linkFor)
         elif linkForHint:
@@ -286,39 +284,16 @@ class RefSource:
         elif explicitFor:
             refs = filterByFor(refs, "/")
         if not refs:
-            return refs, "for"
-
-        def filterByStatus(refs: t.Sequence[t.RefWrapper], status: str) -> list[t.RefWrapper]:
-            if status in constants.refStatus:
-                # If status is "current'", kill snapshot refs unless their spec *only* has a snapshot_url
-                if status == constants.refStatus.current:
-                    return [
-                        ref
-                        for ref in refs
-                        if ref.status == "current"
-                        or (ref.status == "snapshot" and self.specs.get(ref.spec, {}).get("current_url") is None)
-                    ]
-                # If status is "snapshot", kill current refs if there's a corresponding snapshot ref for the same spec.
-                elif status == constants.refStatus.snapshot:
-                    snapshotSpecs = [ref.spec for ref in refs if ref.status == "snapshot"]
-                    return [
-                        ref
-                        for ref in refs
-                        if ref.status == "snapshot" or (ref.status == "current" and ref.spec not in snapshotSpecs)
-                    ]
-            # Status is a non-refStatus, but is a valid linkStatus, like "local"
-            elif status in config.linkStatuses:
-                return [x for x in refs if x.status == status]
-
-            raise Exception(f"Status value of '{status}' not handled")
+            return [], LinkFailure.For, oldRefs
 
+        oldRefs = refs
         if status:
-            refs = filterByStatus(refs, status)
+            refs = filterByStatus(self, refs, status)
         if not refs:
-            return refs, "status"
+            return [], LinkFailure.Status, refs
 
         if status is None and statusHint:
-            tempRefs = filterByStatus(refs, statusHint)
+            tempRefs = filterByStatus(self, refs, statusHint)
             if tempRefs:
                 refs = tempRefs
 
@@ -327,12 +302,13 @@ class RefSource:
             if tempRefs:
                 refs = tempRefs
 
+        oldRefs = refs
         if ignoreObsoletes and not spec:
             # Remove any ignored or obsoleted specs
             # If you specified the spec, don't filter things - you know what you're doing.
             refs = utils.filterObsoletes(refs, replacedSpecs=self.replacedSpecs, ignoredSpecs=self.ignoredSpecs)
         if not refs:
-            return refs, "ignored-specs"
+            return [], LinkFailure.IgnoredSpecs, oldRefs
 
         if dedupURLs:
             # With non-exact texts, you might have multiple "anchors"
@@ -353,7 +329,7 @@ class RefSource:
             # unless that doesn't exist, in which case just prefer the latest level.
             refs = utils.filterOldVersions(refs, status=status or statusHint)
 
-        return refs, None
+        return refs, None, []
 
     def addMethodVariants(self, methodSig: str, forVals: t.Iterable[str], shortname: str | None) -> None:
         # Takes a full method signature, like "foo(bar)",
@@ -373,13 +349,54 @@ class RefSource:
         variants[methodSig].for_.extend(forVals)
 
 
+def filterByFor(refs: t.Sequence[t.RefWrapper], linkFor: str | list[str]) -> list[t.RefWrapper]:
+    return [x for x in refs if matchFor(x.for_, linkFor)]
+
+
+def matchFor(forVals: list[str], forTest: str | list[str]) -> bool:
+    # forTest can be a string, either / for no for or the for value to match,
+    # or an array of strings, of which any can match
+    if forTest == "/":
+        return not bool(forVals)
+    if isinstance(forTest, str):
+        return forTest in forVals
+    return any(matchFor(forVals, ft) for ft in forTest)
+
+
+def filterByStatus(source: RefSource, refs: t.Sequence[t.RefWrapper], status: str) -> list[t.RefWrapper]:
+    if status in constants.refStatus:
+        # If status is "current'", kill snapshot refs unless their spec *only* has a snapshot_url
+        if status == constants.refStatus.current:
+            return [
+                ref
+                for ref in refs
+                if ref.status == "current"
+                or (ref.status == "snapshot" and source.specs.get(ref.spec, {}).get("current_url") is None)
+            ]
+        # If status is "snapshot", kill current refs if there's a corresponding snapshot ref for the same spec.
+        elif status == constants.refStatus.snapshot:
+            snapshotSpecs = [ref.spec for ref in refs if ref.status == "snapshot"]
+            return [
+                ref
+                for ref in refs
+                if ref.status == "snapshot" or (ref.status == "current" and ref.spec not in snapshotSpecs)
+            ]
+    # Status is a non-refStatus, but is a valid linkStatus, like "local"
+    elif status in config.linkStatuses:
+        return [x for x in refs if x.status == status]
+
+    msg = f"Status value of '{status}' not handled"
+    raise Exception(msg)
+
+
 def decodeAnchors(linesIter: t.Iterator[str]) -> defaultdict[str, list[t.RefWrapper]]:
     # Decodes the anchor storage format into {key: [{anchor-data}]}
     anchors = defaultdict(list)
     try:
         while True:
-            key = next(linesIter)[:-1]
-            a: wrapper.RefDataT = {
+            aText = next(linesIter)[:-1]
+            displayText = next(linesIter)[:-1]
+            data: wrapper.RefDataT = {
                 "type": next(linesIter),
                 "spec": next(linesIter),
                 "shortname": next(linesIter),
@@ -394,12 +411,18 @@ def decodeAnchors(linesIter: t.Iterator[str]) -> defaultdict[str, list[t.RefWrap
                 line = next(linesIter)
                 if line == "-\n":
                     break
-                t.cast("list", a["for_"]).append(line)
-            anchors[key].append(wrapper.RefWrapper(key, a))
+                t.cast("list", data["for_"]).append(line)
+            anchors[aText].append(wrapper.RefWrapper(aText, displayText, data))
     except StopIteration:
         return anchors
 
 
+def getArgfulMethodVariants(maybeMethodSig: str, refs: RefSource) -> list[str]:
+    if maybeMethodSig.endswith("()") and maybeMethodSig in refs.methods:
+        return list(refs.methods[maybeMethodSig].variants.keys())
+    return []
+
+
 @dataclasses.dataclass
 class MethodVariants:
     arglessSignature: str
