@@ -4,20 +4,19 @@ import dataclasses
 
 from .. import t
 
-if t.TYPE_CHECKING:
-    # Need to use function form due to "for" key
-    # being invalid as a property name
-    class RefDataT(t.TypedDict, total=False):
-        type: t.Required[str]
-        spec: t.Required[str | None]
-        shortname: t.Required[str | None]
-        level: t.Required[str | None]
-        status: t.Required[str]
-        url: t.Required[str]
-        export: t.Required[bool]
-        normative: t.Required[bool]
-        for_: t.Required[list[str]]
-        el: t.ElementT | None
+
+class RefDataT(t.TypedDict, total=False):
+    type: t.Required[str]
+    spec: t.Required[str | None]
+    shortname: t.Required[str | None]
+    level: t.Required[str | None]
+    status: t.Required[str]
+    url: t.Required[str]
+    export: t.Required[bool]
+    normative: t.Required[bool]
+    for_: t.Required[list[str]]
+    uniquifier: str | None
+    el: t.ElementT | None
 
 
 @dataclasses.dataclass
@@ -27,6 +26,7 @@ class RefWrapper:
     # It also makes all the ref dict keys look like object attributes.
 
     text: str
+    displayText: str
     _ref: RefDataT
 
     @property
@@ -79,9 +79,25 @@ class RefWrapper:
     def el(self) -> t.ElementT | None:
         return self._ref.get("el", None)
 
+    @property
+    def uniquifier(self) -> str | None:
+        return self._ref.get("uniquifier", None)
+
+    def refKey(self) -> str:
+        return self.uniquifier or self.url
+
+    def __eq__(self, other: t.Any) -> bool:
+        if not isinstance(other, RefWrapper):
+            return NotImplemented
+        return self._ref == other._ref
+
+    def __hash__(self) -> int:
+        return hash(frozenset(self._ref))
+
     def __json__(self) -> t.JSONT:
         return {
             "text": self.text,
+            "displayText": self.displayText,
             "type": self.type,
             "spec": self.spec,
             "shortname": self.shortname,
