@@ -2,8 +2,17 @@
 # Module for holding types, for easy importing into the rest of the codebase
 from __future__ import annotations
 
-# The only three things that should be available during runtime.
-from typing import TYPE_CHECKING, cast, overload
+import sys
+
+# The only things that should be available during runtime.
+from typing import TYPE_CHECKING, Generic, TypedDict, TypeVar, cast, overload
+
+# Only available in 3.11, so stub them out for earlier versions
+if sys.version_info >= (3, 11):
+    from typing import assert_never, assert_type
+else:
+    from typing_extensions import assert_never, assert_type
+
 
 if TYPE_CHECKING:
     from typing import (
@@ -12,11 +21,11 @@ if TYPE_CHECKING:
         AnyStr,
         Awaitable,
         Callable,
+        Collection,
         DefaultDict,
         Deque,
         FrozenSet,
         Generator,
-        Generic,
         Iterable,
         Iterator,
         Literal,
@@ -28,18 +37,19 @@ if TYPE_CHECKING:
         Protocol,
         Sequence,
         TextIO,
+        Type,
         TypeAlias,
-        TypedDict,
         TypeGuard,
-        TypeVar,
     )
-    from typing_extensions import (
-        Required,
-        NotRequired,
-    )
-    from _typeshed import SupportsKeysAndGetItem
 
+    from _typeshed import SupportsKeysAndGetItem
     from lxml import etree
+    from typing_extensions import (
+        NotRequired,
+        Required,
+        Self,
+        TypeIs,
+    )
 
     ElementT: TypeAlias = etree._Element
     DocumentT: TypeAlias = etree._ElementTree
@@ -55,18 +65,18 @@ if TYPE_CHECKING:
     # Similar for JSON
     JSONT: TypeAlias = dict[str, Any]
 
-    from types import ModuleType
-
     import sys
+    from types import ModuleType
 
     if "Spec" not in sys.modules:
         from .Spec import Spec
     SpecT = Spec
 
     from .biblio import BiblioEntry
-    from .retrieve import DataFileRequester
+    from .config.BoolSet import BoolSet
     from .metadata import MetadataManager
-    from .refs import RefSource, ReferenceManager, RefWrapper, MethodVariants, MethodVariant
+    from .refs import MethodVariant, MethodVariants, ReferenceManager, RefSource, RefWrapper
+    from .retrieve import DataFileRequester
 
     BiblioStorageT: TypeAlias = DefaultDict[str, list[BiblioEntry]]
 
