@@ -1,5 +1,5 @@
 from __future__ import annotations
 
 
-def _(input: str) -> str:
+def _t(input: str) -> str:
     return input
