@@ -5,88 +5,21 @@ import logging
 import re
 from collections import Counter, defaultdict, namedtuple
 from urllib import parse
+
+import requests
 from PIL import Image
 
-from . import biblio, config, dfnpanels, h, func, t, messages as m, idl, repository
-from .translate import _
+from . import biblio, cddl, config, dfnpanels, h, idl, printjson, repository, t
+from . import messages as m
+from .translate import _t
 
 if t.TYPE_CHECKING:
     import widlparser  # pylint: disable=unused-import
+
     from . import refs
     from .line import Line
 
 
-class MarkdownCodeSpans(func.Functor):
-    # Wraps a string, such that the contained text is "safe"
-    # and contains no markdown code spans.
-    # Thus, functions mapping over the text can freely make substitutions,
-    # knowing they won't accidentally replace stuff in a code span.
-    def __init__(self, text: str):
-        self.__codeSpanReplacements__ = []
-        newText = ""
-        mode = "text"
-        indexSoFar = 0
-        backtickCount = 0
-        for match in re.finditer(r"(\\`)|([\w-]*)(`+)", text):
-            if mode == "text":
-                if match.group(1):
-                    newText += text[indexSoFar : match.start()] + match.group(1)[1]
-                    indexSoFar = match.end()
-                elif match.group(3):
-                    mode = "code"
-                    newText += text[indexSoFar : match.start()]
-                    indexSoFar = match.end()
-                    backtickCount = len(match.group(3))
-                    tag = match.group(2)
-                    literalStart = match.group(0)
-            elif mode == "code":
-                if match.group(1):
-                    pass
-                elif match.group(3):
-                    if len(match.group(3)) != backtickCount:
-                        pass
-                    else:
-                        mode = "text"
-                        innerText = text[indexSoFar : match.start()] + match.group(2)
-                        fullText = literalStart + text[indexSoFar : match.start()] + match.group(0)
-                        replacement = (tag, innerText, fullText)
-                        self.__codeSpanReplacements__.append(replacement)
-                        newText += "\ue0ff"
-                        indexSoFar = match.end()
-        if mode == "text":
-            newText += text[indexSoFar:]
-        elif mode == "code":
-            newText += tag + "`" * backtickCount + text[indexSoFar:]
-
-        super().__init__(newText)
-
-    def map(self, fn: t.Callable) -> MarkdownCodeSpans:
-        x = MarkdownCodeSpans("")
-        x.__val__ = fn(self.__val__)
-        x.__codeSpanReplacements__ = self.__codeSpanReplacements__
-        return x
-
-    def extract(self) -> str:
-        if self.__codeSpanReplacements__:
-            # Reverse the list, so I can use pop() to get them starting from the first.
-            repls = self.__codeSpanReplacements__[::-1]
-
-            def codeSpanReviver(_: t.Any) -> str:
-                # Match object is the PUA character, which I can ignore.
-                repl = repls.pop()
-                if repl[0] == "" or repl[0].endswith("-"):
-                    # Markdown code span, so massage per CommonMark rules.
-                    import string
-
-                    text = h.escapeHTML(repl[1]).strip(string.whitespace)
-                    text = re.sub("[" + string.whitespace + "]{2,}", " ", text)
-                    return f"{repl[0]}<code data-opaque bs-autolink-syntax='{h.escapeAttr(repl[2])}'>{text}</code>"
-                return f"<code data-opaque data-span-tag={repl[0]} bs-autolink-syntax='{h.escapeAttr(repl[2])}'>{h.escapeHTML(repl[1])}</code>"
-
-            return re.sub(r"\ue0ff", codeSpanReviver, self.__val__)
-        return t.cast(str, self.__val__)
-
-
 def stripBOM(doc: t.SpecT) -> None:
     if len(doc.lines) >= 1 and doc.lines[0].text[0:1] == "\ufeff":
         doc.lines[0].text = doc.lines[0].text[1:]
@@ -119,60 +52,68 @@ def fixManualDefTables(doc: t.SpecT) -> None:
         h.replaceContents(cell, newContents)
 
 
-def canonicalizeShortcuts(doc: t.SpecT) -> None:
+def canonicalizeShortcuts(el: t.ElementT) -> None:
     # Take all the invalid-HTML shortcuts and fix them.
+    h.renameAttr(el, "export", "data-export")
+    h.renameAttr(el, "noexport", "data-noexport")
+    h.renameAttr(el, "link-spec", "data-link-spec")
+    h.renameAttr(el, "spec", "data-link-spec")
+    h.renameAttr(el, "link-status", "data-link-status")
+    h.renameAttr(el, "status", "data-link-status")
+    h.renameAttr(el, "dfn-for", "data-dfn-for")
+    h.renameAttr(el, "link-for", "data-link-for")
+    h.renameAttr(el, "link-for-hint", "data-link-for-hint")
+    h.renameAttr(el, "dfn-type", "data-dfn-type")
+    h.renameAttr(el, "link-type", "data-link-type")
+    h.renameAttr(el, "dfn-force", "data-dfn-force")
+    h.renameAttr(el, "force", "data-dfn-force")
+    h.renameAttr(el, "section", "data-section")
+    h.renameAttr(el, "attribute-info", "data-attribute-info")
+    h.renameAttr(el, "dict-member-info", "data-dict-member-info")
+    h.renameAttr(el, "lt", "data-lt")
+    h.renameAttr(el, "local-lt", "data-local-lt")
+    h.renameAttr(el, "algorithm", "data-algorithm")
+    h.renameAttr(el, "ignore", "data-var-ignore")
 
-    attrFixup = {
-        "export": "data-export",
-        "noexport": "data-noexport",
-        "link-spec": "data-link-spec",
-        "spec": "data-link-spec",
-        "link-status": "data-link-status",
-        "status": "data-link-status",
-        "dfn-for": "data-dfn-for",
-        "link-for": "data-link-for",
-        "link-for-hint": "data-link-for-hint",
-        "dfn-type": "data-dfn-type",
-        "link-type": "data-link-type",
-        "dfn-force": "data-dfn-force",
-        "force": "data-dfn-force",
-        "section": "data-section",
-        "attribute-info": "data-attribute-info",
-        "dict-member-info": "data-dict-member-info",
-        "lt": "data-lt",
-        "local-lt": "data-local-lt",
-        "algorithm": "data-algorithm",
-        "ignore": "data-var-ignore",
-    }
-    for el in h.findAll(",".join(f"[{attr}]" for attr in attrFixup), doc):
-        for attr, fixedAttr in attrFixup.items():
-            if el.get(attr) is not None:
-                el.set(fixedAttr, t.cast(str, el.get(attr)))
-                del el.attrib[attr]
-
-    # The next two aren't in the above dict because some of the words conflict with existing attributes on some elements.
+    # Some of these attrs conflict with existing real attributes on some elements.
     # Instead, limit the search/transforms to the relevant elements.
-    for el in h.findAll("dfn, h2, h3, h4, h5, h6", doc):
+    tag = h.tagName(el)
+    if tag in ("dfn", "h2", "h3", "h4", "h5", "h6"):
         for dfnType in config.dfnTypes:
-            if el.get(dfnType) == "":
-                del el.attrib[dfnType]
-                el.set("data-dfn-type", dfnType)
-                break
-    for el in h.findAll("a", doc):
+            if h.hasAttr(el, dfnType):
+                if el.get(dfnType) == "":
+                    h.removeAttr(el, dfnType)
+                    el.set("data-dfn-type", dfnType)
+                    break
+                elif dfnType == "type" and el.get(dfnType) in config.dfnTypes:
+                    realDfnType = el.get(dfnType)
+                    m.die(
+                        f"You used type={realDfnType}, which isn't valid. Instead use just {realDfnType} as a boolean attribute, or dfn-type={realDfnType}.",
+                        el=el,
+                    )
+                    h.removeAttr(el, "type")
+                    break
+
+    if tag == "a":
         for linkType in config.linkTypes:
-            if el.get(linkType) is not None:
-                del el.attrib[linkType]
+            if h.hasAttr(el, linkType):
+                h.removeAttr(el, linkType)
                 el.set("data-link-type", linkType)
                 break
-    for el in h.findAll(config.dfnElementsSelector + ", a", doc):
-        if el.get("for") is None:
-            continue
-        _for = t.cast(str, el.get("for"))
-        del el.attrib["for"]
-        if el.tag == "a":
-            el.set("data-link-for", _for)
-        else:
-            el.set("data-dfn-for", _for)
+
+    if (
+        tag == "a"
+        or (tag == "dfn" and not h.hasAttr(el, "data-var-ignore"))
+        or (tag in ("h2", "h3", "h4", "h5", "h6") and h.hasAttr(el, "data-dfn-type"))
+    ):
+        if h.hasAttr(el, "for"):
+            if el.tag == "a":
+                h.renameAttr(el, "for", "data-link-for")
+            else:
+                h.renameAttr(el, "for", "data-dfn-for")
+
+    for child in h.childElements(el):
+        canonicalizeShortcuts(child)
 
 
 def addImplicitAlgorithms(doc: t.SpecT) -> None:
@@ -182,7 +123,7 @@ def addImplicitAlgorithms(doc: t.SpecT) -> None:
     for el in h.findAll("[data-algorithm='']:not(h1):not(h2):not(h3):not(h4):not(h5):not(h6)", doc):
         dfns = h.findAll("dfn", el)
         if len(dfns) == 1:
-            dfnName = config.firstLinkTextFromElement(dfns[0]) or ""
+            dfnName = h.firstLinkTextFromElement(dfns[0]) or ""
             el.set("data-algorithm", dfnName)
             dfnFor = dfns[0].get("data-dfn-for")
             if dfnFor:
@@ -214,6 +155,7 @@ def checkVarHygiene(doc: t.SpecT) -> None:
         for heading in h.relevantHeadings(el):
             if isAlgo(heading):
                 return heading
+            break
         return None
 
     def algoName(el: t.ElementT) -> str | None:
@@ -261,119 +203,7 @@ def checkVarHygiene(doc: t.SpecT) -> None:
 def addVarClickHighlighting(doc: t.SpecT) -> None:
     if doc.md.slimBuildArtifact:
         return
-    doc.extraStyles[
-        "style-var-click-highlighting"
-    ] = """
-    var { cursor: pointer; }
-    var.selected0 { background-color: #F4D200; box-shadow: 0 0 0 2px #F4D200; }
-    var.selected1 { background-color: #FF87A2; box-shadow: 0 0 0 2px #FF87A2; }
-    var.selected2 { background-color: #96E885; box-shadow: 0 0 0 2px #96E885; }
-    var.selected3 { background-color: #3EEED2; box-shadow: 0 0 0 2px #3EEED2; }
-    var.selected4 { background-color: #EACFB6; box-shadow: 0 0 0 2px #EACFB6; }
-    var.selected5 { background-color: #82DDFF; box-shadow: 0 0 0 2px #82DDFF; }
-    var.selected6 { background-color: #FFBCF2; box-shadow: 0 0 0 2px #FFBCF2; }
-    """
-    # Colors were chosen in Lab using https://nixsensor.com/free-color-converter/
-    # D50 2deg illuminant, L in [0,100], a and b in [-128, 128]
-    # 0 = lab(85,0,85)
-    # 1 = lab(85,80,30)
-    # 2 = lab(85,-40,40)
-    # 3 = lab(85,-50,0)
-    # 4 = lab(85,5,15)
-    # 5 = lab(85,-10,-50)
-    # 6 = lab(85,35,-15)
-
-    # Color-choosing design.
-    # Start with 1, increment as new ones get selected.
-    # Specifically: find lowest-indexed color with lowest usage.
-    # (Usually this'll be zero, but if you click too many vars in same algo, can repeat.)
-    # If you unclick then click again on same var, it should get same color if possible.
-    doc.extraScripts[
-        "script-var-click-highlighting"
-    ] = r"""
-    "use strict";
-    {
-        document.addEventListener("click", e=>{
-            if(e.target.nodeName == "VAR") {
-                highlightSameAlgoVars(e.target);
-            }
-        });
-        const indexCounts = new Map();
-        const indexNames = new Map();
-        function highlightSameAlgoVars(v) {
-            // Find the algorithm container.
-            let algoContainer = null;
-            let searchEl = v;
-            while(algoContainer == null && searchEl != document.body) {
-                searchEl = searchEl.parentNode;
-                if(searchEl.hasAttribute("data-algorithm")) {
-                    algoContainer = searchEl;
-                }
-            }
-
-            // Not highlighting document-global vars,
-            // too likely to be unrelated.
-            if(algoContainer == null) return;
-
-            const algoName = algoContainer.getAttribute("data-algorithm");
-            const varName = getVarName(v);
-            const addClass = !v.classList.contains("selected");
-            let highlightClass = null;
-            if(addClass) {
-                const index = chooseHighlightIndex(algoName, varName);
-                indexCounts.get(algoName)[index] += 1;
-                indexNames.set(algoName+"///"+varName, index);
-                highlightClass = nameFromIndex(index);
-            } else {
-                const index = previousHighlightIndex(algoName, varName);
-                indexCounts.get(algoName)[index] -= 1;
-                highlightClass = nameFromIndex(index);
-            }
-
-            // Find all same-name vars, and toggle their class appropriately.
-            for(const el of algoContainer.querySelectorAll("var")) {
-                if(getVarName(el) == varName) {
-                    el.classList.toggle("selected", addClass);
-                    el.classList.toggle(highlightClass, addClass);
-                }
-            }
-        }
-        function getVarName(el) {
-            return el.textContent.replace(/(\s|\xa0)+/, " ").trim();
-        }
-        function chooseHighlightIndex(algoName, varName) {
-            let indexes = null;
-            if(indexCounts.has(algoName)) {
-                indexes = indexCounts.get(algoName);
-            } else {
-                // 7 classes right now
-                indexes = [0,0,0,0,0,0,0];
-                indexCounts.set(algoName, indexes);
-            }
-
-            // If the element was recently unclicked,
-            // *and* that color is still unclaimed,
-            // give it back the same color.
-            const lastIndex = previousHighlightIndex(algoName, varName);
-            if(indexes[lastIndex] === 0) return lastIndex;
-
-            // Find the earliest index with the lowest count.
-            const minCount = Math.min.apply(null, indexes);
-            let index = null;
-            for(var i = 0; i < indexes.length; i++) {
-                if(indexes[i] == minCount) {
-                    return i;
-                }
-            }
-        }
-        function previousHighlightIndex(algoName, varName) {
-            return indexNames.get(algoName+"///"+varName);
-        }
-        function nameFromIndex(index) {
-            return "selected" + index;
-        }
-    }
-    """
+    doc.extraJC.addVarClickHighlighting()
 
 
 def fixIntraDocumentReferences(doc: t.SpecT) -> None:
@@ -481,7 +311,8 @@ def fillInterDocumentReferenceFromSpecref(doc: t.SpecT, el: t.ElementT, spec: st
         return
     if bib.title is None:
         m.die(
-            f"Can't generate a cross-spec section ref for '{spec}', because the biblio entry has no spec title.", el=el
+            f"Can't generate a cross-spec section ref for '{spec}', because the biblio entry has no spec title.",
+            el=el,
         )
         return
     el.tag = "a"
@@ -533,6 +364,32 @@ def determineDfnType(doc: t.SpecT, dfn: t.ElementT, inferCSS: bool = False) -> s
         ):
             return "value"
         if text[0:1] == "<" and text[-1:] == ">":
+            if "[" in text and "]" in text:
+                # has a range, *must* be a value
+                return "value"
+            if "@" in text:
+                # Trying to dfn an at-rule production
+                m.die(
+                    "CSS at-rules must be defined in prose, with <dfn>@foo</dfn>. Remove the dfn from this grammar term.",
+                    el=dfn,
+                )
+            elif "(" in text and ")" in text:
+                # Trying to define a function production
+                m.die(
+                    "CSS functions must be defined in prose, with <dfn>foo(args)</dfn>. Remove the dfn from this grammar term.",
+                    el=dfn,
+                )
+            elif "''" in text:
+                m.die(
+                    "CSS values must be defined in prose, with <dfn value>foo</dfn>. Remove the dfn from this grammar term.",
+                    el=dfn,
+                )
+            elif "'" in text:
+                m.die(
+                    "CSS properties must be defined in prose, with a <pre class=propdef> table or <dfn property>foo</dfn>. Remove the dfn from this grammar term.",
+                    el=dfn,
+                )
+            # Regardless of errors above, return "type""
             return "type"
         if text[0:1] == ":":
             return "selector"
@@ -550,7 +407,7 @@ def classifyDfns(doc: t.SpecT, dfns: list[t.ElementT]) -> None:
             m.die(f"Unknown dfn type '{dfnType}':\n{h.outerHTML(el)}", el=el)
             continue
         dfnFor = h.treeAttr(el, "data-dfn-for")
-        primaryDfnText = config.firstLinkTextFromElement(el)
+        primaryDfnText = h.firstLinkTextFromElement(el)
         if primaryDfnText is None:
             m.die(f"Dfn has no linking text:\n{h.outerHTML(el)}", el=el)
             continue
@@ -628,7 +485,9 @@ def classifyDfns(doc: t.SpecT, dfns: list[t.ElementT]) -> None:
         # Automatically fill in id if necessary.
         if el.get("id") is None:
             if dfnFor:
-                singleFor = config.splitForValues(dfnFor)[0]
+                singleFor: str | None = config.splitForValues(dfnFor)[0]
+            else:
+                singleFor = None
             if dfnType in config.functionishTypes.intersection(config.idlTypes):
                 match = re.match(r"[^(]*", primaryDfnText)
                 if match:
@@ -639,7 +498,7 @@ def classifyDfns(doc: t.SpecT, dfns: list[t.ElementT]) -> None:
                     "{_for}-{id}".format(
                         _for=singleFor,
                         id=parenlessID + "()",
-                    )
+                    ),
                 )
                 el.set(
                     "data-alternate-id",
@@ -683,7 +542,7 @@ def classifyDfns(doc: t.SpecT, dfns: list[t.ElementT]) -> None:
             if attr is not None:
                 el.set(attr, "")
             else:
-                if dfnType == "dfn":
+                if dfnType == "dfn" or dfnType in config.cddlTypes:
                     el.set("data-noexport", "by-default")
                 else:
                     el.set("data-export", "by-default")
@@ -787,7 +646,7 @@ def processBiblioLinks(doc: t.SpecT) -> None:
                 closeBiblios = biblio.findCloseBiblios(list(doc.refs.biblioKeys), linkText)
                 m.die(
                     f"Couldn't find '{linkText}' in bibliography data. Did you mean:\n"
-                    + "\n".join("  " + b for b in closeBiblios),
+                    + "\n".join("  " + b for b in sorted(closeBiblios)),
                     el=el,
                 )
             el.tag = "span"
@@ -855,15 +714,14 @@ def verifyUsageOfAllLocalBiblios(doc: t.SpecT) -> None:
 
 
 def processAutolinks(doc: t.SpecT) -> None:
+    refsJSON = doc.extraJC.addRefHints()
+
     # An <a> without an href is an autolink.
     # <i> is a legacy syntax for term autolinks. If it links up, we change it into an <a>.
     # We exclude bibliographical links, as those are processed in `processBiblioLinks`.
-    query = "a:not([href]):not([data-link-type='biblio'])"
-    if doc.md.useIAutolinks:
-        m.warn("Use <i> Autolinks is deprecated and will be removed. Please switch to using <a> elements.")
-        query += ", i"
-    autolinks = h.findAll(query, doc)
-    for el in autolinks:
+    for el in h.collectAutolinks(doc.body):
+        if el.tag == "bs-link":
+            el.tag = "a"
         # Explicitly empty linking text indicates this shouldn't be an autolink.
         if el.get("data-lt") == "":
             continue
@@ -910,6 +768,14 @@ def processAutolinks(doc: t.SpecT) -> None:
             el=el,
             error=not okayToFail and not ignorable,
         )
+
+        if isinstance(ref, str):
+            # Error message, annotate the link with it and continue.
+            el.tag = "u"
+            h.addClass(doc, el, "link-error")
+            el.set("title", f"LINK ERROR: {ref}")
+            continue
+
         # Capture the reference (and ensure we add a biblio entry) if it
         # points to an external specification.
         if ref and ref.status != "local":
@@ -935,7 +801,25 @@ def processAutolinks(doc: t.SpecT) -> None:
         if ref:
             el.set("href", ref.url)
             el.tag = "a"
+            replacementText = el.get("bs-replace-text-on-link-success")
+            if replacementText is not None:
+                h.clearContents(el)
+                el.text = replacementText
             decorateAutolink(doc, el, linkType=linkType, linkText=linkText, ref=ref)
+
+            # Store the ref so ref-hint panels can get generated for it
+            cachedRef = refsJSON.get(ref.url)
+            if cachedRef is None:
+                refsJSON[ref.url] = ref
+            elif cachedRef != ref:
+                # whoops, multiple refs with the same URL
+                subKey = ref.refKey() + "_" + ref.url
+                if subKey not in refsJSON:
+                    refsJSON[subKey] = ref
+                el.set("data-refhint-key", ref.refKey())
+            else:
+                # Ref was already stored
+                pass
         else:
             if linkType == "maybe":
                 el.tag = "css"
@@ -943,6 +827,7 @@ def processAutolinks(doc: t.SpecT) -> None:
                     del el.attrib["data-link-type"]
                 if el.get("data-lt"):
                     del el.attrib["data-lt"]
+
     h.dedupIDs(doc)
 
 
@@ -974,7 +859,9 @@ def decorateAutolink(doc: t.SpecT, el: t.ElementT, linkType: str, linkText: str,
         unused1, unused2, id = ref.url.partition("#")  # pylint: disable=unused-variable
         if id:
             el.set("id", f"ref-for-{id}")
-            el.set("data-silently-dedup", "")
+        else:
+            el.set("id", "ref-for-something")
+        el.set("data-silently-dedup", "")
 
     # Get all the values that the type expands to, add it as a title.
     if linkType == "type":
@@ -982,18 +869,15 @@ def decorateAutolink(doc: t.SpecT, el: t.ElementT, linkType: str, linkText: str,
         if linkText in doc.typeExpansions:
             titleText = doc.typeExpansions[linkText]
         else:
-            if linkText == "":
-                print(h.outerHTML(el, literal=True))
-                assert linkText != ""
-
             typeRefs = doc.refs.queryAllRefs(linkFor=linkText, ignoreObsoletes=True)
             texts = sorted({ref.text for ref in typeRefs})
             if typeRefs:
-                titleText = _("Expands to: ") + " | ".join(texts)
+                titleText = _t("Expands to: ") + " | ".join(texts)
                 assert titleText is not None
                 doc.typeExpansions[linkText] = titleText
         if titleText:
-            el.set("title", titleText)
+            titleData = doc.extraJC.addLinkTitles()
+            titleData[ref.url] = titleText
 
 
 def removeMultipleLinks(doc: t.SpecT) -> None:
@@ -1021,40 +905,45 @@ def processIssuesAndExamples(doc: t.SpecT) -> None:
     # examples, and link to remote issues if possible:
     for el in h.findAll(".issue:not([id])", doc):
         el.set("id", h.safeID(doc, "issue-" + h.hashContents(el)))
-        remoteIssueID = el.get("data-remote-issue-id")
-        if remoteIssueID:
-            del el.attrib["data-remote-issue-id"]
-            # Eventually need to support a way to trigger other repo url structures,
-            # but defaulting to GH is fine for now.
-            githubMatch = re.match(r"\s*([\w-]+)/([\w-]+)#(\d+)\s*$", remoteIssueID)
-            numberMatch = re.match(r"\s*(\d+)\s*$", remoteIssueID)
-            remoteIssueURL = None
-            if githubMatch:
-                remoteIssueURL = "https://github.com/{}/{}/issues/{}".format(*githubMatch.groups())
-                if doc.md.inlineGithubIssues:
-                    el.set(
-                        "data-inline-github",
-                        "{} {} {}".format(*githubMatch.groups()),
-                    )
-            elif numberMatch and isinstance(doc.md.repository, repository.GithubRepository):
-                remoteIssueURL = doc.md.repository.formatIssueUrl(numberMatch.group(1))
-                if doc.md.inlineGithubIssues:
-                    el.set(
-                        "data-inline-github",
-                        "{} {} {}".format(
-                            doc.md.repository.user,
-                            doc.md.repository.repo,
-                            numberMatch.group(1),
-                        ),
-                    )
-            elif doc.md.issueTrackerTemplate:
-                remoteIssueURL = doc.md.issueTrackerTemplate.format(remoteIssueID)
-            if remoteIssueURL:
-                h.appendChild(
-                    el, " ", h.E.a({"href": remoteIssueURL}, _("[Issue #{number}]").format(number=remoteIssueID))
-                )
     for el in h.findAll(".example:not([id])", doc):
         el.set("id", h.safeID(doc, "example-" + h.hashContents(el)))
+    for el in h.findAll("[data-remote-issue-id]", doc):
+        remoteIssueID = el.get("data-remote-issue-id", "").strip()
+        del el.attrib["data-remote-issue-id"]
+        # Eventually need to support a way to trigger other repo url structures,
+        # but defaulting to GH is fine for now.
+        githubMatch = re.match(r"([\w-]+)/([\w-]+)#(\d+)$", remoteIssueID)
+        numberMatch = re.match(r"#?(\d+)$", remoteIssueID)
+        remoteIssueURL = None
+        if githubMatch:
+            org, repo, num = githubMatch.groups()
+            text = f"[{org}/{repo} Issue #{num}]"
+            remoteIssueURL = f"https://github.com/{org}/{repo}/issues/{num}"
+            if doc.md.inlineGithubIssues:
+                el.set("data-inline-github", f"{org} {repo} {num}")
+        elif numberMatch and isinstance(doc.md.repository, repository.GithubRepository):
+            remoteIssueURL = doc.md.repository.formatIssueUrl(numberMatch.group(1))
+            num = numberMatch.group(1)
+            text = f"[Issue #{num}]"
+            if doc.md.inlineGithubIssues:
+                org = doc.md.repository.user
+                repo = doc.md.repository.repo
+                el.set("data-inline-github", f"{org} {repo} {num}")
+        elif doc.md.issueTrackerTemplate:
+            text = f"[Issue {remoteIssueID}]"
+            remoteIssueURL = doc.md.issueTrackerTemplate.format(remoteIssueID)
+        else:
+            m.die(f"Can't parse '{remoteIssueID}' as an issue ID.", el=el)
+            continue
+        if remoteIssueURL:
+            if h.tagName(el) == "a":
+                el.set("href", remoteIssueURL)
+            else:
+                h.appendChild(
+                    el,
+                    " ",
+                    h.E.a({"href": remoteIssueURL}, text),
+                )
     h.fixupIDs(doc, h.findAll(".issue, .example", doc))
 
 
@@ -1187,6 +1076,10 @@ def cleanupHTML(doc: t.SpecT) -> None:
         if el.tag == "pre" and h.hasClass(doc, el, "idl") and not h.hasClass(doc, el, "def"):
             h.addClass(doc, el, "def")
 
+        # Mark pre.cddl blocks as .def, for styling
+        if el.tag == "pre" and h.hasClass(doc, el, "cddl") and not h.hasClass(doc, el, "def"):
+            h.addClass(doc, el, "def")
+
         # Tag classes on wide types of dfns/links
         if el.tag in config.dfnElements:
             if el.get("data-dfn-type") in config.idlTypes:
@@ -1295,7 +1188,6 @@ def cleanupHTML(doc: t.SpecT) -> None:
             "line-numbers",
             "data-opaque",
             "data-no-self-link",
-            "line-number",
             "caniuse",
             "data-silently-dedup",
             "nocrossorigin",
@@ -1387,7 +1279,7 @@ def formatElementdefTables(doc: t.SpecT) -> None:
                                 {"data-link-type": "element-attr", "for": groupName},
                                 ref.text.strip(),
                             ),
-                        )
+                        ),
                     ),
                 )
 
@@ -1448,8 +1340,6 @@ def inlineRemoteIssues(doc: t.SpecT) -> None:
     if not inlineIssues:
         return
 
-    import requests
-
     logging.captureWarnings(True)
 
     responses = json.loads(doc.dataFile.fetch("github-issues.json", str=True))
@@ -1470,7 +1360,7 @@ def inlineRemoteIssues(doc: t.SpecT) -> None:
 
         res = None
         try:
-            res = requests.get(url, headers=headers)
+            res = requests.get(url, headers=headers, timeout=5)
         except requests.exceptions.ConnectionError:
             # Offline or something, recover if possible
             if key in responses:
@@ -1494,23 +1384,23 @@ def inlineRemoteIssues(doc: t.SpecT) -> None:
                 m.die(f"'{doc.token}' is not a valid GitHub OAuth token. See https://github.com/settings/tokens")
             else:
                 m.die(
-                    "401 error when fetching GitHub Issues:\n" + config.printjson(error),
+                    "401 error when fetching GitHub Issues:\n" + printjson.printjson(error),
                 )
             continue
         elif res.status_code == 403:
             error = res.json()
             if error["message"].startswith("API rate limit exceeded"):
                 m.die(
-                    "GitHub Issues API rate limit exceeded. Get an OAuth token from https://github.com/settings/tokens to increase your limit, or just wait an hour for your limit to refresh; Bikeshed has cached all the issues so far and will resume from where it left off."
+                    "GitHub Issues API rate limit exceeded. Get an OAuth token from https://github.com/settings/tokens to increase your limit, or just wait an hour for your limit to refresh; Bikeshed has cached all the issues so far and will resume from where it left off.",
                 )
             else:
                 m.die(
-                    "403 error when fetching GitHub Issues:\n" + config.printjson(error),
+                    "403 error when fetching GitHub Issues:\n" + printjson.printjson(error),
                 )
             continue
         elif res.status_code >= 400:
             try:
-                error = config.printjson(res.json())
+                error = printjson.printjson(res.json())
             except ValueError:
                 error = "First 100 characters of error:\n" + res.text[0:100]
             m.die(f"{res.status_code} error when fetching GitHub Issues:\n" + error)
@@ -1535,7 +1425,7 @@ def inlineRemoteIssues(doc: t.SpecT) -> None:
                 el,
                 h.E.a(
                     {"href": href, "class": "marker"},
-                    _("Issue #{number} on GitHub: “{title}”").format(number=data["number"], title=data["title"]),
+                    _t("Issue #{number} on GitHub: “{title}”").format(number=data["number"], title=data["title"]),
                 ),
                 *h.parseHTML(data["body_html"]),
             )
@@ -1556,17 +1446,36 @@ def addNoteHeaders(doc: t.SpecT) -> None:
     for el in h.findAll("[heading]", doc):
         h.addClass(doc, el, "no-marker")
         if h.hasClass(doc, el, "note"):
-            preText = _("NOTE: ")
+            preText = _t("NOTE: ")
         elif h.hasClass(doc, el, "issue"):
-            preText = _("ISSUE: ")
+            preText = _t("ISSUE: ")
         elif h.hasClass(doc, el, "example"):
-            preText = _("EXAMPLE: ")
+            preText = _t("EXAMPLE: ")
         else:
             preText = ""
-        h.prependChild(el, h.E.div({"class": "marker"}, preText, *h.parseHTML(el.get("heading", ""))))
+        lineNum = lineNumberFromBsLineNumber(el.get("bs-line-number"))
+        parseConfig = h.ParseConfig.fromSpec(doc=doc, context="heading='' attribute")
+        parsedHeading = h.parseHTML(h.parseText(el.get("heading", ""), parseConfig, startLine=lineNum))
+        h.prependChild(el, h.E.div({"class": "marker"}, preText, *parsedHeading))
         h.removeAttr(el, "heading")
 
 
+def lineNumberFromBsLineNumber(value: str | None) -> int:
+    if value is None:
+        return 1
+    try:
+        return int(value)
+    except:
+        pass
+    try:
+        # If it's a 'loc' value: line:col + maybe other context
+        num, _, _ = value.partition(":")
+        return int(num)
+    except:
+        pass
+    return 1
+
+
 def locateFillContainers(doc: t.SpecT) -> t.FillContainersT:
     fillContainers = defaultdict(list)
     for el in h.findAll("[data-fill-with]", doc):
@@ -1639,9 +1548,15 @@ def addImageSize(doc: t.SpecT) -> None:
         try:
             im = Image.open(imgPath)
             width, height = im.size
+        except FileNotFoundError:
+            m.warn(
+                f"Image doesn't exist, so I couldn't determine its width and height: '{src}'",
+                el=el,
+            )
+            continue
         except Exception as e:
             m.warn(
-                f"Couldn't determine width and height of this image: {src}\n{e}",
+                f"Couldn't determine width and height of this image: '{src}'\n{e}",
                 el=el,
             )
             continue
@@ -1671,6 +1586,23 @@ def processIDL(doc: t.SpecT) -> None:
         localDfns.update(idl.markupIDLBlock(pre, doc))
 
     dfns = h.findAll("pre.idl:not([data-no-idl]) dfn, xmp.idl:not([data-no-idl]) dfn", doc) + list(localDfns)
+    dfns = sorted(dfns, key=lambda x: (x.get("bs-line-number") or "", h.textContent(x)))
+    classifyDfns(doc, dfns)
+    h.fixupIDs(doc, dfns)
+    doc.refs.addLocalDfns(doc, (dfn for dfn in dfns if dfn.get("id") is not None))
+
+
+def processCDDL(doc: t.SpecT) -> None:
+    localDfns = set()
+    for pre in h.findAll("pre.cddl, xmp.cddl", doc):
+        if pre.get("data-no-cddl") is not None:
+            continue
+        if not h.isNormative(doc, pre):
+            continue
+        localDfns.update(cddl.markupCDDLBlock(pre, doc))
+
+    dfns = h.findAll("pre.cddl:not([data-no-cddl]) dfn, xmp.cddl:not([data-no-cddl]) dfn", doc) + list(localDfns)
+    dfns = sorted(dfns, key=lambda x: (x.get("bs-line-number") or "", h.textContent(x)))
     classifyDfns(doc, dfns)
     h.fixupIDs(doc, dfns)
     doc.refs.addLocalDfns(doc, (dfn for dfn in dfns if dfn.get("id") is not None))
