@@ -1,11 +1,12 @@
 from .main import fixupDataFiles, update, updateReadonlyDataFiles
-from .manifest import createManifest
+from .manifest import Manifest, createManifest
+from .mode import UpdateMode
 
 __all__ = [
     "updateBackRefs",
-    "updateCrossRefs",
     "updateBiblio",
     "updateCanIUse",
-    "updateLinkDefaults",
+    "updateCrossRefs",
     "updateLanguages",
+    "updateLinkDefaults",
 ]
