@@ -7,7 +7,8 @@ from collections import defaultdict
 
 import requests
 
-from .. import biblio, messages as m, t
+from .. import biblio, t
+from .. import messages as m
 
 
 def update(path: str, dryRun: bool = False) -> set[str] | None:
@@ -15,7 +16,6 @@ def update(path: str, dryRun: bool = False) -> set[str] | None:
     biblios: t.BiblioStorageT = defaultdict(list)
     biblio.processSpecrefBiblioFile(getSpecrefData(), biblios, order=3)
     biblio.processSpecrefBiblioFile(getWG21Data(), biblios, order=3)
-    biblio.processReferBiblioFile(getCSSWGData(), biblios, order=4)
     writtenPaths: set[str] = set()
     if not dryRun:
         groupedBiblios, allNames = groupBiblios(biblios)
@@ -92,7 +92,7 @@ def update(path: str, dryRun: bool = False) -> set[str] | None:
 
 def getSpecrefData() -> str:
     try:
-        return requests.get("https://api.specref.org/bibrefs").text
+        return requests.get("https://api.specref.org/bibrefs", timeout=5).text
     except Exception as e:
         m.die(f"Couldn't download the SpecRef biblio data.\n{e}")
         return "{}"
@@ -100,7 +100,7 @@ def getSpecrefData() -> str:
 
 def getWG21Data() -> str:
     try:
-        return requests.get("https://wg21.link/specref.json").text
+        return requests.get("https://wg21.link/specref.json", timeout=5).text
     except Exception as e:
         m.die(f"Couldn't download the WG21 biblio data.\n{e}")
         return "{}"
@@ -108,7 +108,10 @@ def getWG21Data() -> str:
 
 def getCSSWGData() -> list[str]:
     try:
-        return requests.get("https://raw.githubusercontent.com/w3c/csswg-drafts/master/biblio.ref").text.splitlines()
+        return requests.get(
+            "https://raw.githubusercontent.com/w3c/csswg-drafts/main/biblio.ref",
+            timeout=5,
+        ).text.splitlines()
     except Exception as e:
         m.die(f"Couldn't download the CSSWG biblio data.\n{e}")
         return []
