@@ -5,20 +5,21 @@ import os
 import re
 from collections import defaultdict
 
-import tenacity
 import requests
+import tenacity
 from alive_progress import alive_it
 
-from .. import config, messages as m, t
+from .. import config, t
+from .. import messages as m
 
 if t.TYPE_CHECKING:
-    AnchorsT: t.TypeAlias = defaultdict[str, list[AnchorT]]
-    HeadingGroupT: t.TypeAlias = dict[SpecStatusT, HeadingT]
+    AnchorsT: t.TypeAlias = defaultdict[str, list["AnchorT"]]
+    HeadingGroupT: t.TypeAlias = dict["SpecStatusT", "HeadingT"]
     HeadingKeyT: t.TypeAlias = str
     SpecStatusT: t.TypeAlias = str
     HeadingsT: t.TypeAlias = dict[HeadingKeyT, list[HeadingKeyT] | HeadingGroupT]
     AllHeadingsT: t.TypeAlias = dict[str, HeadingsT]
-    SpecsT: t.TypeAlias = dict[str | None, SpecT]
+    SpecsT: t.TypeAlias = dict[str | None, "SpecT"]
     MethodsT: t.TypeAlias = defaultdict[str, dict[str, t.Any]]
     ForsT: t.TypeAlias = defaultdict[str, list[str]]
 
@@ -29,6 +30,7 @@ if t.TYPE_CHECKING:
         title: t.Required[str]
         level: t.Required[int]
         number: str
+        alternateIds: list[str]
 
     class HeadingT(t.TypedDict):
         url: str
@@ -94,7 +96,7 @@ if t.TYPE_CHECKING:
 
 
 def update(path: str, dryRun: bool = False) -> set[str] | None:
-    specs: SpecsT = dict()
+    specs: SpecsT = {}
     anchors: AnchorsT = defaultdict(list)
     headings: AllHeadingsT = {}
 
@@ -183,7 +185,7 @@ def gatherWebrefData(specs: SpecsT, anchors: AnchorsT, headings: AllHeadingsT) -
                     rawSnapshotSpec = s
                     break
             else:
-                print(f"Despite claiming to have a snapshot url, no snapshot data found for '{spec['vshortname']}'.")
+                m.warn(f"Despite claiming to have a snapshot url, no snapshot data found for '{spec['vshortname']}'.")
             if rawSnapshotSpec:
                 if "dfns" in rawSnapshotSpec:
                     snapshotAnchors = anchorsFromWebref("snapshot", rawSnapshotSpec["dfns"])
@@ -201,7 +203,8 @@ def specsFromWebref(status: t.Literal["current" | "snapshot"]) -> list[WebrefSpe
     url = ("ed" if status == "current" else "tr") + "/index.json"
     j = dataFromWebref(url)
     if j is None or j.get("results") is None:
-        raise Exception(f"No {status} specs data from WebRef. Got:\n{json.dumps(j, indent=1)}")
+        msg = f"No {status} specs data from WebRef. Got:\n{json.dumps(j, indent=1)}"
+        raise Exception(msg)
     rawSpecs = t.cast("list[WebrefSpecT]", j["results"])
     filteredSpecs: list[WebrefSpecT] = []
     for spec in rawSpecs:
@@ -215,7 +218,8 @@ def anchorsFromWebref(status: t.Literal["current" | "snapshot"], urlSuffix: str)
     url = ("ed" if status == "current" else "tr") + "/" + urlSuffix
     j = dataFromWebref(url)
     if j is None or j.get("dfns") is None:
-        raise Exception(f"No WebRef dfns data at {url}. Got:\n{json.dumps(j, indent=1)}")
+        msg = f"No WebRef dfns data at {url}. Got:\n{json.dumps(j, indent=1)}"
+        raise Exception(msg)
     return t.cast("list[WebrefAnchorT]", j["dfns"])
 
 
@@ -223,7 +227,8 @@ def headingsFromWebref(status: t.Literal["current" | "snapshot"], urlSuffix: str
     url = ("ed" if status == "current" else "tr") + "/" + urlSuffix
     j = dataFromWebref(url)
     if j is None or j.get("headings") is None:
-        raise Exception(f"No WebRef headings data at {url}. Got:\n{json.dumps(j, indent=1)}")
+        msg = f"No WebRef headings data at {url}. Got:\n{json.dumps(j, indent=1)}"
+        raise Exception(msg)
     return t.cast("list[WebrefHeadingT]", j["headings"])
 
 
@@ -231,15 +236,15 @@ def headingsFromWebref(status: t.Literal["current" | "snapshot"], urlSuffix: str
 def dataFromWebref(url: str) -> t.JSONT:
     webrefAPIUrl = "https://raw.githubusercontent.com/w3c/webref/main/"
     try:
-        response = requests.get(webrefAPIUrl + url)
+        response = requests.get(webrefAPIUrl + url, timeout=5)
     except Exception as e:
-        raise Exception(f"Couldn't download data from Webref.\n{e}") from e
+        msg = f"Couldn't download data from Webref.\n{e}"
+        raise Exception(msg) from e
     try:
         data = response.json()
     except Exception as e:
-        raise Exception(
-            f"Data retrieved from Webref wasn't valid JSON for some reason. Try downloading again?\n{e}"
-        ) from e
+        msg = f"Data retrieved from Webref wasn't valid JSON for some reason. Try downloading again?\n{e}"
+        raise Exception(msg) from e
     return t.cast("t.JSONT", data)
 
 
@@ -258,15 +263,20 @@ def canonSpecFromWebref(rawSpec: WebrefSpecT) -> SpecT:
         "title": rawSpec["shortTitle"],
         "description": rawSpec["title"],
         "abstract": None,
-        "level": int(rawSpec["seriesVersion"])
-        if "seriesVersion" in rawSpec and re.match(r"^\d+$", rawSpec.get("seriesVersion", ""))
-        else None,
+        "level": (
+            int(rawSpec["seriesVersion"])
+            if "seriesVersion" in rawSpec and re.match(r"^\d+$", rawSpec.get("seriesVersion", ""))
+            else None
+        ),
     }
     return spec
 
 
 def addToAnchors(
-    rawAnchor: WebrefAnchorT, anchors: AnchorsT, spec: SpecT, status: t.Literal["current"] | t.Literal["snapshot"]
+    rawAnchor: WebrefAnchorT,
+    anchors: AnchorsT,
+    spec: SpecT,
+    status: t.Literal["current"] | t.Literal["snapshot"],
 ) -> None:
     baseUrl = spec["snapshot_url"] if status == "snapshot" else spec["current_url"]
     assert baseUrl is not None
@@ -291,8 +301,6 @@ def addToAnchors(
     for text in rawAnchor["linkingText"]:
         text = re.sub(r"‘|’", "'", text)
         text = re.sub(r"“|”", '"', text)
-        if anchor["type"] in config.lowercaseTypes:
-            text = text.lower()
         text = re.sub(r"\s+", " ", text)
         anchors[text].append(anchor)
 
@@ -305,7 +313,7 @@ def addToHeadings(
 ) -> None:
     heading: HeadingT = {
         "url": rawAnchor["href"],
-        "number": rawAnchor["number"] if "number" in rawAnchor else "",
+        "number": rawAnchor.get("number", ""),
         "text": rawAnchor["title"],
         "spec": spec["title"],
     }
@@ -314,7 +322,7 @@ def addToHeadings(
     if heading["url"].startswith(specUrl):
         truncatedUrl = heading["url"][len(specUrl) :]
     else:
-        print(f"Invalid heading - URL doesn't start with the spec's url <{specUrl}>.\n{heading}")
+        m.warn(f"Invalid heading - URL doesn't start with the spec's url <{specUrl}>.\n{heading}")
         return
     if truncatedUrl[0] == "#":
         # Either single-page spec, or link on the top page of a multi-page spec
@@ -338,15 +346,26 @@ def addToHeadings(
             page = "/" + page
             fragment = "#"
         shorthand = page + fragment
-    if shorthand not in specHeadings:
-        specHeadings[shorthand] = {}
-    headingGroup = t.cast("HeadingGroupT", specHeadings[shorthand])
-    headingGroup[status] = heading
-    if fragment not in specHeadings:
-        specHeadings[fragment] = []
-    keyList = t.cast("list[HeadingKeyT]", specHeadings[fragment])
-    if shorthand not in specHeadings[fragment]:
-        keyList.append(shorthand)
+
+    def addShorthandAndFragmentToSpecHeadings(shorthand: str, fragment: str) -> None:
+        if shorthand not in specHeadings:
+            specHeadings[shorthand] = {}
+        headingGroup = t.cast("HeadingGroupT", specHeadings[shorthand])
+        headingGroup[status] = heading
+        if fragment not in specHeadings:
+            specHeadings[fragment] = []
+        keyList = t.cast("list[HeadingKeyT]", specHeadings[fragment])
+        if shorthand not in specHeadings[fragment]:
+            keyList.append(shorthand)
+
+    addShorthandAndFragmentToSpecHeadings(shorthand, fragment)
+
+    # A heading may have multiple IDs: add the others as aliases of the primary one.
+    if "alternateIds" in rawAnchor:
+        shorthandBase, _, _ = shorthand.partition("#")
+        for alternateId in rawAnchor["alternateIds"]:
+            fragment = "#" + alternateId
+            addShorthandAndFragmentToSpecHeadings(shorthandBase + fragment, fragment)
 
 
 def cleanSpecHeadings(headings: AllHeadingsT) -> None:
@@ -439,7 +458,9 @@ def writeAnchorsFile(anchors: AnchorsT, path: str) -> set[str]:
         with open(p, "w", encoding="utf-8") as fh:
             for key, entries in sorted(group_anchors.items(), key=lambda x: x[0]):
                 for e in sorted(entries, key=lambda x: x["url"]):
-                    fh.write(key + "\n")
+                    refKey, displayKey = config.adjustKey(key, e.get("type", ""))
+                    fh.write(refKey + "\n")
+                    fh.write(displayKey + "\n")
                     for field in [
                         "type",
                         "spec",
