@@ -11,7 +11,8 @@ def update(path: str, dryRun: bool = False) -> set[str] | None:
     try:
         m.say("Downloading languages...")
         data = requests.get(
-            "https://raw.githubusercontent.com/speced/bikeshed/master/bikeshed/spec-data/readonly/languages.json"
+            "https://raw.githubusercontent.com/speced/bikeshed/main/bikeshed/spec-data/readonly/languages.json",
+            timeout=5,
         ).text
     except Exception as e:
         m.die(f"Couldn't download languages data.\n{e}")
@@ -27,4 +28,4 @@ def update(path: str, dryRun: bool = False) -> set[str] | None:
             m.die(f"Couldn't save languages database to disk.\n{e}")
             return None
     m.say("Success!")
-    return set([filePath])
+    return {filePath}
