@@ -11,7 +11,8 @@ def update(path: str, dryRun: bool = False) -> set[str] | None:
     try:
         m.say("Downloading link defaults...")
         data = requests.get(
-            "https://raw.githubusercontent.com/speced/bikeshed/main/bikeshed/spec-data/readonly/link-defaults.infotree"
+            "https://raw.githubusercontent.com/speced/bikeshed/main/bikeshed/spec-data/readonly/link-defaults.infotree",
+            timeout=5,
         ).text
     except Exception as e:
         m.die(f"Couldn't download link defaults data.\n{e}")
@@ -27,4 +28,4 @@ def update(path: str, dryRun: bool = False) -> set[str] | None:
             m.die(f"Couldn't save link-defaults database to disk.\n{e}")
             return None
     m.say("Success!")
-    return set([filePath])
+    return {filePath}
