@@ -1,7 +1,10 @@
 from __future__ import annotations
 
-from .. import config, h, messages as m, t
-from ..translate import _
+from collections import Counter
+
+from .. import h, t
+from .. import messages as m
+from ..translate import _t
 
 
 def processWptElements(doc: t.SpecT) -> None:
@@ -22,9 +25,10 @@ def processWptElements(doc: t.SpecT) -> None:
         assert testData is not None
         testNames = testNamesFromEl(el, pathPrefix=pathPrefix)
         for testName in testNames:
-            if testName not in testData:
-                m.warn(f"Couldn't find WPT test '{testName}' - did you misspell something?", el=el)
-                continue
+            if el.get("skip-existence-check") is None:
+                if testName not in testData:
+                    m.warn(f"Couldn't find WPT test '{testName}' - did you misspell something?", el=el)
+                    continue
             seenTestNames.add(testName)
             if atLeastOneVisibleTest is False and el.get("hidden") is None:
                 atLeastOneVisibleTest = True
@@ -66,7 +70,7 @@ def processWptElements(doc: t.SpecT) -> None:
             atLeastOneVisibleTest = True
             createHTML(doc, wptRestElements[0], prefixedNames, testData)
             m.warn(
-                "<wpt-rest> is intended for debugging only. Move the tests to <wpt> elements next to what they're testing."
+                "<wpt-rest> is intended for debugging only. Move the tests to <wpt> elements next to what they're testing.",
             )
     else:
         if pathPrefix:
@@ -75,27 +79,35 @@ def processWptElements(doc: t.SpecT) -> None:
             checkForOmittedTests(pathPrefix, testData, seenTestNames)
 
     if atLeastOneVisibleTest:
-        if pathPrefix is None:
-            pathPrefix = commonPathPrefix(seenTestNames)
-        if pathPrefix and not pathPrefix.startswith("/"):
-            pathPrefix = "/" + pathPrefix
-        if pathPrefix != "/":
-            doc.md.otherMetadata.setdefault(_("Test Suite"), []).append(
+        if pathPrefix and pathPrefix.startswith("https:"):
+            dataPaths = [pathPrefix]
+        else:
+            dataPaths = commonPathPrefixes(seenTestNames)
+
+        testSuiteUrl = None
+        if pathPrefix and not pathPrefix.startswith("https:"):
+            pathPrefix = "/" + pathPrefix.strip("/") + "/"
+            testSuiteUrl = f"https://wpt.fyi/results{pathPrefix}"
+        elif not pathPrefix:
+            testSuiteUrl = guessTestSuiteUrl(seenTestNames)
+        if testSuiteUrl:
+            doc.md.otherMetadata.setdefault(_t("Test Suite"), []).append(
                 h.E.dd(
                     {"class": "wpt-overview"},
                     h.E.a(
-                        {"href": f"https://wpt.fyi/results{pathPrefix}"},
-                        f"https://wpt.fyi/results{pathPrefix}",
+                        {"href": testSuiteUrl},
+                        testSuiteUrl,
                     ),
-                )
+                ),
             )
 
-    if doc.md.wptDisplay != "none" and atLeastOneElement:
-        # Empty <wpt> blocks still need styles
-        doc.extraStyles["style-wpt"] = wptStyle
+    if doc.md.wptDisplay != "none":
+        if atLeastOneElement:
+            # Empty <wpt> blocks still need styles
+            doc.extraJC.addWptCSS()
         if atLeastOneVisibleTest:
             # But I only need script if there's actually some tests.
-            doc.extraScripts["script-wpt"] = getWptScript(pathPrefix)
+            doc.extraJC.addWpt(dataPaths)
 
 
 def createHTML(
@@ -119,7 +131,7 @@ def createHTML(
         if doc.md.wptDisplay in ("open", "inline"):
             blockEl.set("open", "")
         h.clearContents(blockEl)
-        testSummaryEl = h.E.summary(_("Tests"))
+        testSummaryEl = h.E.summary(_t("Tests"))
         h.appendChild(blockEl, testSummaryEl)
         appendTestList(blockEl, testNames, testData, title, titleLang, titleDir)
     else:
@@ -147,8 +159,26 @@ def appendTestList(
     h.appendChild(blockEl, testListEl)
     for testName in testNames:
         if testName not in testData:
-            m.warn(f"Cannot find '{testName}' in the test data.")
-            continue
+            if testName.startswith("https:"):
+                # Assume it's a custom URL, just link to it.
+                singleTestEl = h.E.li(
+                    {"class": "wpt-test"},
+                    h.E.a(
+                        {
+                            "href": testName,
+                            "class": "wpt-name",
+                            # Since the name is a URL and might be long,
+                            # let it break freely.
+                            "style": "word-break: break-all",
+                        },
+                        testName,
+                    ),
+                )
+                h.appendChild(testListEl, singleTestEl)
+                continue
+            else:
+                m.warn(f"Cannot find '{testName}' in the test data.")
+                continue
         if ".https." in testName or ".serviceworker." in testName:
             liveTestScheme = "https"
         else:
@@ -172,7 +202,7 @@ def appendTestList(
                         "href": f"{liveTestScheme}://wpt.live/{testName}",
                         "class": "wpt-live",
                     },
-                    h.E.small(_("(live test)")),
+                    h.E.small(_t("(live test)")),
                 ),
                 " ",
                 h.E.a(
@@ -180,7 +210,7 @@ def appendTestList(
                         "href": "https://github.com/web-platform-tests/wpt/blob/master/" + testName,
                         "class": "wpt-source",
                     },
-                    h.E.small(_("(source)")),
+                    h.E.small(_t("(source)")),
                 ),
             )
         elif testType in ["manual", "visual"]:
@@ -192,7 +222,7 @@ def appendTestList(
                         "href": "https://github.com/web-platform-tests/wpt/blob/master/" + testName,
                         "class": "wpt-source",
                     },
-                    h.E.small(_("(source)")),
+                    h.E.small(_t("(source)")),
                 ),
             )
         elif testType in ["wdspec"]:
@@ -211,12 +241,12 @@ def appendTestList(
                         "href": "https://github.com/web-platform-tests/wpt/blob/master/" + testName,
                         "class": "wpt-source",
                     },
-                    h.E.small(_("(source)")),
+                    h.E.small(_t("(source)")),
                 ),
             )
         else:
             m.warn(
-                f"Programming error, the test {testName} is of type {testType}, which I don't know how to render. Please report this!"
+                f"Programming error, the test {testName} is of type {testType}, which I don't know how to render. Please report this!",
             )
             continue
         h.appendChild(testListEl, singleTestEl)
@@ -254,13 +284,11 @@ def prefixInPath(prefix: str | None, path: str) -> bool:
 
 
 @t.overload
-def normalizePathSegment(pathSeg: str) -> str:
-    ...
+def normalizePathSegment(pathSeg: str) -> str: ...
 
 
 @t.overload
-def normalizePathSegment(pathSeg: None) -> None:
-    ...
+def normalizePathSegment(pathSeg: None) -> None: ...
 
 
 def normalizePathSegment(pathSeg: str | None) -> str | None:
@@ -276,7 +304,7 @@ def normalizePathSegment(pathSeg: str | None) -> str | None:
 
 def checkForOmittedTests(pathPrefix: str, testData: dict[str, str], seenTestNames: set[str]) -> None:
     unseenTests = []
-    for testPath in testData.keys():
+    for testPath in testData:
         if ".tentative." in testPath:
             continue
         if prefixInPath(pathPrefix, testPath):
@@ -302,170 +330,33 @@ def xor(a: t.Any, b: t.Any) -> bool:
     return bool(a) != bool(b)
 
 
-def commonPathPrefix(paths: t.Iterable[str]) -> str | None:
-    splitPaths = [x.split("/")[:-1] for x in paths]
-    commonPrefix = splitPaths[0]
-    for pathSegs in splitPaths[1:]:
-        # can't have a common prefix longer than the shortest path
-        if len(pathSegs) < len(commonPrefix):
-            commonPrefix = commonPrefix[: len(pathSegs)]
-        # now compare the remaining segments
-        for i in range(0, min(len(commonPrefix), len(pathSegs))):
-            if pathSegs[i] != commonPrefix[i]:
-                commonPrefix = commonPrefix[:i]
-                break
-    if len(commonPrefix) >= 1:
-        return "/" + "/".join(commonPrefix) + "/"
-    return None
+def commonPathPrefixes(paths: t.Iterable[str]) -> list[str]:
+    # Split the paths into segments, and drop the filename
+    splitPaths = [x.strip("/").split("/")[:-1] for x in paths]
+    # Only record the first two segments, that's good enough
+    # for limiting the data slurped.
+    oneSegs = set()
+    twoSegs = set()
+    for pathSegs in splitPaths:
+        prefix = pathSegs[0:2]
+        if len(prefix) == 1:
+            oneSegs.add("/" + prefix[0] + "/")
+        else:
+            twoSegs.add("/" + "/".join(prefix) + "/")
+    # Filter out any 2-length segments already covered by a 1-length seg
+    for short in oneSegs:
+        twoSegs = {x for x in twoSegs if not x.startswith(short)}
+    return list(oneSegs) + list(twoSegs)
 
 
-wptStyle = """
-:root {
-    --wpt-border: hsl(0, 0%, 60%);
-    --wpt-bg: hsl(0, 0%, 95%);
-    --wpt-text: var(--text);
-    --wptheading-text: hsl(0, 0%, 30%);
-}
-@media (prefers-color-scheme: dark) {
-    :root {
-        --wpt-border: hsl(0, 0%, 30%);
-        --wpt-bg: var(--borderedblock-bg);
-        --wpt-text: var(--text);
-        --wptheading-text: hsl(0, 0%, 60%);
-    }
-}
-.wpt-tests-block {
-    list-style: none;
-    border-left: .5em solid var(--wpt-border);
-    background: var(--wpt-bg);
-    color: var(--wpt-text);
-    margin: 1em auto;
-    padding: .5em;
-}
-.wpt-tests-block summary {
-    color: var(--wptheading-text);
-    font-weight: normal;
-    text-transform: uppercase;
-}
-.wpt-tests-block summary::marker{
-    color: var(--wpt-border);
-}
-.wpt-tests-block summary:hover::marker{
-    color: var(--wpt-text);
-}
-/*
-   The only content  of a wpt test block in its closed state is the <summary>,
-   which contains the word TESTS,
-   and that is absolutely positioned.
-   In that closed state, wpt test blocks are styled
-   to have a top margin whose height is exactly equal
-   to the height of the absolutely positioned <summary>,
-   and no other background/padding/margin/border.
-   The wpt test block elements will therefore allow the maring
-   of the previous/next block elements
-   to collapse through them;
-   if this combined margin would be larger than its own top margin,
-   it stays as is,
-   and therefore the pre-existing vertical rhythm of the document is undisturbed.
-   If that combined margin would be smaller, it is grown to that size.
-   This means that the wpt test block ensures
-   that there's always enough vertical space to insert the summary,
-   without adding more than is needed.
-*/
-.wpt-tests-block:not([open]){
-    padding: 0;
-    border: none;
-    background: none;
-    font-size: 0.75em;
-    line-height: 1;
-    position: relative;
-    margin: 1em 0 0;
-}
-.wpt-tests-block:not([open]) summary {
-    position: absolute;
-    right: 0;
-    bottom: 0;
-}
-/*
-   It is possible that both the last child of a block element
-   and the block element itself
-   would be annotated with a <wpt> block each.
-   If the block element has a padding or a border,
-   that's fine, but otherwise
-   the bottom margin of the block and of its last child would collapse
-   and both <wpt> elements would overlap, being both placed there.
-   To avoid that, add 1px of padding to the <wpt> element annotating the last child
-   to prevent the bottom margin of the block and of its last child from collapsing
-   (and as much negative margin,
-   as wel only want to prevent margin collapsing,
-   but are not trying to actually take more space).
-*/
-.wpt-tests-block:not([open]):last-child {
-    padding-bottom: 1px;
-    margin-bottom: -1px;
-}
-/*
-   Exception to the previous rule:
-   don't do that in non-last list items,
-   because it's not necessary,
-   and would therefore consume more space than strictly needed.
-   Lists must have list items as children, not <wpt> elements,
-   so a <wpt> element cannot be a sibling of a list item,
-   and the collision that the previous rule avoids cannot happen.
-*/
-li:not(:last-child) > .wpt-tests-block:not([open]):last-child,
-dd:not(:last-child) > .wpt-tests-block:not([open]):last-child {
-    padding-bottom: 0;
-    margin-bottom: 0;
-}
-.wpt-tests-block:not([open]):not(:hover){
-    opacity: 0.5;
-}
-.wpt-tests-list {
-    list-style: none;
-    display: grid;
-    margin: 0;
-    padding: 0;
-    grid-template-columns: 1fr max-content auto auto;
-    grid-column-gap: .5em;
-}
-.wpt-tests-block hr:last-child {
-    display: none;
-}
-.wpt-test {
-    display: contents;
-}
-.wpt-test > a {
-    text-decoration: underline;
-    border: none;
-}
-.wpt-test > .wpt-name { grid-column: 1; }
-.wpt-test > .wpt-results { grid-column: 2; }
-.wpt-test > .wpt-live { grid-column: 3; }
-.wpt-test > .wpt-source { grid-column: 4; }
-
-.wpt-test > .wpt-results {
-    display: flex;
-    gap: .1em;
-}
-.wpt-test .wpt-result {
-    display: inline-block;
-    height: 1em;
-    width: 1em;
-    border-radius: 50%;
-    position: relative;
-}
-"""
-
-
-def getWptScript(path: str | None) -> str:
-    if path is None:
-        path = "/"
-    if not path.startswith("/"):
-        path = "/" + path
-    if not path.endswith("/"):
-        path = path + "/"
-    script = f'let wptPath = "{path}";\n'
-    with open(config.scriptPath("wpt", "wptScript.js"), "r", encoding="utf-8") as fh:
-        script += fh.read()
-    return script
+def guessTestSuiteUrl(paths: t.Iterable[str]) -> str:
+    # Do the same transforms as commonPathPrefixes, but then
+    # just count the number of paths under each,
+    # and return the one that wins.
+    splitPaths = [x.strip("/").split("/")[:-1] for x in paths]
+    prefixCounter: Counter[str] = Counter()
+    for pathSegs in splitPaths:
+        prefix = "/" + "/".join(pathSegs[0:2]) + "/"
+        prefixCounter[prefix] += 1
+    bestGuess = prefixCounter.most_common(1)[0][0]
+    return f"https://wpt.fyi/results{bestGuess}"
